#include "stdafx.h"
#include <commctrl.h>
#include <shlobj.h>			// SHMalloc...
#include <winbase.h>		// MoveFile: Rename file


#include "DlgID3Organize.h"
#include "resource.h"
#include "definition.h"
#include "Util.h"
#include "FileDir.h"
#include "TagInfo.h"
#include "Language.h"

// external declaration
extern INITSTRUCT gParam;
extern CLanguage    *lng;

/* ************************************************************************ */
BOOL CDlgID3Organize::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	switch (uMsg)
	{
	case WM_INITDIALOG:
		InitLanguage();
		LoadParam();
		InitFileList(FALSE); // Full name mode

		// Tooltips creation + install a hook procedure to monitor the message stream for mouse 
		// messages intended for the controls in the dialog box. 
		m_hTooltips = SetHookTooltipDialog(TRUE, m_hDlg);
		return FALSE;

	case WM_CLOSE:
		::DestroyWindow (m_hDlg);
		return 0;

	case WM_DESTROY:
		SaveParam();
		SetHookTooltipDialog(FALSE, NULL);
		return 0;

	case WM_NOTIFY:
		OnNotify(lParam);
		return TRUE;

	case WM_COMMAND: 
		switch (LOWORD(wParam))
		{ 
		case IDOK:
		case IDCANCEL:
			EndDialog(m_hDlg,0);
			return TRUE;

		case IDC_TAG_LIST_FILE:	    
			if (HIWORD(wParam) == LBN_SELCHANGE)
				SaveSelectedItem();
			return TRUE;

		case IDC_ORGANIZE_BTN_DEST:
			ChooseDirectory();
			return TRUE;

		case IDC_TAG_CHK_LOG:
			LOGEnable();
			return TRUE;
		case IDC_TAG_BTN_LOG:
			LOGChooseFile();
			return TRUE;
		case IDC_TAG_BTN_GO:
			BtnGO();
			return TRUE;

		}
	}
	return FALSE;
}



// **********************************************
void CDlgID3Organize::InitLanguage()
{
	LNGRESSTRUCT lrs[5] = 
	{
		{ IDC_DIRECTORYNAME, LNG_DLGID3TAG_DIRECTORY },
		{ IDC_DIRECTORY_TARGET, LNG_DLGID3TAG_TARGET },
		{ IDC_TAG_BTN_GO, LNG_BTN_TRANSFER },
		{ IDC_TAG_CHK_LOG, LNG_DLGID3TAG_TRACE_LOG },
		{ IDC_TAG_CHK_ONLYSELECTED, LNG_DLGID3TAG_ONLY_SELECTED_ITEM }
	};
	
	for (int i = 0 ; i<NUMELEMENTS(lrs); i++)
		DlgSetText(m_hDlg, lrs[i].idRes, GS(lrs[i].idLNG) );
}


/* ************************************************************** */
void CDlgID3Organize::LoadParam()
{
	const TCHAR *szID3v[] = {_T("ID3v1"), _T("ID3v2"), _T("ID3v2 / ID3v1")};
	TCHAR szTmp[80];
	
	HWND hwndCtrl = GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG);
	SetWindowLong(hwndCtrl, GWL_STYLE, GetWindowLong(hwndCtrl, GWL_STYLE) | SS_PATHELLIPSIS);

	hwndCtrl = GetDlgItem(m_hDlg, IDC_ORGANIZE_EDIT_DEST);
	SetWindowLong(hwndCtrl, GWL_STYLE, GetWindowLong(hwndCtrl, GWL_STYLE) | SS_PATHELLIPSIS);

	CheckDlgButton(m_hDlg, IDC_TAG_CHK_ONLYSELECTED, BST_CHECKED); 

	// Directory name from field of ID3Tag
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_RESETCONTENT, 0, 0);
	_stprintf(szTmp, _T("..\\%s"), GS(LNG_ARTIST));
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR)szTmp);	
	_stprintf(szTmp, _T("..\\%s"), GS(LNG_ALBUM));
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR)szTmp);	
	_stprintf(szTmp, _T("..\\%s"), GS(LNG_GENRE));
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR)szTmp);	
	_stprintf(szTmp, _T("..\\%s\\%s"), GS(LNG_ARTIST), GS(LNG_ALBUM));
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR)szTmp);	

	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_SETCURSEL, gParam.nOrganizeField, 0);
	
	// Show option information from ID3v1 or ID3v2
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_ID3,  CB_RESETCONTENT, 0, 0);
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_ID3,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR) szID3v[0]);	
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_ID3,  CB_ADDSTRING, 0, (LPARAM) (LPCTSTR) szID3v[1]);	
	SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_ID3,  CB_SETCURSEL, gParam.nOrganizeFromID3, 0);




}


/* ************************************************************** */
void CDlgID3Organize::SaveParam()
{
	gParam.nOrganizeField  = SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_FIELD,  CB_GETCURSEL, 0, 0);
	gParam.nOrganizeFromID3 = SendDlgItemMessage(m_hDlg, IDC_ORGANIZE_CBO_ID3,  CB_GETCURSEL, 0, 0);
}



// **********************************************
// OnNotify - provides the tooltip control with the appropriate text 
//     to display for a control window. This function is called by 
//     the dialog box procedure in response to a WM_NOTIFY message. 
// lParam - second message parameter of the WM_NOTIFY message. 
void CDlgID3Organize::OnNotify(LPARAM lParam) 
{ 
	LPTOOLTIPTEXT lpttt; 
	int idCtrl; 

	if ((((LPNMHDR) lParam)->code) == TTN_NEEDTEXT) 
	{ 
		idCtrl = GetDlgCtrlID((HWND) ((LPNMHDR) lParam)->idFrom); 
		lpttt = (LPTOOLTIPTEXT) lParam; 

		switch (idCtrl) 
		{ 
			
		case IDC_TAG_BTN_LOG:
			lpttt->lpszText = (TCHAR*)GS(LNG_MSG_CHOOSELOG); 
			break;
		case IDC_TAG_EDIT_LOG: 
			{
				int len = GetWindowTextLength(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG));
				m_strTooltipText.reserve(len+1);
				GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG),  (LPTSTR)m_strTooltipText.c_str(), len+1);

				lpttt->lpszText = (TCHAR*)m_strTooltipText.c_str(); 			
			}
			break; 
		
		} 
	} 
	return; 
} 

/* ************************************************************************ */
void CDlgID3Organize::BtnGO()
{
	HCURSOR hcurOld;
	int i, nCount;
	int *pArray = NULL;
	FILE *filelog = NULL;
	BOOL bOK, bSelected, bLog;
	const TCHAR *Msg_label[] = {GS(LNG_FAILED), GS(LNG_SUCCESS)};
	TCHAR szMsg[1024];
	TCHAR szPathDest[MAX_PATH];
	
	SaveParam();


	// Path target exist ?	
	szPathDest[0] = '\0';
	GetWindowText(GetDlgItem(m_hDlg, IDC_ORGANIZE_EDIT_DEST), (LPTSTR)szPathDest, sizeof(szPathDest));
	
	if (_tcslen(szPathDest) == 0)
	{	MessageBox(m_hDlg, GS(LNG_MSG_CHOOSEPATHDEST), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
		return;
	}

	if (GetDriveType((LPTSTR)szPathDest) == DRIVE_CDROM)
	{	MessageBox(m_hDlg, GS(LNG_FAILED_COPYMOVE_READONLY), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
		return;
	}

	bOK = PrepareItemList(bSelected, nCount, (int**)&pArray, 8);
	if(!bOK)
		return;


	// Trace LOG 
	bLog = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_LOG)== BST_CHECKED);
	
	if (bLog)
	{
		TCHAR szFileLog[MAX_PATH];
		szFileLog[0] = '\0';
		GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG), (LPTSTR)szFileLog, sizeof(szFileLog));
		
		if (_tcslen(szFileLog) == 0)
		{	MessageBox(m_hDlg, GS(LNG_MSG_CHOOSELOG), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
			return;
		}
		if ((filelog = _tfopen(szFileLog, _T("w+t"))) == NULL)
		{	MessageBox(m_hDlg, GS(LNG_MSG_ERRLOG), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
			return;
		}

		_ftprintf(filelog, GS(LNG_APPNAME) );
		_ftprintf(filelog, _T("\n----------------------\n") );
		_ftprintf(filelog, GS(LNG_HTML_DATE), GetFormattedDateTime());
		_ftprintf(filelog, _T("\n\n"));
		_ftprintf(filelog, GS(LNG_MSG_ORGANIZEFILE_LOG), nCount);
		_ftprintf(filelog, _T("\n\n"));
	}

	hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), FALSE);

	// Init Progress bar
	HWND hProgress = GetDlgItem(m_hDlg, IDC_PROGRESS1);
	HWND hStatus = GetDlgItem(m_hDlg, IDC_TAG_ORGANIZE_STATUS);
	ShowWindow(hProgress, SW_SHOW);
	ShowWindow(hStatus, SW_SHOW);
	SendMessage(hProgress, PBM_SETPOS, 1, 0);	// these two are required to display the empty bar
	SendMessage(hProgress, PBM_SETPOS, 0, 0);
	SendMessage(hProgress, PBM_SETRANGE, 0, MAKELPARAM(0, nCount)); 
	SendMessage(hProgress, PBM_SETSTEP, (WPARAM) 1, 0);

	// Scan file item	
	for (i = 0; i<nCount; i++)
	{
		TCHAR szOldFilename[MAX_PATH];

		CMP3Info *mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_GETITEMDATA, (WPARAM)(bSelected ? pArray[i] : i), 0);

		_tcscpy(szOldFilename, mp3i->GetCFilename());
		bOK =  OrganizeFile(mp3i, szPathDest);
		
		if (bLog)
		{
			_ftprintf(filelog, GS(LNG_LOG_ORGANIZE_FILE), i+1, Msg_label[bOK], szOldFilename, mp3i->GetCFilename());	
			_ftprintf(filelog, _T("\n") );
		}

		SendMessage(hProgress, PBM_STEPIT, 0, 0);
		_stprintf(szMsg, GS(LNG_STATUS_WORKING), i+1, nCount, mp3i->GetCFilename());
		SetWindowText(hStatus, szMsg);

	}
	
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), TRUE);
	SetCursor(hcurOld);

	if (bSelected)
		free(pArray);

	if (bLog)
		fclose(filelog);

	// Show resume action in MsgBox
	_stprintf(szMsg, GS(LNG_MSG_ORGANIZE_RESULT), nCount, szPathDest);
	MessageBox(m_hDlg, szMsg, GS(LNG_INFORMATION), MB_OK | MB_ICONINFORMATION);

	
	// Refresh list
	InitFileList(FALSE);		// Full name mode
	
	ShowWindow(hProgress, SW_HIDE);
	ShowWindow(hStatus, SW_HIDE);
}


/* ************************************************************************ */
BOOL CDlgID3Organize::OrganizeFile(CMP3Info *mp3i, TCHAR *szPathDest)
{
	int nLen, i;
	TCHAR szFile[MAX_PATH] ={0};
	TCHAR szFileDest[MAX_PATH]={0};
	TCHAR szDir[MAX_PATH]={0};
	TCHAR szFullPathDest[MAX_PATH]={0};
	
	GetField2Dir(mp3i, szDir);
	
	// No field for target directory !!!
	nLen = _tcslen(szDir);
	if (nLen < 1)
		return FALSE;


	// Construct target path
	_stprintf(szFullPathDest, _T("%s\\%s"), szPathDest, szDir);
	
	// no directory  --> exit
	if (*szFullPathDest == 0)	
		return FALSE; 
	
	CFileDir fd;

	// create directory if no exist
	if ( fd.MakeFullDir(szFullPathDest)== FALSE)
		return FALSE;

	// construct the future full file name (Target path + short file name)
	fd.SetFilename(mp3i->GetFilename());
	fd.ModifyPath(szFullPathDest); 
	tstring filename = fd.GetFullFilenameNoExt();
	
	i = 0;
	// rename x100 time the same filename if exist
	while (i<100)
	{
		if (i>0)
			_stprintf(szFileDest, _T("%s(%d).mp3"), (LPCSTR)filename.c_str(), i);
		else
			_stprintf(szFileDest, _T("%s.mp3"), (LPCSTR)filename.c_str());

		
		if (fd.FileExists(szFileDest) == FALSE)
		{		
			TCHAR szDrive[MAX_PATH];
			BOOL bRet=0;
			
			_tcscpy(szDrive, mp3i->GetCFilename());

			if (GetDriveType((LPTSTR)szDrive) == DRIVE_CDROM)
				bRet = CopyFile((LPTSTR)mp3i->GetCFilename(), (LPTSTR)szFileDest, TRUE);
			else	
				bRet = MoveFile((LPTSTR)mp3i->GetCFilename(), (LPTSTR)szFileDest);
			

			if (bRet)
			{
				// OK change in memory the new filename
				mp3i->SetFilename(szFileDest);
				return TRUE;
			}
			else
				return FALSE;
		}
		

		i++;

	}

	if (i > 100)
		return FALSE;

	return TRUE;
}


/* ****************************************************************** */
void CDlgID3Organize::GetField2Dir(CMP3Info *mp3i, TCHAR *szDir)
{
	if (mp3i->HaveTag() == FALSE)
		return;

	if (gParam.nOrganizeFromID3 == 0)
	{
		// From ID3v1 TAG
		struct ID3V1MP3Info *id3i;
		
		id3i = mp3i->GetID3v1();
		
		if(id3i == NULL)
			return;

		switch(gParam.nOrganizeField)
		{
			default:
			case 0: // Artist
				if (_tcslen((const TCHAR*)&id3i->Artist)==0)
					_tcscpy(szDir, GS(LNG_UNKOWNARTIST));
				else	
					_tcscpy(szDir, (const TCHAR*)&id3i->Artist);
				break;
			
			case 1: // Album
				if (_tcslen((const TCHAR*)&id3i->Album)==0)
					_tcscpy(szDir,  GS(LNG_UNKOWNALBUM));
				else	
					_tcscpy(szDir,  (const TCHAR*)&id3i->Album);
				break;
			
			case 2: // Genre
				if (id3i->Genre == 0)			
					_tcscpy(szDir, GS(LNG_UNKOWNGENRE));
				else
					_tcscpy(szDir, (const TCHAR*)GenreInt2Str(id3i->Genre)); 
				break;	

			case 3: // Artist\Album
				if (_tcslen((const TCHAR*)&id3i->Artist)==0)
					_tcscpy(szDir, GS(LNG_UNKOWNARTIST));
				else	
					_tcscpy(szDir, (const TCHAR*)&id3i->Artist);
				
				_tcscat(szDir,  _T("\\"));

				if (_tcslen((const TCHAR*)&id3i->Album)==0)
					_tcscat(szDir,  GS(LNG_UNKOWNALBUM));
				else	
					_tcscat(szDir,  (const TCHAR*)&id3i->Album);
				break;

		}

	}


	else
	{
		// From ID3v2 Tag
		tstring str = _T("");
		CTaginfo *tag = new CTaginfo(1); // from ID3v2

		switch(gParam.nOrganizeField)
		{
			default:
			case 0: // ..\Artist
				tag->GetFrameInfo(mp3i, ID3TAG_TPE1, str);
				if (_tcslen(str.c_str())==0)
					_tcscpy(szDir, GS(LNG_UNKOWNARTIST));
				else	
					_tcscpy(szDir, str.c_str());
				break;

			case 1: // ..\Album
				tag->GetFrameInfo(mp3i, ID3TAG_TALB, str);
				if (_tcslen(str.c_str())==0)
					_tcscpy(szDir, GS(LNG_UNKOWNALBUM));
				else	
					_tcscpy(szDir, str.c_str());
				break;

			case 2: // ..\Genre
				tag->GetFrameInfo(mp3i, ID3TAG_TCON, str);
				if (_tcslen(str.c_str())==0)
					_tcscpy(szDir, GS(LNG_UNKOWNGENRE));
				else	
					_tcscpy(szDir, str.c_str());
				break;

			case 3: // ..\Artist\Album
				tag->GetFrameInfo(mp3i, ID3TAG_TPE1, str);
				if (_tcslen(str.c_str())==0)
					_tcscpy(szDir, GS(LNG_UNKOWNARTIST));
				else	
					_tcscpy(szDir, str.c_str());

				_tcscat(szDir,  _T("\\"));

				tag->GetFrameInfo(mp3i, ID3TAG_TALB, str);
				if (_tcslen(str.c_str())==0)
					_tcscat(szDir, GS(LNG_UNKOWNALBUM));
				else	
					_tcscat(szDir, str.c_str());
				break;


		}

	}

	return;
}

/* ************************************************************************ */
void CDlgID3Organize::ChooseDirectory()
{
	static TCHAR szDir[MAX_PATH]= {0};
	
	CFileDir fd;

	if (fd.ChooseDirectoryBrowser(m_hDlg, szDir, NULL))
		SetWindowText(GetDlgItem(m_hDlg, IDC_ORGANIZE_EDIT_DEST), szDir);

}

