#include "stdafx.h"
#include <commctrl.h>
#include <winbase.h>		// MoveFile: Rename file


#include "DlgID3RenameFile.h"
#include "resource.h"
#include "definition.h"
#include "Util.h"
#include "Language.h"
#include "TagInfo.h"

// external declaration
extern INITSTRUCT gParam;
extern CLanguage    *lng;


/* ************************************************************************ */
BOOL CDlgID3RenameFileFromTag::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{

	case WM_INITDIALOG:
		InitLanguage();
		LoadParam();
		InitFileList();
		// Tooltips creation + install a hook procedure to monitor the message stream for mouse 
		// messages intended for the controls in the dialog box. 
		m_hTooltips = SetHookTooltipDialog(TRUE, m_hDlg);
		SendMessage(m_hTooltips, TTM_SETMAXTIPWIDTH, 0, (LPARAM)300);  // multi-line of 300 caract
		return FALSE;

	case WM_CLOSE:
		::DestroyWindow (m_hDlg);
		return 0;

	case WM_DESTROY:
		SaveParam(TRUE);
		SetHookTooltipDialog(FALSE, NULL);
		return 0;

	case WM_NOTIFY:
		OnNotify(lParam);
		return TRUE;

	
	case WM_DRAWITEM:
		OwnerDrawBtn((LPDRAWITEMSTRUCT)lParam);		
		return TRUE;
	
	case WM_CONTEXTMENU: 
		if(doContextMenuSelect(wParam, lParam)) 
			return FALSE; 
		return TRUE; 



	case WM_COMMAND: 
		switch (LOWORD(wParam))
		{ 
		case IDC_TAG_LIST_FILE: 	
			if (HIWORD(wParam) == LBN_SELCHANGE)
				SaveSelectedItem();
			return TRUE;


		case IDC_FRMT_ARTIST: 
		case IDC_FRMT_TITLE:  
		case IDC_FRMT_ALBUM:   
		case IDC_FRMT_GENRE:   
		case IDC_FRMT_TRACK:   
		case IDC_FRMT_COMMENT: 
		case IDC_FRMT_YEAR:    
			UpdateFormat(LOWORD(wParam));
			return TRUE;

		case IDOK:
		case IDCANCEL:
			EndDialog(m_hDlg,0);
			return TRUE;
		
		case IDC_TAG_OPTIONCAPITALIZE:
			MenuButton(IDC_TAG_OPTIONCAPITALIZE, IDR_MENU1, 5); // 5:Capitalize SubMenu 				
			return FALSE;

		case ID_CAPITALIZE_UNCHANGED:
		case ID_CAPITALIZE_ALLLOWERCASE:
		case ID_CAPITALIZE_ALLUPPERCASE:
		case ID_CAPITALIZE_FIRSTLETTEROFSECTIONUPPERCASE:
		case ID_CAPITALIZE_FIRSTLETTEROFEACHWORDUPPERCASE:
			gParam.nOptionCapitalize  = LOWORD(wParam) - ID_CAPITALIZE_UNCHANGED;
			return TRUE;

		case ID_CONVERT_NO:
		case ID_CONVERT_UNDERSCORE_TO_SPACE:
		case ID_CONVERT_SPACE_TO_UNDERSCORE:
			gParam.nOptionConvert  = LOWORD(wParam) - ID_CONVERT_NO;
			return TRUE;

		case IDC_TAG_CHK_LOG:
			LOGEnable();
			return TRUE;
		case IDC_TAG_BTN_LOG:
			LOGChooseFile();
			return TRUE;
		case IDC_TAG_BTN_GO:
			BtnGO();
			return TRUE;

		}
	}
	return FALSE;
}


// **********************************************
void CDlgID3RenameFileFromTag::InitLanguage()
{
	LNGRESSTRUCT lrs[5] = 
	{
		{ IDC_FILEFORMAT, LNG_DLGID3TAG_FILE_FORMAT } ,
		{ IDC_SEPARATOR, LNG_DLGID3TAG_SEPARATOR } ,
		{ IDC_TAG_BTN_GO, LNG_BTN_GO } ,
		{ IDC_TAG_CHK_LOG, LNG_DLGID3TAG_TRACE_LOG } ,
		{ IDC_TAG_CHK_ONLYSELECTED, LNG_DLGID3TAG_ONLY_SELECTED_ITEM }
	};                                                                        
                                                                              
	for (int i = 0 ; i<NUMELEMENTS(lrs); i++)                                                
		DlgSetText(m_hDlg, lrs[i].idRes, GS(lrs[i].idLNG) ); 


	string str;

	str = "%A = ";
	str += GS(LNG_ARTIST);
	DlgSetText(m_hDlg, IDC_FRMT_ARTIST, (TCHAR*)str.c_str() ); 

	str = "%T = ";
	str += GS(LNG_TITLE);
	DlgSetText(m_hDlg, IDC_FRMT_TITLE, (TCHAR*)str.c_str() ); 

	str = "%L = ";
	str += GS(LNG_ALBUM);
	DlgSetText(m_hDlg, IDC_FRMT_ALBUM, (TCHAR*)str.c_str() ); 

	str = "%Y = ";
	str += GS(LNG_YEAR);
	DlgSetText(m_hDlg, IDC_FRMT_YEAR, (TCHAR*)str.c_str() );  

	str = "%G = ";
	str += GS(LNG_GENRE);
	DlgSetText(m_hDlg, IDC_FRMT_GENRE, (TCHAR*)str.c_str() ); 

	str = "%C = ";
	str += GS(LNG_COMMENT);
	DlgSetText(m_hDlg, IDC_FRMT_COMMENT, (TCHAR*)str.c_str() ); 

	str = "%N = ";
	str += GS(LNG_TRACK);
	DlgSetText(m_hDlg, IDC_FRMT_TRACK, (TCHAR*)str.c_str() ); 

}


/* ************************************************************** */
void CDlgID3RenameFileFromTag::LoadParam()
{
	HWND hwndCtrl = GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG);
	SetWindowLong(hwndCtrl, GWL_STYLE, GetWindowLong(hwndCtrl, GWL_STYLE) | SS_PATHELLIPSIS);

	CheckDlgButton(m_hDlg, IDC_TAG_CHK_ONLYSELECTED, BST_CHECKED); 
	
	LoadMRU(IDC_TAG_CBO_FRMT);

	SendDlgItemMessage(m_hDlg, IDC_CBO_RENAMETAG_ID3VERSION,  CB_SETCURSEL, 0, 0);
	SendDlgItemMessage(m_hDlg, IDC_CBO_RENAMETAG_ID3VERSION,  CB_ADDSTRING, 0, (LPARAM)_T("ID3v1")); 
	SendDlgItemMessage(m_hDlg, IDC_CBO_RENAMETAG_ID3VERSION,  CB_ADDSTRING, 0, (LPARAM)_T("ID3v2")); 
	SendDlgItemMessage(m_hDlg, IDC_CBO_RENAMETAG_ID3VERSION,  CB_ADDSTRING, 0, (LPARAM)_T("ID3v2 / ID3v1")); 
	SendDlgItemMessage(m_hDlg, IDC_CBO_RENAMETAG_ID3VERSION,  CB_ADDSTRING, 0, (LPARAM)_T("ID3v1 / ID3v2")); 
	SendDlgItemMessage(m_hDlg, IDC_CBO_RENAMETAG_ID3VERSION,  CB_SETCURSEL, gParam.nRenameFileFromTagID3version, 0);

}


/* ************************************************************** */
void CDlgID3RenameFileFromTag::SaveParam(BOOL bCheckEntryMRU)
{
	gParam.nRenameFileFromTagID3version  = SendDlgItemMessage(m_hDlg, IDC_CBO_RENAMETAG_ID3VERSION,  CB_GETCURSEL, 0, 0);

	SaveMRU(IDC_TAG_CBO_FRMT, bCheckEntryMRU);
}

// **********************************************
// OnNotify - provides the tooltip control with the appropriate text 
//     to display for a control window. This function is called by 
//     the dialog box procedure in response to a WM_NOTIFY message. 
// lParam - second message parameter of the WM_NOTIFY message. 
void CDlgID3RenameFileFromTag::OnNotify(LPARAM lParam) 
{ 
	LPTOOLTIPTEXT lpttt; 
	int idCtrl; 

	if ((((LPNMHDR) lParam)->code) == TTN_NEEDTEXT) 
	{ 
		idCtrl = GetDlgCtrlID((HWND) ((LPNMHDR) lParam)->idFrom); 
		lpttt = (LPTOOLTIPTEXT) lParam; 

	
		// for DropDown listbox
		// trap edit box child window --> combobox dropdown list
		int idCtrlParent = GetDlgCtrlID( GetParent((HWND) ((LPNMHDR) lParam)->idFrom));
		if (idCtrlParent == IDC_TAG_CBO_FRMT)
			idCtrl = idCtrlParent;

		switch (idCtrl) 
		{ 
			
		case IDC_TAG_BTN_LOG:
			lpttt->lpszText = (TCHAR*)GS(LNG_MSG_CHOOSELOG); 
			break;
		case IDC_TAG_EDIT_LOG: 
			{
				int len = GetWindowTextLength(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG));
				m_strTooltipText.reserve(len+1);
				GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG),  (LPTSTR)m_strTooltipText.c_str(), len+1);

				lpttt->lpszText = (TCHAR*)m_strTooltipText.c_str(); 			
			}
			break; 
		case IDC_TAG_CBO_FRMT:
			{

				m_strTooltipText = "Preview: ";
				
				Preview(m_strTooltipText);
			
				lpttt->lpszText = (LPTSTR)m_strTooltipText.c_str();		
			}
			
			break;
		
		} 
	} 
	return; 
} 





/* ************************************************************************ */
void CDlgID3RenameFileFromTag::Preview(tstring &str)
{
	int i, nCount, len;
	int *pArray = NULL;
	BOOL bOK, bSelected;
	tstring strFormat;

#define LIMIT_MAX_PREVIEW 4
	int max;          // limit preview the x first items for rename

	SaveParam(FALSE);

	len = GetWindowTextLength(GetDlgItem(m_hDlg, IDC_TAG_CBO_FRMT));
	strFormat.reserve(len+1);
	GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_CBO_FRMT),  (LPTSTR)strFormat.c_str(), len+1);

	if (checkGoodFormat((TCHAR*)strFormat.c_str()) == FALSE)
	{
		str += GS(LNG_INVALID_FORMAT);
		return;
	}

	// Only selected items Options
	bSelected  = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ONLYSELECTED)== BST_CHECKED);

	if (!bSelected)
	{	// All Items 
		nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETCOUNT, 0, 0);
	}
	else
	{	// Only selected item 
		nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELCOUNT, 0, 0);
		if (nCount == LB_ERR || nCount == 0)
			return; 
		pArray = (int*)malloc(nCount*sizeof(int));
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELITEMS, (WPARAM)nCount, (LPARAM)pArray);
	}
	


	max =  (nCount > LIMIT_MAX_PREVIEW) ? LIMIT_MAX_PREVIEW : nCount;

	str += "\n";

	// Scan file item	
	for (i = 0; i<max; i++)
	{
		CMP3Info *mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_GETITEMDATA, (WPARAM)(bSelected ? pArray[i] : i), 0);
		
		str +="\n";
		bOK =  RenameFileFromTAG(mp3i, (TCHAR*)strFormat.c_str(), str);
		
	}
	
	if (nCount > max)
		str +="\n...";

	if (bSelected)
		free(pArray);

}


/* ************************************************************************ */
void CDlgID3RenameFileFromTag::BtnGO()
{
	HCURSOR hcurOld;
	int i, nCount;
	int *pArray = NULL;
	FILE *filelog = NULL;
	BOOL bOK, bSelected;
	const TCHAR *Msg_label[] = {GS(LNG_FAILED), GS(LNG_SUCCESS)};
	TCHAR szMsg[1024];
	int lenFormat;
	LPTSTR pszFormat;

	SaveParam(FALSE);

	lenFormat = SendDlgItemMessage(m_hDlg, IDC_TAG_CBO_FRMT,  CB_GETLBTEXTLEN, 0, 0);
	pszFormat  = (LPTSTR)calloc(sizeof(TCHAR), lenFormat+2);
	SendDlgItemMessage(m_hDlg, IDC_TAG_CBO_FRMT,  CB_GETLBTEXT, 0, (LPARAM) (LPCSTR)pszFormat);


	if (checkGoodFormat(pszFormat) == FALSE)
	{
		MessageBox(m_hDlg, GS(LNG_INVALID_FORMAT), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
		SetFocus(GetDlgItem(m_hDlg, IDC_TAG_CBO_FRMT));
		free(pszFormat);
		return;
	}


	bOK = PrepareItemList(bSelected, nCount, (int**)&pArray, 1);
	if(!bOK)
	{   free(pszFormat);
		return;
	}

	filelog = PrepareLogFile(&bOK);
	if(!bOK)
	{   free(pszFormat);
		return;
	}
	
	if (filelog)
	{	_ftprintf(filelog, GS(LNG_MSG_RENAMEFILE_LOG), nCount);
		_ftprintf(filelog, _T("\n\n"));
	}




	hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), FALSE);

	// Init Progress bar
	HWND hProgress = GetDlgItem(m_hDlg, IDC_PROGRESS1);
	HWND hStatus = GetDlgItem(m_hDlg, IDC_TAG_RENAMEFILETAG_STATUS);
	ShowWindow(hStatus, SW_SHOW);
	
	InitProgressBar(hProgress, nCount);

	// Scan file item	
	for (i = 0; i<nCount; i++)
	{
		CMP3Info *mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_GETITEMDATA, (WPARAM)(bSelected ? pArray[i] : i), 0);

		bOK =  ExecuteRenameFileFromTAG(mp3i, pszFormat);
		
		if (filelog)
		{	_ftprintf(filelog, GS(LNG_LOG_FILE), i+1, Msg_label[bOK], mp3i->GetCFilename());	
			_ftprintf(filelog, _T("\n") );
		}

		SendMessage(hProgress, PBM_STEPIT, 0, 0);
		_stprintf(szMsg, GS(LNG_STATUS_WORKING), i+1, nCount, mp3i->GetCFilename());
		SetWindowText(hStatus, szMsg);

	}
	
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), TRUE);
	SetCursor(hcurOld);

	if (bSelected)
		free(pArray);

	if (filelog)
		fclose(filelog);

	free(pszFormat);


	// Show resume action in MsgBox
	_stprintf(szMsg, GS(LNG_MSG_RENAMEFILE_RESULT), nCount);
	MessageBox(m_hDlg, szMsg, GS(LNG_INFORMATION), MB_OK | MB_ICONINFORMATION);

	
	// Refresh list
	InitFileList();
	
	ShowWindow(hProgress, SW_HIDE);
	ShowWindow(hStatus, SW_HIDE);
}


// *****************************************************************************************
BOOL CDlgID3RenameFileFromTag::ExecuteRenameFileFromTAG(CMP3Info *mp3i, LPTSTR pszFormat)
{
	TCHAR szOutputFile[1024];
	TCHAR *p, *pStart, *pEnd;
	size_t size;
	tstring output;
	tstring tmp;

	if (!RenameFileFromTAG(mp3i, pszFormat, output))
		return FALSE;

	szOutputFile[0] = '\0';

	pEnd = (TCHAR*)_tcsrchr(mp3i->GetCFilename(), '\\');
	pStart = (TCHAR*)mp3i->GetCFilename();
	size = pEnd - pStart;
	
	if (size)
	{
		// Copy path
		memcpy(szOutputFile, mp3i->GetCFilename(),  size);
		p= szOutputFile + size;
		*p = '\0';

		// copy new filename
		_tcscat(szOutputFile, _T("\\"));
		_tcscat(szOutputFile, output.c_str());
		_tcscat(szOutputFile, _T(".mp3"));
	}
	else
		_stprintf(szOutputFile, _T("%s.mp3"), output.c_str());
		

	// Rename MP3 file
	if (MoveFile((LPTSTR)mp3i->GetCFilename(), (LPTSTR)szOutputFile))
	{
		// OK change in memory the new filename
		mp3i->SetFilename(szOutputFile);
		return TRUE;
	}
	else
		return FALSE;


}





// *****************************************************************************************
BOOL CDlgID3RenameFileFromTag::RenameFileFromTAG(CMP3Info *mp3i, LPTSTR pszFormat, tstring  &output)
{
	TCHAR *p;
	tstring tmp;
	CTaginfo *tag;

	if (!mp3i) 
		return FALSE;

	// if file did not contain tag exit !!!
	if (mp3i->HaveTag() == FALSE)
		return FALSE;


	tag = (CTaginfo*)new CTaginfo(gParam.nRenameFileFromTagID3version); 

	
	p = pszFormat;


	while(1)
	{
		if (*p == 0) break; //exit loop  parse format

		if (*p != '%')
		{
			output += *p;
		}
		else
		{
			++p;
			
			tmp = "";

			switch(*p)
			{
				case '\0':
					goto endparse; //exit loop  parse format
				case 'A': 
					tag->GetFrameInfo(mp3i, ID3TAG_TPE1, tmp);
					output += tmp;			
					break;
				case 'T': 
					tag->GetFrameInfo(mp3i, ID3TAG_TIT2, tmp);
					output += tmp;			
					break;
				case 'L': 
					tag->GetFrameInfo(mp3i, ID3TAG_TALB, tmp);
					output += tmp;			
					break;
				case 'Y': 
					tag->GetFrameInfo(mp3i, ID3TAG_TYER, tmp);
					output += tmp;			
					break;
				case 'C':
					tag->GetFrameInfo(mp3i, ID3TAG_COMM, tmp);
					output += tmp;			
					break;
				case 'G':
					tag->GetFrameInfo(mp3i, ID3TAG_TCON, tmp);
					output += tmp;			
					break;
				case 'N':
					tag->GetFrameInfo(mp3i, ID3TAG_TRCK, tmp);
							
					if (tmp.size()<2) // "00" 2 digits
						output += "0";	

					output += tmp;	

					break;
			}
			


		}
		p++;
	}

endparse:


	free(tag);

	// Capitalize options
	CapitalizeString((TCHAR*)output.c_str());

	// Convert options Underscore <--> space
	ConvertString((TCHAR*)output.c_str());


	ValidateFilename((TCHAR*)output.c_str());

	return TRUE;
}



/* ************************************************************************ */
void CDlgID3RenameFileFromTag::ValidateFilename(TCHAR *filename)
{
	// Replace invalid character <>:"/\|? by space in the filename
	TCHAR *p = filename;
	int len = _tcslen(filename);

	while (p < filename+len)
	{
		switch(*p)
		{
			case '<':
			case '>':
			case ':':
			case '"':
			case '/':
			case '\\':
			case '|':
			case '?':
				*p = ' ';
				break;
			default:
				break;
		}

		p++;
	}

}


