#include "stdafx.h"
#include <commctrl.h>
#include <commdlg.h>
#include <malloc.h>


#include "DlgID3Template.h"
#include "resource.h"
#include "gen.h"
#include "definition.h"
#include "Util.h"
#include "FileDir.h"
#include "Language.h"

// external declaration
extern winampGeneralPurposePlugin plugin;
extern INITSTRUCT gParam;
extern CLanguage    *lng;


// Array OutputFormat 
const TCHAR *CDlgID3Template::m_listFrmt[6];


// Convert table Resource ID <--> Frame ID3v2
const int CDlgID3Template::m_Res2Frame[MAX_RES2FRAME][2] =
{
	{IDC_TAG_EDIT_TITLE2,	ID3TAG_TIT2},
	{IDC_TAG_EDIT_ARTIST2,	ID3TAG_TPE1},
	{IDC_TAG_EDIT_ALBUM2,	ID3TAG_TALB},
	{IDC_TAG_EDIT_YEAR2,	ID3TAG_TYER},
	{IDC_TAG_EDIT_GENRE2,	ID3TAG_TCON},
	{IDC_TAG_EDIT_COMMENT2,	ID3TAG_COMM},
	{IDC_TAG_EDIT_TRACK2,	ID3TAG_TRCK},
	{IDC_TAG_EDIT_RATE2,	ID3TAG_POPM}, // Popularitymeter
	{IDC_TAG_EDIT_BPM2,		ID3TAG_TBPM},
	{IDC_TAG_EDIT_SUBTITLE2,ID3TAG_TIT3},
	{IDC_TAG_EDIT_REMIXED2, ID3TAG_TPE4},
	{IDC_TAG_EDIT_COMPOSER2,ID3TAG_TCOM}

/*	
	
	{IDC_ORCHESTRA, ID3TAG_TPE2},
	{IDC_CONDUCTOR, ID3TAG_TPE3},
	{IDC_URL,		ID3TAG_WXXX},
	{IDC_OARTIST,	ID3TAG_TOPE},
	{IDC_ENCODER,	ID3TAG_TENC},
	{IDC_TCOPYRIGHT, ID3TAG_TCOP},
	{IDC_LYRIC_EDIT,ID3TAG_USLT}
*/
};

extern bool IsStringEmpty(const string& s);
extern bool IsStringEmpty(string *s);
extern bool IsStringEmpty(wstring *s);
extern bool IsStringEmpty(const wstring& s);
extern void TrimString(tstring& str);


/* ************************************************************************ */
CDlgID3Template::CDlgID3Template(CDlgPlaylist* pThis): CDlg()
{
	m_pThisPlaylist = pThis;
	m_hwndList = pThis->m_hwndList;

	m_listFrmt[0] = GS(LNG_ARTIST); 
	m_listFrmt[1] = GS(LNG_TITLE);
	m_listFrmt[2] = GS(LNG_ALBUM);
	m_listFrmt[3] = GS(LNG_YEAR);
	m_listFrmt[4] = GS(LNG_COMMENT);
	m_listFrmt[5] = GS(LNG_TRACK);

}	



/* ************************************************************************ */
void CDlgID3Template::InitFileList(BOOL bShortFileName)
{
	int i, nCount, idx;
	LV_ITEM lvi;
	
	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_RESETCONTENT, 0, 0);

	nCount = ListView_GetItemCount(m_hwndList);

	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_PARAM; 
	
	for (i = 0; i< nCount; i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(m_hwndList, &lvi);

		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;

		if (!mp3i) 
			continue;
	
		tstring filename;
		CFileDir fd(mp3i->GetFilename());

		if (bShortFileName)
			filename = fd.GetFilename();
		else
			filename = fd.GetFullFilename();
	  
		idx = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_ADDSTRING, 0, (LPARAM)(LPTSTR)filename.c_str());		
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_SETITEMDATA, idx, (LPARAM)mp3i);		

		for (int iSel=0; iSel<m_pThisPlaylist->m_nSelCount; iSel++)
		{
			if (m_pThisPlaylist->m_mp3iSelArray[iSel] == mp3i)
			{
				SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_SETSEL, TRUE, idx);
				break;
			}
		}	

	}	


}



/* ************************************************************************ */
int CDlgID3Template::GetFrameID(int resID)
{
	for(int i=0; i<MAX_RES2FRAME; i++)
		if (m_Res2Frame[i][0] == resID) 
			return(m_Res2Frame[i][1]);
	
	return(0); // not found
}

/* ************************************************************************ */
int CDlgID3Template::GetResID(int fid)
{
	for(int i=0; i<MAX_RES2FRAME; i++)
		if (m_Res2Frame[i][1] == fid) 
			return(m_Res2Frame[i][0]);
	
	return(0);	// not found
}


/* ************************************************************************ */
// Displays the info for the Info of the specific frame
void CDlgID3Template::SetTextMP3info(int fid, CMP3Info *mp3i)
{
	// only supported frames allowed
	if (!ID3Frames[fid].Supported) return;

	CMP3Frame *mp3f = mp3i->GetFrame(fid);
	
	if (ID3Frames[fid].Type != ID3ENC_STRING) 
		return;
	else
	{
		HWND hEdit = GetDlgItem(m_hDlg, GetResID(fid));

		tstring str(_T(""));
		
		if (mp3f)
			str = ((CMP3StringFrame *)mp3f)->GetString();	// handle only the first String

		if (hEdit)
			SetWindowText(hEdit, str.c_str());
	
	}
	
}

/* ************************************************************************ */
// Retrieve the info for a specific frame from Dialog
void CDlgID3Template::GetTextMP3info(int fid, CMP3Info *mp3i)
{
	// only supported frames allowed
	if (!ID3Frames[fid].Supported) return;

	CMP3Frame *mp3f = mp3i->GetFrame(fid);
	
	if (ID3Frames[fid].Type != ID3ENC_STRING)
		return;
	else
	{

		HWND hEdit = GetDlgItem(m_hDlg, GetResID(fid));

		if (hEdit)
		{
			int len = GetWindowTextLength(hEdit);
			LPTSTR str = (LPTSTR)alloca((len+1)*sizeof(TCHAR));
			if (str)
			{
				CMP3StringFrame *sf;
				GetWindowText(hEdit, str, len+1);
			
				sf = new CMP3StringFrame(fid, str);
			
				
				if (sf)
				{	// Update Frame MP3Info
					mp3i->UpdateFrame(sf);
					delete sf;
				}
			}
		}

		
	}	
	
}



/* ************************************************************************ */
void CDlgID3Template::SetRatingMP3info(CMP3Info *mp3i)
{
	int fid = ID3TAG_POPM;
	unsigned char rating = mp3i->GetRatingValue();

	SendMessage(GetDlgItem(m_hDlg, GetResID(fid)), CB_SETCURSEL,(WPARAM)rating, 0);
}



/* ************************************************************************ */
// Retrieve the info for a Rating Popularitymeter frame from Dialog
unsigned char CDlgID3Template::GetRatingMP3info(CMP3Info *mp3i)
{
	int fid = ID3TAG_POPM;
	CMP3Frame *mp3f = mp3i->GetFrame(fid);
	HWND hCbo = GetDlgItem(m_hDlg, GetResID(fid));

	if (hCbo)
	{
		int nSel = SendMessage(hCbo, CB_GETCURSEL, 0, 0);
		
		if (nSel == CB_ERR) 
			return 0;
	
		CMP3BinFrame *bf = new CMP3BinFrame(fid);
		
		if (bf)
		{	// Update Frame MP3Info
			bf->SetRating((unsigned char) nSel);

			mp3i->SetRatingValue(nSel);

			mp3i->UpdateFrame(bf);
			delete bf;

			return nSel;
		}

	}

	return 0;
}



/* ************************************************************************ */
void CDlgID3Template::LOGEnable()
{
	BOOL bEnable;

	bEnable = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_LOG)==BST_CHECKED);
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_EDIT_LOG), bEnable);
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_LOG), bEnable);
}

/* ************************************************************************ */
void CDlgID3Template::LOGChooseFile()
{
	OPENFILENAME ofn; 
	TCHAR szDirName[256]; 
	TCHAR *pszFile = NULL;
	int len;
	TCHAR szFileTitle[256]; 
    TCHAR szFilterTmp[MAX_PATH];
	
	_tcscpy(szFilterTmp,  GS(LNG_FILTEROPENDIALOG_LOG) );
	TCHAR *szFilter = szFilterTmp;
    
	SetFilter(szFilter);
	GetCurrentDirectory(sizeof(szDirName), (LPTSTR)szDirName);
	
	len = GetWindowTextLength(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG));

	if (len == 0)
		len = 1024;

	pszFile = (TCHAR*)malloc ((len+10)*sizeof(TCHAR));   // + 10 bytes to add extension if missing
	GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG),  pszFile, len+1);

	

	ofn.lStructSize = sizeof(OPENFILENAME); 
	ofn.hwndOwner = m_hDlg; 
	ofn.lpstrFilter = szFilter; 
	ofn.lpstrCustomFilter = (LPTSTR) NULL; 
	ofn.nMaxCustFilter = 0L; 
	ofn.nFilterIndex = 0L; 
	ofn.lpstrFile = pszFile; 
	ofn.nMaxFile = (len+1)*sizeof(TCHAR); //sizeof(szFile); 
	ofn.lpstrFileTitle = szFileTitle; 
	ofn.nMaxFileTitle = sizeof(szFileTitle); 
	ofn.lpstrInitialDir = szDirName; 
	ofn.lpstrTitle = GS(LNG_MSG_CHOOSELOG); 
	ofn.Flags = OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY; 
	ofn.nFileOffset = 0; 
	ofn.nFileExtension = 0; 
	ofn.lpstrDefExt = (LPTSTR)NULL; 

	if (GetSaveFileName(&ofn)) 
	{ 
		switch (ofn.nFilterIndex) 
		{
			case 1:	AddFileExtension(pszFile, _T(".txt")); 	break;
			case 2:	AddFileExtension(pszFile, _T(".log")); 	break;
		}
	
		SetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG), pszFile);

	} 

	free(pszFile);
}


/* ************************************************************************ */
BOOL CDlgID3Template::MsgConfirm(int nCount, int *pArray, int nPage)
{
	TCHAR szTemp[1024];
	TCHAR szMsg[8192];
	BOOL bLimitList = FALSE;
	int nIdMsg;
	
	switch(nPage)
	{	
		case 0:	nIdMsg = LNG_MSG_CONFIRM_MASS;					break;
		case 1:	nIdMsg = LNG_MSG_CONFIRM_RENAME;				break;
		case 2:	nIdMsg = LNG_MSG_CONFIRM_WRITETAGFROMFILENAME;	break;
		case 3:	nIdMsg = LNG_MSG_CONFIRM_COPYID3V1TOID3V2;		break;
		case 4:	nIdMsg = LNG_MSG_CONFIRM_COPYID3V2TOID3V1;		break;
		case 5:	nIdMsg = LNG_MSG_CONFIRM_REMOVEID3V1;			break;
		case 6:	nIdMsg = LNG_MSG_CONFIRM_REMOVEID3V2;			break;
		case 7:	nIdMsg = LNG_MSG_CONFIRM_REMOVEID3V1ID3V2;		break;
		case 8:	nIdMsg = LNG_MSG_CONFIRM_ORGANIZE;				break;
	}

	_stprintf(szMsg, GS(nIdMsg), nCount);
	_tcscat(szMsg, _T("\n\n") );

	BOOL bSelect = (pArray != NULL);

	// Afiche uniquement les 10 premiers fichiers
	if (nCount > 10)
	{	nCount = 10;
		bLimitList = TRUE;
	}

	for (int i = 0; i<nCount; i++)
	{
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_GETTEXT, (WPARAM)(bSelect ? pArray[i] : i), (LPARAM)(LPCTSTR)szTemp);
		_tcscat(szMsg, szTemp);
		_tcscat(szMsg, _T("\n"));
	}
	
	if (bLimitList)
	{	_tcscat(szMsg, _T("\n\n"));
		_tcscat(szMsg, GS(LNG_MOREFILES));

	}
	_tcscat(szMsg, _T("\n\n"));
	_tcscat(szMsg, GS(LNG_PROCEEDTHEM) );

	if (MessageBox(m_hDlg, szMsg, GS(LNG_QUESTION), MB_YESNO | MB_ICONQUESTION | MB_DEFBUTTON2) 
		== IDYES)
		return(TRUE);
	else
		return(FALSE);
	

}


/* ************************************************************************ */
BOOL  CDlgID3Template::PrepareItemList(BOOL& bSelected, int& nCount, int **pArray, int nAction)
{

	bSelected = (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_ONLYSELECTED)== BST_CHECKED);

	if (!bSelected)
	{	// All Items 
		nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETCOUNT, 0, 0);
	}
	else
	{	// Only selected item 
		nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELCOUNT, 0, 0);
		if (nCount == LB_ERR || nCount == 0)
		{	MessageBox(m_hDlg, GS(LNG_MSG_SELECT_FILE), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
			return FALSE; 
		}
	
		*pArray = (int*)malloc(nCount*sizeof(int));
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELITEMS, (WPARAM)(nCount), (LPARAM)(*pArray));
	}

	
	// Msgbox confirm before execute ?
	if (MsgConfirm(nCount, *pArray, nAction) == FALSE)
	{
		if (bSelected == TRUE)
			free(*pArray);
		return FALSE;
	}

	return TRUE;
}


/* ************************************************************************ */
FILE* CDlgID3Template::PrepareLogFile(BOOL *bOK)
{
	FILE *filelog  = NULL;

	if (IsDlgButtonChecked(m_hDlg, IDC_TAG_CHK_LOG)== BST_CHECKED)
	{
		TCHAR szFileLog[MAX_PATH];
		szFileLog[0] = '\0';
		GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG), (LPTSTR)szFileLog, sizeof(szFileLog));
		
		if (_tcslen(szFileLog) == 0)
		{	MessageBox(m_hDlg, GS(LNG_MSG_CHOOSELOG), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
			*bOK = FALSE;
			return NULL;
		}
		if ((filelog = _tfopen(szFileLog, _T("w+t"))) == NULL)
		{	MessageBox(m_hDlg, GS(LNG_MSG_ERRLOG), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
			*bOK = FALSE;
			return NULL;
		}
	
		_ftprintf(filelog, GS(LNG_APPNAME) );
		_ftprintf(filelog, _T("\n----------------------\n") );
		_ftprintf(filelog, GS(LNG_HTML_DATE), GetFormattedDateTime());
		_ftprintf(filelog, _T("\n\n"));
	}

	*bOK = TRUE;
	return filelog;
}


/* ************************************************************************ */
void CDlgID3Template::InitProgressBar(HWND hProgress, int nCount)
{
	ShowWindow(hProgress, SW_SHOW);
	SendMessage(hProgress, PBM_SETPOS, 1, 0);	// these two are required to display the empty bar
	SendMessage(hProgress, PBM_SETPOS, 0, 0);
	SendMessage(hProgress, PBM_SETRANGE, 0, MAKELPARAM(0, nCount)); 
	SendMessage(hProgress, PBM_SETSTEP, (WPARAM) 1, 0);
}
/* ************************************************************************ */
void CDlgID3Template::SaveSelectedItem()
{
	int *pArray = NULL;
	int nSelCount, i;
	CMP3Info *mp3i = NULL;

	
	nSelCount = m_pThisPlaylist->m_nSelCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELCOUNT, 0, 0);

	pArray = (int*)malloc(nSelCount*sizeof(int));
	if (pArray == NULL) return;


	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSELITEMS, (WPARAM)nSelCount, (LPARAM)pArray);


	// re-construct selection array
	m_pThisPlaylist->m_mp3iSelArray.clear();

	for (i = 0; i < nSelCount; i++)
	{
		mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_GETITEMDATA, (WPARAM)pArray[i], 0);

		if (mp3i != NULL)
			m_pThisPlaylist->m_mp3iSelArray.push_back(mp3i);
	}

	if (pArray != NULL)
		free(pArray);
}


/* ************************************************************************ */
void CDlgID3Template::MenuSelectedToggle(void)
{
	int nCount = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETCOUNT, 0, 0);
	BOOL bSelect;

	for (int i=0; i<nCount;i++)
	{
		bSelect = SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETSEL, i, 0);
		SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_SETSEL, (WPARAM)!bSelect, i);
	}
	SaveSelectedItem();
}

/* ************************************************************************ */
void CDlgID3Template::MenuSelectedAll(BOOL bSelect)
{
	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_SETSEL, (WPARAM)bSelect, -1);
	SaveSelectedItem();
}


/* ************************************************************************ */
void CDlgID3Template::OwnerDrawBtn(LPDRAWITEMSTRUCT lpdis)
{
	HDC hDC;
    RECT rect;

    hDC = lpdis->hDC;
    CopyRect (&rect, &(lpdis->rcItem));    

	if (lpdis->CtlID == IDC_TAG_OPTIONCAPITALIZE)
	{
		UINT uState = DFCS_SCROLLRIGHT;
		
		switch (lpdis->itemAction) 
		{	
			case ODA_DRAWENTIRE:  
			case ODA_FOCUS:
				break;

			case ODA_SELECT:
				if (lpdis->itemState & ODS_SELECTED)
					uState = DFCS_SCROLLRIGHT | DFCS_PUSHED;
				break;

			default:
				return;

		}

		DrawFrameControl(hDC, &rect, DFC_SCROLL, uState);
	}

}

/* ************************************************************ */
void CDlgID3Template::MenuButton(int nIDButton, int nIDMenu, int nSubMenu) 
{ 
	HMENU hMenuLoad, hMenu; 
	HWND hwndButton;
	RECT rect;
	POINT pt;

	hMenuLoad = LoadMenu(plugin.hDllInstance, MAKEINTRESOURCE(nIDMenu)); 
	
	hMenu = GetSubMenu(hMenuLoad, nSubMenu); 
			
	// Initialise Menu Option 'Capitalize/Convert'
	if (nIDButton == IDC_TAG_OPTIONCAPITALIZE)	
	{
		MenuSetText( hMenu, 0, GS(LNG_MENU_CAPITALIZE_UNCHANGED));                     
		MenuSetText( hMenu, 1, GS(LNG_MENU_CAPITALIZE_ALLLOWERCASE));                  
		MenuSetText( hMenu, 2, GS(LNG_MENU_CAPITALIZE_ALLUPPERCASE));                  
		MenuSetText( hMenu, 3, GS(LNG_MENU_CAPITALIZE_FIRSTLETTEROFSECTIONUPPERCASE)); 
		MenuSetText( hMenu, 4, GS(LNG_MENU_CAPITALIZE_FIRSTLETTEROFEACHWORDUPPERCASE));
		MenuSetText( hMenu, 6, GS(LNG_MENU_CONVERT_NO));
		MenuSetText( hMenu, 7, GS(LNG_MENU_CONVERT_UNDERSCORE_TO_SPACE));
		MenuSetText( hMenu, 8, GS(LNG_MENU_CONVERT_SPACE_TO_UNDERSCORE));

		CheckMenuRadioItem((HMENU) hMenu, 
		   ID_CAPITALIZE_UNCHANGED, 
		   ID_CAPITALIZE_FIRSTLETTEROFEACHWORDUPPERCASE,
		   ID_CAPITALIZE_UNCHANGED + gParam.nOptionCapitalize , MF_BYCOMMAND);
	
		CheckMenuRadioItem((HMENU) hMenu, 
		   ID_CONVERT_NO, 
		   ID_CONVERT_SPACE_TO_UNDERSCORE,
		   ID_CONVERT_NO + gParam.nOptionConvert , 
		   MF_BYCOMMAND);
	}


	if(nIDMenu==IDR_MENUSWAP)// Swap Field (EditTag)
	{
		switch(nSubMenu)
		{
			case 0:// IDC_TAG_EDIT_SWAPFIELD_ID3V1_TITLE
			case 3:// IDC_TAG_EDIT_SWAPFIELD_ID3V2_TITLE
				MenuSetText(hMenu, 0, GS(LNG_MENU_SWAPID3_TITLE_ARTIST));                 
				MenuSetText(hMenu, 1, GS(LNG_MENU_SWAPID3_TITLE_ALBUM));                 
				break;
			case 1:// IDC_TAG_EDIT_SWAPFIELD_ID3V1_ARTIST
			case 4:// IDC_TAG_EDIT_SWAPFIELD_ID3V2_ARTIST
				MenuSetText(hMenu, 0, GS(LNG_MENU_SWAPID3_ARTIST_TITLE));                 
				MenuSetText(hMenu, 1, GS(LNG_MENU_SWAPID3_ARTIST_ALBUM));                 
				break;
			case 2:// IDC_TAG_EDIT_SWAPFIELD_ID3V1_ALBUM
			case 5:// IDC_TAG_EDIT_SWAPFIELD_ID3V2_ALBUM
				MenuSetText(hMenu, 0, GS(LNG_MENU_SWAPID3_ALBUM_ARTIST));                 
				MenuSetText(hMenu, 1, GS(LNG_MENU_SWAPID3_ALBUM_TITLE));                 
				break;

	
		}

		MenuSetText( hMenu, 3, GS(LNG_MENU_CAPITALIZE_ALLLOWERCASE));                  
		MenuSetText( hMenu, 4, GS(LNG_MENU_CAPITALIZE_ALLUPPERCASE));                  
		MenuSetText( hMenu, 5, GS(LNG_MENU_CAPITALIZE_FIRSTLETTEROFSECTIONUPPERCASE)); 
		MenuSetText( hMenu, 6, GS(LNG_MENU_CAPITALIZE_FIRSTLETTEROFEACHWORDUPPERCASE));


	}

	
	hwndButton = GetDlgItem(m_hDlg, nIDButton); 
	GetClientRect(hwndButton, &rect);
	pt.x  = rect.right;
	pt.y  = rect.top;
	ClientToScreen(hwndButton, &pt);

	TrackPopupMenu(hMenu, TPM_LEFTALIGN | TPM_RIGHTBUTTON, 
		pt.x, pt.y, 0, m_hDlg, NULL); 

	DestroyMenu(hMenuLoad); 
} 




/*
	case WM_CONTEXTMENU: 
		if(doContextMenuSelect(wParam, lParam)) 
			return FALSE; 
		break; 

*/
/* ****************************************************************** */
BOOL CDlgID3Template::doContextMenuSelect(WPARAM wParam, LPARAM lParam) 
{ 
	HWND  hwndList = (HWND)wParam; 
	HMENU hMenuLoad, hMenu; 

	if(hwndList != GetDlgItem(m_hDlg, IDC_TAG_LIST_FILE)) 
		return FALSE; 

	hMenuLoad = LoadMenu(plugin.hDllInstance, MAKEINTRESOURCE(IDR_MENU1)); 
	
	// subMenu 'Select'
	hMenu = GetSubMenu(hMenuLoad, 2); 

	MenuSetText(hMenu,0, GS(LNG_MENU_PLAYLIST_SELECTALL    ));                  
	MenuSetText(hMenu,1, GS(LNG_MENU_PLAYLIST_SELECTNONE   ));                  
	MenuSetText(hMenu,2, GS(LNG_MENU_PLAYLIST_SELECTTOGGLE ));                  

	TrackPopupMenu(hMenu, 
				  TPM_LEFTALIGN | TPM_RIGHTBUTTON, 
				  LOWORD(lParam), 
				  HIWORD(lParam), 
				  0, 
				  m_hDlg, 
				  NULL); 

	DestroyMenu(hMenuLoad); 

	return TRUE; 
}

/* ************************************************************ */
void CDlgID3Template::CapitalizeString(TCHAR* szTemp)
{


	switch(gParam.nOptionCapitalize)
	{
		case 0:	// Unchanged
			break;
		
		case 1:	// all lowercase
			CharLower(szTemp);
			break;
		
		case 2:	// all Uppercase
			CharUpper(szTemp);
			break;
		
		case 3: // Capitilize only 1st letter 
			{
				CharLower(szTemp);

				TCHAR *p = &szTemp[0];
				*p = toupper(*p);
			}
			break;
		case 4: // Capitilize all words
			{
				CharLower(szTemp);

				int nLen = _tcslen(szTemp);

				TCHAR *p, *q;

				for (int i = 0; i < nLen-1; i++)
				{
					p = &szTemp[i];
					q = &szTemp[i+1];

					if (i ==0)
						*p = toupper(*p);

					if (*p ==  ' ' && *q  != ' ')
						*q = toupper(*q);


				}

			}
			break;

	}

}


/* ************************************************************ */
void CDlgID3Template::ConvertString(TCHAR* szTemp)
{


	switch(gParam.nOptionConvert)
	{
		case 0:	// Unchanged
			break;
		
		case 1:	// Convert Underscore to space
			{
				int nLen = _tcslen(szTemp);
				TCHAR *p;
				for (int i = 0; i < nLen; i++)
				{
					p = &szTemp[i];
					if (*p == '_')	*p = ' ';
				}

			}
			break;

		case 2:	// Convert Space to Underscore 
			{
				int nLen = _tcslen(szTemp);
				TCHAR *p;
				for (int i = 0; i < nLen; i++)
				{
					p = &szTemp[i];
					if (*p == ' ')	*p = '_';
				}

			}
			break;

	}

}





/* ************************************************************************ */
void CDlgID3Template::LoadMRU(int nIDCtrl)
{
	SendDlgItemMessage(m_hDlg, nIDCtrl, EM_SETLIMITTEXT , 50L, 0L);
	SendDlgItemMessage(m_hDlg, nIDCtrl, CB_RESETCONTENT, 0, 0);
	for (int i=0; i<MRU_DEPTH; i++)
	{	if (gParam.mruFormat[i] != NULL && strlen(gParam.mruFormat[i]) > 0)
			SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_ADDSTRING, 0, (LPARAM)gParam.mruFormat[i]); 
	}
	SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_SETCURSEL, 0, 0);

}

/* ************************************************************************ */
void CDlgID3Template::SaveMRU(int nIDCtrl, BOOL bCheckEntryMRU)
{
	int index, i, max, len;
	
	// MRU Most Recent Use combolistbox.
	index = SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_GETCURSEL, 0, 0);
	
	if (index  == CB_ERR)
	{
		// check inf the new entry is correct and not exist yet before store item into list
		len = GetWindowTextLength(GetDlgItem(m_hDlg, nIDCtrl));
		LPTSTR pszEntry = (LPTSTR)malloc((len+2)*sizeof(TCHAR));
		GetWindowText(GetDlgItem(m_hDlg, nIDCtrl), pszEntry, len+1);
		
		BOOL bExist = FALSE;
		max = 	SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_GETCOUNT, 0, 0);
		for (i=0; i<max; i++)
		{
			int lenFormat = SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_GETLBTEXTLEN, i, 0);
			LPTSTR pszFormat  = (LPTSTR)calloc(sizeof(TCHAR), lenFormat+2);
			SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_GETLBTEXT, i, (LPARAM) (LPCSTR)pszFormat);

			if (strcmp(pszFormat, pszEntry) == FALSE)
			{	free(pszFormat);
				bExist = TRUE;
				break;
			}
			
			free(pszFormat);
		}

		if (bExist == FALSE)
		{
			BOOL bAddEntry = FALSE;

			if (bCheckEntryMRU == TRUE)
			{
				if (checkGoodFormat(pszEntry) == TRUE)
					bAddEntry = TRUE;
			}
			else
			{
				bAddEntry = TRUE;
			}

			if (bAddEntry == TRUE)
			{
				max = 	SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_GETCOUNT, 0, 0);

				//Rollup delete last item of the list
				if (max > MRU_DEPTH)
					SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_DELETESTRING , MRU_DEPTH-1, 0);
			
				SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_INSERTSTRING, 0, (LPARAM)pszEntry);
		
			}
		}
		free(pszEntry);
	}


	if (bCheckEntryMRU == FALSE)
		return;


	//delete bad item format
	max = 	SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_GETCOUNT, 0, 0);
	for (i=max; i>=0; i--)
	{
		int lenFormat = SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_GETLBTEXTLEN, i, 0);
		LPTSTR pszFormat  = (LPTSTR)calloc(sizeof(TCHAR), lenFormat+2);
		SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_GETLBTEXT, i, (LPARAM) (LPCSTR)pszFormat);

		if (checkGoodFormat(pszFormat) == FALSE)
			max = SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_DELETESTRING , i, (LPARAM) (LPCSTR)pszFormat);
		
		free(pszFormat);
	}


	// save mru array
	memset(gParam.mruFormat, 0, sizeof(TCHAR)*MRU_DEPTH*(MRU_MAXLENGTH+1) );
	
	max = 	SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_GETCOUNT, 0, 0);

	if (max > MRU_DEPTH) max = MRU_DEPTH;


	index = SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_GETCURSEL, 0, 0);
	
	if (index  == CB_ERR)
	{	for (i=0; i<max; i++)
			SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_GETLBTEXT, i, (LPARAM) (LPCSTR)gParam.mruFormat[i]);
	}
	else
	{
		// save the select item first
		int j=0;
		SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_GETLBTEXT, index, (LPARAM) (LPCSTR)gParam.mruFormat[j++]);	
		for (i=0; i<max; i++)
		{
			if (i != index)
				SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_GETLBTEXT, i, (LPARAM) (LPCSTR)gParam.mruFormat[j++]);
		}
	}

	SendDlgItemMessage(m_hDlg, nIDCtrl,  CB_SETCURSEL, 0, 0);
}




/* ************************************************************************ */
BOOL CDlgID3Template::checkGoodFormat(LPTSTR pszFormat)
{
	TCHAR *p = pszFormat;
	int countfield = 0;
	BOOL bOK = FALSE;
	tstring str;

	if (pszFormat==NULL)
		return FALSE;
	
	str = pszFormat;
	TrimString(str);
	
	if (_tcslen(str.c_str()) == 0)
		return FALSE;

	while(1)
	{
		if (*p == 0) break;

		if (*p == '%')
		{
			++p;
			switch(*p)
			{
				case '\0':
					goto endcheckparse;
				case 'A': // artist
				case 'T': // title
				case 'L': // album
				case 'Y': // year
				case 'C': // comment
				case 'N': // track#
				case 'G': // genre
					countfield++;
					break;
				default:
					break;
			}
		}
		p++;

	}
endcheckparse:

	return (countfield>0);
}


/* ************************************************************************ */
void CDlgID3Template::UpdateFormat(int nIDEntry) 
{ 
	// Retreive the handle child 'Edit' from Combox
	POINT pt;
	RECT rectCbo;
	GetClientRect(GetDlgItem(m_hDlg, IDC_TAG_CBO_FRMT), &rectCbo);
	pt.x = (rectCbo.left + rectCbo.right)/2;
	pt.y = (rectCbo.top  + rectCbo.bottom)/2;
	HWND hEditCombolist = ChildWindowFromPoint( GetDlgItem(m_hDlg, IDC_TAG_CBO_FRMT), pt);


	// Get de 2 first charact : example %A
	TCHAR szEntry[3];
	GetWindowText(GetDlgItem(m_hDlg, nIDEntry), (LPTSTR)szEntry, 2+1);
	szEntry[2] = '\0';

	SendMessage(hEditCombolist, EM_REPLACESEL, TRUE, (LPARAM)(LPCTSTR)szEntry);

	return; 
} 
