#include "stdafx.h"

#include <commctrl.h>


#include "DlgID3WriteTagFromFile.h"
#include "resource.h"
#include "definition.h"
#include "Util.h"
#include "Language.h"

// external declaration
extern INITSTRUCT gParam;
extern CLanguage    *lng;


// Use to WriteTAGFromFile
extern bool IsStringEmpty(const string& s);
extern bool IsStringEmpty(string *s);
extern bool IsStringEmpty(wstring *s);
extern bool IsStringEmpty(const wstring& s);
extern void TrimString(tstring& str);



/* ************************************************************************ */
BOOL CDlgID3WriteTagFromFile::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		InitLanguage();
		LoadParam();
		InitFileList();
		// Tooltips creation + install a hook procedure to monitor the message stream for mouse 
		// messages intended for the controls in the dialog box. 
		m_hTooltips = SetHookTooltipDialog(TRUE, m_hDlg);
	
		SendMessage(m_hTooltips, TTM_SETMAXTIPWIDTH, 0, (LPARAM)300);


		return FALSE;

	case WM_CLOSE:
		::DestroyWindow (m_hDlg);
		return 0;

	case WM_DESTROY:
		SaveParam(TRUE);
		SetHookTooltipDialog(FALSE, NULL);
		return 0;

	case WM_NOTIFY:
		OnNotify(lParam);
		return TRUE;


	case WM_DRAWITEM:
		OwnerDrawBtn((LPDRAWITEMSTRUCT)lParam);		
		return TRUE;
	
	case WM_CONTEXTMENU: 
		if(doContextMenuSelect(wParam, lParam)) 
			return FALSE; 
		return TRUE; 
	
	case WM_COMMAND: 
		switch (LOWORD(wParam))
		{ 
		case IDC_TAG_LIST_FILE: 	
			if (HIWORD(wParam) == LBN_SELCHANGE)
				SaveSelectedItem();
			return TRUE;

		case IDC_FRMT_ARTIST: 
		case IDC_FRMT_TITLE:  
		case IDC_FRMT_ALBUM:   
		case IDC_FRMT_GENRE:   
		case IDC_FRMT_TRACK:   
		case IDC_FRMT_COMMENT: 
		case IDC_FRMT_YEAR:    
			UpdateFormat(LOWORD(wParam));
			return TRUE;
		
		case IDOK:
		case IDCANCEL:
			EndDialog(m_hDlg,0);
			return TRUE;

		case IDC_TAG_OPTIONCAPITALIZE:
			MenuButton(IDC_TAG_OPTIONCAPITALIZE, IDR_MENU1,5); // 5:Capitalize SubMenu
			return FALSE;

		case ID_CAPITALIZE_UNCHANGED:
		case ID_CAPITALIZE_ALLLOWERCASE:
		case ID_CAPITALIZE_ALLUPPERCASE:
		case ID_CAPITALIZE_FIRSTLETTEROFSECTIONUPPERCASE:
		case ID_CAPITALIZE_FIRSTLETTEROFEACHWORDUPPERCASE:
			gParam.nOptionCapitalize  = LOWORD(wParam) - ID_CAPITALIZE_UNCHANGED;
			return TRUE;

		case ID_CONVERT_NO:
		case ID_CONVERT_UNDERSCORE_TO_SPACE:
		case ID_CONVERT_SPACE_TO_UNDERSCORE:
			gParam.nOptionConvert  = LOWORD(wParam) - ID_CONVERT_NO;
			return TRUE;

		case IDC_TAG_CHK_LOG:
			LOGEnable();
			return TRUE;
		case IDC_TAG_BTN_LOG:
			LOGChooseFile();
			return TRUE;
		case IDC_TAG_BTN_GO:
			BtnGO();
			return TRUE;

		}
	}
	return FALSE;
}

// **********************************************
void CDlgID3WriteTagFromFile::InitLanguage()
{
	LNGRESSTRUCT lrs[4] = 
	{
		{ IDC_FILEFORMAT, LNG_DLGID3TAG_FILE_FORMAT },
		{ IDC_TAG_BTN_GO, LNG_BTN_GO },
		{ IDC_TAG_CHK_LOG, LNG_DLGID3TAG_TRACE_LOG },
		{ IDC_TAG_CHK_ONLYSELECTED, LNG_DLGID3TAG_ONLY_SELECTED_ITEM }
	};                                                                        
                                                                              
	for (int i = 0 ; i<NUMELEMENTS(lrs); i++)                                                
		DlgSetText(m_hDlg, lrs[i].idRes, GS(lrs[i].idLNG) );          

	string str;

	str = "%A = ";
	str += GS(LNG_ARTIST);
	DlgSetText(m_hDlg, IDC_FRMT_ARTIST, (TCHAR*)str.c_str() ); 

	str = "%T = ";
	str += GS(LNG_TITLE);
	DlgSetText(m_hDlg, IDC_FRMT_TITLE, (TCHAR*)str.c_str() ); 

	str = "%L = ";
	str += GS(LNG_ALBUM);
	DlgSetText(m_hDlg, IDC_FRMT_ALBUM, (TCHAR*)str.c_str() ); 

	str = "%Y = ";
	str += GS(LNG_YEAR);
	DlgSetText(m_hDlg, IDC_FRMT_YEAR, (TCHAR*)str.c_str() );  

	str = "%G = ";
	str += GS(LNG_GENRE);
	DlgSetText(m_hDlg, IDC_FRMT_GENRE, (TCHAR*)str.c_str() ); 

	str = "%C = ";
	str += GS(LNG_COMMENT);
	DlgSetText(m_hDlg, IDC_FRMT_COMMENT, (TCHAR*)str.c_str() ); 

	str = "%N = ";
	str += GS(LNG_TRACK);
	DlgSetText(m_hDlg, IDC_FRMT_TRACK, (TCHAR*)str.c_str() ); 

}



/* ************************************************************** */
void CDlgID3WriteTagFromFile::LoadParam()
{
	HWND hwndCtrl = GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG);
	SetWindowLong(hwndCtrl, GWL_STYLE, GetWindowLong(hwndCtrl, GWL_STYLE) | SS_PATHELLIPSIS);
	
	CheckDlgButton(m_hDlg, IDC_TAG_CHK_ONLYSELECTED, BST_CHECKED); 

	LoadMRU(IDC_TAG_CBO_FRMT);

	SendDlgItemMessage(m_hDlg, IDC_CBO_WRITETAG_ID3VERSION,  CB_SETCURSEL, 0, 0);
	SendDlgItemMessage(m_hDlg, IDC_CBO_WRITETAG_ID3VERSION,  CB_ADDSTRING, 0, (LPARAM)_T("ID3v1")); 
	SendDlgItemMessage(m_hDlg, IDC_CBO_WRITETAG_ID3VERSION,  CB_ADDSTRING, 0, (LPARAM)_T("ID3v2")); 
	SendDlgItemMessage(m_hDlg, IDC_CBO_WRITETAG_ID3VERSION,  CB_ADDSTRING, 0, (LPARAM)_T("ID3v1 + ID3v2")); 
	SendDlgItemMessage(m_hDlg, IDC_CBO_WRITETAG_ID3VERSION,  CB_SETCURSEL, gParam.nWriteTagfromFileID3version, 0);


}


/* ************************************************************** */
void CDlgID3WriteTagFromFile::SaveParam(BOOL bCheckEntryMRU)
{
	gParam.nWriteTagfromFileID3version  = SendDlgItemMessage(m_hDlg, IDC_CBO_WRITETAG_ID3VERSION,  CB_GETCURSEL, 0, 0);

	SaveMRU(IDC_TAG_CBO_FRMT, bCheckEntryMRU);
}


// **********************************************
// OnNotify - provides the tooltip control with the appropriate text 
//     to display for a control window. This function is called by 
//     the dialog box procedure in response to a WM_NOTIFY message. 
// lParam - second message parameter of the WM_NOTIFY message. 
void CDlgID3WriteTagFromFile::OnNotify(LPARAM lParam) 
{ 
	LPTOOLTIPTEXT lpttt; 
	int idCtrl; 

	if ((((LPNMHDR) lParam)->code) == TTN_NEEDTEXT) 
	{ 
		idCtrl = GetDlgCtrlID((HWND) ((LPNMHDR) lParam)->idFrom); 
		lpttt = (LPTOOLTIPTEXT) lParam; 

	
		// special action for DropDown listbox
		// trap edit box child window --> combobox dropdown list
		int idCtrlParent = GetDlgCtrlID(GetParent((HWND) ((LPNMHDR) lParam)->idFrom));
		if (idCtrlParent == IDC_TAG_CBO_FRMT)
			idCtrl = idCtrlParent;

		switch (idCtrl) 
		{ 
			
		case IDC_TAG_BTN_LOG:
			lpttt->lpszText = (TCHAR*)GS(LNG_MSG_CHOOSELOG); 
			break;
		case IDC_TAG_EDIT_LOG: 
			{
				int len = GetWindowTextLength(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG));
				m_strTooltipText.reserve(len+1);
				GetWindowText(GetDlgItem(m_hDlg, IDC_TAG_EDIT_LOG),  (LPTSTR)m_strTooltipText.c_str(), len+1);

				lpttt->lpszText = (TCHAR*)m_strTooltipText.c_str(); 			
			}
			break; 
		case IDC_TAG_CBO_FRMT:
			lpttt->lpszText = (TCHAR*)_T("format filename output");  //$$$$$$$$$$$$$$ a faire
			break;
		
		} 
	} 


	return; 
} 


/* ************************************************************************ */
void CDlgID3WriteTagFromFile::BtnGO()
{
	HCURSOR hcurOld;
	int i, nCount;
	int *pArray = NULL;
	FILE *filelog = NULL;
	BOOL bOK, bSelected;
	const TCHAR *Msg_label[] = {GS(LNG_FAILED), GS(LNG_SUCCESS)};
	TCHAR szMsg[1024];
	int lenFormat;
	LPTSTR pszFormat;

	SaveParam(FALSE);

	lenFormat = SendDlgItemMessage(m_hDlg, IDC_TAG_CBO_FRMT,  CB_GETLBTEXTLEN, 0, 0);
	pszFormat  = (LPTSTR)calloc(sizeof(TCHAR), lenFormat+2);
	SendDlgItemMessage(m_hDlg, IDC_TAG_CBO_FRMT,  CB_GETLBTEXT, 0, (LPARAM) (LPCSTR)pszFormat);


	if (checkGoodFormat(pszFormat) == FALSE)
	{
		MessageBox(m_hDlg, GS(LNG_INVALID_FORMAT), GS(LNG_ERROR), MB_OK | MB_ICONSTOP);
		SetFocus(GetDlgItem(m_hDlg, IDC_TAG_CBO_FRMT));
		free(pszFormat);
		return;
	}

	BuildParseList(pszFormat); 

	
	bOK = PrepareItemList(bSelected, nCount, (int**)&pArray, 2);
	if(!bOK)
		return;

	filelog = PrepareLogFile(&bOK);
	if(!bOK)
		return;

	if (filelog)
	{	_ftprintf(filelog, GS(LNG_MSG_WRITETAG_LOG), ID3TagName[gParam.nWriteTagfromFileID3version+1], nCount);
		_ftprintf(filelog, _T("\n\n"));
	}

	hcurOld = SetCursor(LoadCursor(NULL, IDC_WAIT));
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), FALSE);


	// Init Progress bar
	HWND hProgress = GetDlgItem(m_hDlg, IDC_PROGRESS1);
	HWND hStatus = GetDlgItem(m_hDlg, IDC_TAG_WRITETAGFROMFILE_STATUS);
	ShowWindow(hStatus, SW_SHOW);

	InitProgressBar(hProgress, nCount);

	// Scan file item	
	for (i = 0; i<nCount; i++)
	{
		int iSel = (bSelected ? pArray[i] : i);

		CMP3Info *mp3i = (CMP3Info *)SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE,  LB_GETITEMDATA , (WPARAM)iSel, 0);
		
		bOK = WriteTAGFromFile(mp3i, iSel, pszFormat);
		
		if (filelog)
		{	_ftprintf(filelog, GS(LNG_LOG_FILE), i+1, Msg_label[bOK], mp3i->GetCFilename());	
			_ftprintf(filelog, _T("\n") );
		}
		
		SendMessage(hProgress, PBM_STEPIT, 0, 0);
		_stprintf(szMsg, GS(LNG_STATUS_WORKING), i+1, nCount, mp3i->GetCFilename());
		SetWindowText(hStatus, szMsg);
	}
	
	EnableWindow(GetDlgItem(m_hDlg,IDC_TAG_BTN_GO), TRUE);
	SetCursor(hcurOld);

	if (bSelected)
		free(pArray);
	
	if (filelog)
		fclose(filelog);

	// Show resume action in MsgBox
	_stprintf(szMsg, GS(LNG_MSG_WRITETAG_RESULT), nCount);
	MessageBox(m_hDlg, szMsg, GS(LNG_INFORMATION), MB_OK | MB_ICONINFORMATION);

	ShowWindow(hProgress, SW_HIDE);
	ShowWindow(hStatus, SW_HIDE);
}


/* ************************************************************************ */
BOOL CDlgID3WriteTagFromFile::WriteTAGFromFile(CMP3Info *mp3i, int iItem, TCHAR *pszFormat)
{
	TCHAR szTemp[1024];
	TCHAR szBuf[1024];
	TCHAR *p, *pSep;
	int sizeSep;
	int sizeFrame;
	int nAddressLength;
	int nCurrentAddress = 0;
	struct ID3V1MP3Info id3i;
	BOOL bEnableID3v1 = FALSE;
	BOOL bEnableID3v2 = FALSE;
	int nSep = 0;
	int nTag = 0;
	
	int ntagVersion = mp3i->GetTagVersion();

	if (!mp3i) 
		return FALSE;

	CMP3Info *mp3iSave = new CMP3Info(mp3i->GetFilename());

	if (!mp3iSave)
		return FALSE;	


	switch(gParam.nWriteTagfromFileID3version)
	{
		case 0: bEnableID3v1 = TRUE; break;
		case 1: bEnableID3v2 = TRUE; break;
		case 2: bEnableID3v1 = bEnableID3v2 = TRUE; break;
	}


	// Get filename without path
	SendDlgItemMessage(m_hDlg, IDC_TAG_LIST_FILE, LB_GETTEXT, (WPARAM)iItem, (LPARAM)(LPCTSTR)szBuf);
	_tcscpy(szTemp, szBuf);


	// Prepare and set Tag Version 
	if (bEnableID3v1)
	{
		struct ID3V1MP3Info *id3iSave = NULL;
		id3iSave = mp3iSave->GetID3v1();
		
		if (id3iSave != NULL)
			memcpy(&id3i, id3iSave, sizeof(struct ID3V1MP3Info));
		else
			memset(&id3i, 0, sizeof(struct ID3V1MP3Info));

		ntagVersion |= VERSION_ID3V1;

	}
	if (bEnableID3v2)
	{
		ntagVersion |= VERSION_ID3V2;
	}

	mp3iSave->SetTagVersion(ntagVersion);

	// Remove  extension '.mp3'
	nAddressLength = _tcslen(szTemp);
	nAddressLength -= 4,
	szTemp[nAddressLength] = '\0';


	// skip first separator if exist
	if (m_bFirstSeparator == TRUE)
	{
		sizeSep = _tcslen( (m_listSeperator[nSep]).c_str());

		nCurrentAddress += sizeSep;
		nSep++;
	}
	
	
	while (nCurrentAddress < nAddressLength)
	{
		BOOL bFoundSeparator = FALSE;

		p = &szTemp[nCurrentAddress];

		if (nSep < m_listSeperator.size())
		{
			sizeSep = _tcslen( (m_listSeperator[nSep]).c_str());
			
			// find the next separator
			pSep = _tcsstr(&szTemp[nCurrentAddress], (m_listSeperator[nSep]).c_str() );
		
			if (pSep)
			{	// separator found
				sizeFrame = pSep - p;

				bFoundSeparator = TRUE;
			}
		}


		if (bFoundSeparator == FALSE)
		{	// no found separator
			sizeSep = 0;
			sizeFrame = _tcslen(&szTemp[nCurrentAddress]);
			nCurrentAddress = nAddressLength; // force exit while loop
		}


		if (!IsStringEmpty((LPSTR)p)) 
		{	
			TCHAR szFrame[1024];
		
			_tcsncpy(szFrame,  (LPTSTR)p, sizeFrame);
			szFrame[sizeFrame] = '\0';

			// trim frame
			tstring frame(szFrame);
			TrimString(frame);

			// Capitalize options
			CapitalizeString((TCHAR*)frame.c_str());

			// Convert options Underscore <--> space
			ConvertString((TCHAR*)frame.c_str());

	
			//  ** ID3v1 **
			if (bEnableID3v1)
			{
				TCHAR *pTag = (TCHAR*)(m_listTag[nTag]).c_str();
				
				switch(*pTag)
				{
					case 'A':	_tcsncpy((LPTSTR)&id3i.Artist, frame.c_str(), sizeof(id3i.Artist));	break;
					case 'T':	_tcsncpy((LPTSTR)&id3i.SongName, frame.c_str(), sizeof(id3i.SongName));	break;
					case 'L':	_tcsncpy((LPTSTR)&id3i.Album, frame.c_str(), sizeof(id3i.Album));	break;
					case 'Y':	_tcsncpy((LPTSTR)&id3i.Year, frame.c_str(), sizeof(id3i.Year));	break;
					case 'C':	_tcsncpy((LPTSTR)&id3i.Comment, frame.c_str(), sizeof(id3i.Comment));	break;
					case 'N':	id3i.Track = _ttoi(frame.c_str()); break;
				}

			}
			
			//  ** ID3v2 **
			if (bEnableID3v2)
			{
				CMP3Frame *mp3f = NULL;
		
				TCHAR *pTag = (TCHAR*)(m_listTag[nTag]).c_str();
				
				switch(*pTag)
				{
					case 'A':	mp3f = new CMP3StringFrame(ID3TAG_TPE1, frame);	break;
					case 'T':	mp3f = new CMP3StringFrame(ID3TAG_TIT2, frame);	break;
					case 'L':	mp3f = new CMP3StringFrame(ID3TAG_TALB, frame);	break;
					case 'Y':	mp3f = new CMP3StringFrame(ID3TAG_TYER, frame);	break;
					case 'C':	mp3f = new CMP3StringFrame(ID3TAG_COMM, frame);	break;
					case 'N':	
						{
							int nTrack = _ttoi(szFrame);
							_stprintf(szFrame, _T("%d"), nTrack);
							tstring sTrack(szFrame);
							mp3f = new CMP3StringFrame(ID3TAG_TRCK, sTrack);	break;
						}
				}

				if (mp3f)
					mp3iSave->ReplaceFrame(mp3f);

			}

			nTag++;
		
			
			
		}
		// Increment the address for next iteration. 
		nCurrentAddress += sizeFrame + sizeSep;

		nSep++;

	}
	


	// Set the new ID3v1 in MP3Info
	if (bEnableID3v1)
		mp3iSave->SetID3v1(&id3i); 	
	
	// Write modification ID3TAG
	if (mp3iSave->SaveTag() != ID3V2_ERR_OK)
		return FALSE;			

	mp3i->CopyAll(mp3iSave);
	delete mp3iSave;


	return TRUE;			
}



/* ************************************************************************ */
BOOL CDlgID3WriteTagFromFile::BuildParseList(TCHAR *pszFormat)
{
	TCHAR *p = pszFormat;
	string separator;
	BOOL bFirst = TRUE;

	m_bFirstSeparator =  FALSE;
	m_listSeperator.clear(); 
	m_listTag.clear(); 

	separator = "";

	while(1)
	{
		if (*p == 0) break;

		if (*p != '%')
		{
			separator += *p;
		}
		else
		{
			++p;
			
			switch(*p)
			{
			case '\0':
				goto endparse;
			case 'A': // Artist
			case 'T': // title
			case 'L': // Album
			case 'Y': // year
			case 'C': // comment
			case 'N': // track#
 			case 'G': // genre
				{
					// Store separator	
					if (separator.length() > 0)
					{					
						m_listSeperator.push_back(separator);
						separator = "";

						if (bFirst == TRUE)
						{	if (m_listSeperator.size()> m_listTag.size())	
								m_bFirstSeparator = TRUE;
						}
						bFirst = FALSE;

					}

					// Store tagID
					string tag; 
					tag = *p;
				
					m_listTag.push_back(tag);
					
				}
				break;
			default:
				separator += *p;
				break;
			}
		}
		p++;

	}

endparse:

	return TRUE;
}
