//*******************************************************************************************
//
//	Definitions of CDlgPlaylist
//
// Copyright (c) 2001 JL BOYER. All rights reserved
//
//*******************************************************************************************

#ifndef _DlgPlaylist_H_
#define _DlgPlaylist_H_

#include <commctrl.h>
#include "Dlg.h"

#include "mp3info/MP3Info.h"
#include "mp3info/MP3MultiInfo.h"
#include "FooButton.h"

#define IDC_STATUS 400
#define ID_LISTVIEW 501

// Imagelist "Playlist"
#define IMGL_COUNT          16    
#define IMGL_BMWIDTH	    16
#define IMGL_BMHEIGHT		16
#define IMGL_RGBREPLACE		0x00FF0000 // solid blue 0x00BBGGRR

#define IMGL_NOID3          0
#define IMGL_ID3V1          1
#define IMGL_ID3V2          2
#define IMGL_ID3V1_V2       3
#define IMGL_STAR_ON		4  // position imagelist StarOn  (rate popularity-meter)
#define IMGL_STAR_OFF       5  // position imagelist StarOff (rate popularity-meter)
#define IMGL_MEDIA          6
#define IMGL_ARTIST         7
#define IMGL_GENRE          8
#define IMGL_COMMENT        9
#define IMGL_ALBUM          10
#define IMGL_STEREO         11
#define IMGL_JOINSTEREO     12
#define IMGL_DOUBLECHANNEL  13
#define IMGL_SINGLECHANNEL  14
#define IMGL_FOLDER         15



typedef struct tag_listviewUserData
{
	WNDPROC  lpOldWndProc;
	COLORREF clrBkg;
} LISTVIEWUSERDATA;


typedef struct tag_ThreadParam
{
	int type;
	int count; 
	LPARAM lParam;
} THREADPARAM;

class CDlgPlaylist: public CDlg
{

public:
	UINT m_idTimer ;    // Timer for current slider position
	HWND m_hwndList;
	MP3MultiInfo m_minfo;		// store the info of all files

	typedef vector<CMP3Info *> MSelArray;     // array selection 
	MSelArray m_mp3iSelArray;



	int m_nSelCount;
	tstring m_filterArtist;		// filter
	tstring m_filterAlbum;
	tstring m_filterComment;

	long m_nTotalLength;
	long m_nAvgLength;
	long m_nSelLength;

	LISTVIEWUSERDATA m_listViewUsetData;
	COLORREF m_clrBkg;
	COLORREF m_clrBkgLine;
	COLORREF m_clrText;
	COLORREF m_clrBkgSel;
	COLORREF m_clrTextSel;
	COLORREF m_clrTextPlay;
	COLORREF m_clrBkgPlay;
	//COLORREF m_clrTextHilite;

	tstring m_strCol;       // temporary string see: LV_GetInfoCol

	int m_filterMode;
	int m_filterID3Tag;
	HANDLE m_hThread;		// Thread Filter
	HANDLE m_hQuitEvent;
	BOOL m_bThreadActive;

	int m_nFrmt_FileSize;  // 0: Bytes, 1: Kb, 2: Mb
	int m_nFrmt_Time;      // 0: hh:mm:ss ou mm:ss, 1: sss

	HWND m_hwndMain;		// use by CSystray to create dialogbox and show/hide windowss
	BOOL m_bVisible;	
	
	THREADPARAM m_ThreadParam;    // parameters structure for Thread CAbortProgress

	int m_itemPlayingOld;
	int m_itemPlayingCurrent;


private:
	
	FooButton *m_btnOptionSearch;
	FooButton *m_btnFile;
	FooButton *m_btnAdd;
	FooButton *m_btnPlay;
	FooButton *m_btnPause;
	FooButton *m_btnStop;
	FooButton *m_btnTrackPrevious;
	FooButton *m_btnTrackNext;

	int  m_realColumn[COLUMN_MAX];
	int  m_last_sort;
	BOOL m_bShowDuplicate;
	BOOL m_bRefreshSelLength;	// use :ST_UpdateLengthSelected
								//      Need to compute selected length

	int  m_nFilterBy;
	HWND m_hwndStat;
	HWND m_hwndSplitter;
	HWND m_hwndTree;
	HICON m_hIcon;


	HCURSOR m_hSplitterCursor;	// Splitter variables
	BOOL m_bSplitFirstTime; 
	RECT m_SplitDragRect;
	RECT m_SplitLimitRect;
	int m_nSplitterOffset;
	BOOL m_bTracking;

	StrArray m_StrArray;	// Treeview explorer
	HTREEITEM m_hRoot[6];

	enum { COMDLG_OPEN = 0, COMDLG_SAVE, COMDLG_SAVE_AS, COMDLG_INSERT};
	enum { NO_FILTER = 0, FILTER_GENRE = 1,FILTER_ARTIST = 2,FILTER_ALBUM = 4, FILTER_COMMENT = 8, FILTER_ID3TAG = 16, FILTER_MODE = 32};
	enum { ROOT_ARTIST = 0, ROOT_GENRE, ROOT_ID3TAG, ROOT_COMMENT, ROOT_MODE, ROOT_ALBUM};
	enum eSplitterHits {   SPLITTER_HITNOTHING = 0,    SPLITTER_HITITEM, };


public:
	CDlgPlaylist() ;
	~CDlgPlaylist() ;
	void Create();
	void LV_ColumnGetRealOrder(int order[]);
	void LV_UpdateListWithFilter();
	int InsertFile(tstring& file);
	HIMAGELIST CreateImageList();

	void OnChangeTheme(BOOL refresh=TRUE);
	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	void OnInitDialog();
	void OnDestroy();
	void OnClose();
	void OnTimer();
	void OnActivate(WPARAM wParam);
	BOOL OnCommand (WPARAM wParam, LPARAM lParam);
	BOOL OnSize(WPARAM wParam, LPARAM lParam);
	void OnDrawItem (LPDRAWITEMSTRUCT lpdis);
	BOOL OnContextMenu(WPARAM wParam, LPARAM lParam);
	void OnInitMenu(WPARAM wParam);
	void OnDropFiles(HANDLE hFilesInfo);
	BOOL OnSlider(WPARAM wParam, LPARAM lParam);

	void changeColorCtrl(void);

	void LoadParam(void);
	void SaveParam(void);
	LRESULT ProcessListViewCustomDraw (LPARAM lParam);
	LRESULT ProcessTreeViewCustomDraw (LPARAM lParam);
	LRESULT NotifyListView(LPARAM lParam);
	LRESULT NotifyTreeView(LPARAM lParam);
	
	static int CALLBACK ListViewCompareProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort); //int nSubItem);

	static  LRESULT CALLBACK ListViewWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);


	void LV_ColumnResetDisplay(BOOL firstime);
	void LV_ColumnSort(int column, BOOL bColumn, BOOL bReverse=TRUE);
	int  LV_ColumnGetNum();
	void LV_ColumnSaveSetting();
	void LV_InitIcon();
	void LV_ColumnsGrid();
	void LV_AddFiles();
	void LV_AddDirectory();
	void LV_AddDir2List(TCHAR* dir);
	void LV_New();
	void LV_ComDlg(int OPENSAVE);
	void LV_Import();
	void LV_Export();
	void LV_Playlist(int);
	void LV_PlaylistWinamp(int);
	void LV_LoadEZ3(TCHAR *filename, BOOL bInitList=TRUE);
	void LV_LoadM3U(TCHAR *filename);
	void LV_LoadPLS(TCHAR *filename);
	void LV_WriteEZ3(TCHAR*);
	BOOL LV_WriteM3U(TCHAR*);
	BOOL LV_WritePLS(TCHAR*);
	BOOL LV_WriteHTML_Table(TCHAR*);
	BOOL LV_WriteHTML_Listing(TCHAR*);
	void LV_WriteHTML_TableColumn(FILE *file, CMP3Info *mp3i, int iCol, int iColArtist);
	void LV_WriteHTML_CSS(FILE *file, TCHAR *title);
	void LV_WriteHTML_Footer(FILE *file, unsigned long uTotalSize, long nTotalLength, int nCountSelect);
	BOOL LV_WriteXML(TCHAR*);
	void LV_WriteXML_ReplaceSpecChar(FILE *file, const TCHAR *szTmp);
	BOOL LV_WriteCSV(TCHAR*);
	BOOL LV_WriteASCII(TCHAR*);
	BOOL LV_WriteASCIIFixed(TCHAR* filename);
	void LV_SelectAll(BOOL bSelect);
	void LV_SelectToggle();
	void LV_SelectWithSame(int nCol);
	void LV_SelectCrop();
	void LV_CopyMove(int type);
	void LV_SelectDel();
	void LV_SelectRemoveFile();
	void LV_UpdateStatus();
	void LV_StatGenre();
	void LV_UpdatePlaylist();
	void LV_DetermineFilter();
	BOOL LV_Filter(CMP3Info *mp3i);
	void LV_TitleWindowFilter();
	void LV_ShowDuplicate();
	void LV_DeadFiles();
	void LV_Redraw(BOOL bUpdate);
	void LV_MatchTime(int nLengthMax);
	void LV_ChangeID3version(WORD wID);
	void LV_UpdateItemPlaying(int iItem);

	void GetInfoCol(CMP3Info *mp3i, int col, tstring  &strCol, int iItem=-1); 

	void UpdateMenu(HMENU hMenu); 
	void MenuButton(int nButton);				
	void MenuCheckItem(int nItem);				

	void SetActive();

	void NoSelection();
	void GetMP3Selected();
	void SetMP3Selected();
	
	void ST_UpdateLengthSelected();
	BOOL ST_CreateStatus(int nMode);
	BOOL ST_ChangeText(LPTSTR szText, int iPart );

	void winamp_enqueue_to_playlist(BOOL bPlay);
	void winamp_enqueue_all_to_playlist(BOOL bPlay);

	void UpdateStatus();
	void ResizeControls();

	BOOL TV_Create();
	void TV_Destroy();
	void TV_Load();
	void TV_Update();
	void TV_Redraw(BOOL bUpdate);
	void TV_SetCurrentItem();

	int AddVector(tstring& str);
	int FindVector(tstring& str);


	void Splitter_OnInvertTracker(HWND hWnd, const RECT *rect);
	void Splitter_Recalc(void);
	void Splitter_StartTracking(HWND hWnd, UINT hitArea);
	void Splitter_StopTracking(HWND hWnd);
	void Splitter_OnMouseMove(HWND hWnd, UINT nFlags, POINTS p);
	void Splitter_OnLButtonDown(HWND hWnd, UINT nFlags, POINTS p);
	void Splitter_OnLButtonUp(HWND hWnd, UINT nFlags, POINTS p);
	UINT Splitter_HitTest(HWND hWnd, POINTS p);	

	BOOL LaunchThreadFilter();
	void SetTextMenu();
	void SetTextMenuPop(HMENU menu, int nType);
	void RefreshAllCtrl_Lang();
	void VolumeUpdate(BOOL);
	void SetNameColumn();


	// for listview header 
	// dessine triangle Tri Croissant/dessendant dans l'entete de colonne
	static LRESULT CALLBACK Header_WndProc(HWND, UINT, WPARAM, LPARAM);
	static void             Header_SetSortInfo(HWND hwndList, int nCol, BOOL bAsc);
	static void             Header_Initialize(HWND hwndList);

	static BOOL m_reverse_sort;
	static UINT m_uHeaderSortCol;
	static BOOL m_fHeaderSortAsc;

	#ifdef STRICT 
		static WNDPROC g_lpHeaderWndProc;
	#else 
		static FARPROC g_lpHeaderWndProc;
	#endif 



	UINT CDlgPlaylist::GetTextJustify(const int headerpos);

	void CDlgPlaylist::ListViewDrawText(int iItem, int iSubItem, HDC hdc, COLORREF clrText, COLORREF clrBkg);
	void CDlgPlaylist::ListViewDrawItem (int iItem, int iSubItem, HDC hdc);


};

#endif