// DlgThemeColor.cpp: implementation of the CDlgThemeColor class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "DlgThemeColor.h"

#include "gen.h"
#include "definition.h"
#include "Util.h"
#include "Language.h"
#include "settings.h"
#include "resource.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// external declaration
extern INITSTRUCT gParam;
extern winampGeneralPurposePlugin plugin;
extern CLanguage    *lng;



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CDlgThemeColor::CDlgThemeColor(CDlgPlaylist* pThis): CDlgButtonColor()
{
	m_pThisPlaylist = pThis;
}

CDlgThemeColor::~CDlgThemeColor()
{

}





/* ************************************************************************ */
BOOL CDlgThemeColor::RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		InitLanguage();
		LoadParam();
		return FALSE;

	case WM_DRAWITEM:
     	doDrawBtnColor((LPDRAWITEMSTRUCT)lParam, IDC_THEME_COLORBUTTON1, IDC_THEME_COLORBUTTON7, IDC_THEME_COLOREDIT1);
		return TRUE;

	case WM_SYSCOLORCHANGE:         
	case WM_PALETTECHANGED:         
	case WM_PALETTEISCHANGING:         
	case WM_QUERYNEWPALETTE:         
	case WM_DEVMODECHANGE:         
	case WM_DISPLAYCHANGE: 
		// Forward this message to common controls so that they will
		// be properly updated when the user changes the color settings.
		{
			CSetting setting;
			setting.UpdateThemeWindowColor(); 
		}
		return TRUE;

	case WM_COMMAND:

		switch (LOWORD(wParam))
		{
		case IDC_THEME_COLORBUTTON1:
		case IDC_THEME_COLORBUTTON2:
		case IDC_THEME_COLORBUTTON3:
		case IDC_THEME_COLORBUTTON4:
		case IDC_THEME_COLORBUTTON5:
		case IDC_THEME_COLORBUTTON6:
		case IDC_THEME_COLORBUTTON7:
			doChooseColor(LOWORD(wParam), IDC_THEME_COLORBUTTON1);
			return TRUE;

		case IDC_THEME_COLOREDIT1:
		case IDC_THEME_COLOREDIT2:
		case IDC_THEME_COLOREDIT3:
		case IDC_THEME_COLOREDIT4:
		case IDC_THEME_COLOREDIT5:
		case IDC_THEME_COLOREDIT6:
		case IDC_THEME_COLOREDIT7:
			if (HIWORD(wParam) == EN_UPDATE)
				doEditHexColor(LOWORD(wParam), IDC_THEME_COLOREDIT1, IDC_THEME_COLORBUTTON1 );
			return TRUE;		
		

		case IDC_THEME_CBO:
			doChangeStyle();
			return TRUE;

		case IDOK:
			if (SaveParam() == TRUE)
				EndDialog(m_hDlg, 1);
			return TRUE;

		case IDCANCEL:
			EndDialog(m_hDlg, 0);
			return TRUE;
		}
		
	}
	return FALSE;
}


/* ************************************************************** */
BOOL  CDlgThemeColor::LoadParam()
{
	int i;
	m_bIsLoading = TRUE;

	// Init control Combobox
	for  (i = 0; i<MAX_THEMECOLOR; i++)
		SendDlgItemMessage(m_hDlg, IDC_THEME_CBO, CB_ADDSTRING, 0, (LPARAM)gParam.Theme[i].szName);	
	

	// update Windows Standard color
	{
		CSetting setting;
		setting.UpdateThemeWindowColor(); 
	}

	m_pTheme = &gParam.Theme[gParam.nCurrentTheme];
	
	SendDlgItemMessage(m_hDlg, IDC_THEME_CBO,  CB_SETCURSEL, gParam.nCurrentTheme, 0);
	SetFocus(GetDlgItem(m_hDlg, IDC_THEME_CBO));


	SendDlgItemMessage(m_hDlg, IDC_THEME_CBO2, CB_ADDSTRING, 0, (LPARAM)"EZPlaylist (classic)");	
	SendDlgItemMessage(m_hDlg, IDC_THEME_CBO2, CB_ADDSTRING, 0, (LPARAM)"EZPlaylist (modern)");	
	SendDlgItemMessage(m_hDlg, IDC_THEME_CBO2, CB_SETCURSEL, gParam.nThemeIconPackage, 0);

	doChangeStyle();

	m_bIsLoading = FALSE;
	return TRUE;
}


/* ************************************************************** */
BOOL  CDlgThemeColor::SaveParam()
{
	// Save options
	gParam.nCurrentTheme     = SendDlgItemMessage(m_hDlg, IDC_THEME_CBO,   CB_GETCURSEL, 0, 0);
	gParam.nThemeIconPackage = SendDlgItemMessage(m_hDlg, IDC_THEME_CBO2,  CB_GETCURSEL, 0, 0);


#ifdef _DEBUG
{
TCHAR tmp[1024];
TCHAR szBuffer[100];
strcpy(tmp, "{ ");
for (int i=0; i<7; i++)
{	COLORREF crColorEdit = GetColorButton(i);
	sprintf(szBuffer, "0x00%6lx, ", crColorEdit);
	strcat	(tmp, szBuffer);
}
strcat	(tmp, " }, \n");
OutputDebugString(tmp);
}
#endif

	return TRUE;
}

/* ************************************************************** */
void CDlgThemeColor::doChangeStyle()
{
	int i;
	int nTheme = SendDlgItemMessage(m_hDlg, IDC_THEME_CBO,  CB_GETCURSEL, 0, 0);
	BOOL bEnable = (nTheme == 0 ? FALSE : TRUE);

	m_pTheme = &gParam.Theme[nTheme];


	// Refresh Color Theme Button + Edit color
	for (i=0; i< 7; i++)
	{
		SendDlgItemMessage( m_hDlg, IDC_THEME_COLOREDIT1 + i, EM_SETLIMITTEXT , 6L, 0L);
		InvalidateRect(GetDlgItem(m_hDlg,IDC_THEME_COLORBUTTON1 + i), NULL, TRUE);

		EnableWindow(GetDlgItem(m_hDlg, IDC_THEME_COLOREDIT1 + i), bEnable);
		EnableWindow(GetDlgItem(m_hDlg, IDC_THEME_COLORBUTTON1 + i), bEnable);
	}
	

	gParam.nCurrentTheme = nTheme;
	m_pThisPlaylist->OnChangeTheme();
}

/* ************************************************************************** */
COLORREF CDlgThemeColor::GetColorButton(int index)
{                                                    
	switch(index)
	{
		case 0:	return m_pTheme->cText;
		case 1:	return m_pTheme->cBkg;
		case 2:	return m_pTheme->cBkgAlternate;
		case 3:	return m_pTheme->cTextSel;
		case 4:	return m_pTheme->cBkgSel;
		case 5:	return m_pTheme->cTextPlay;
		case 6:	return m_pTheme->cBkgPlay;
	}

	return RGB(0,0,0);
}

/* ************************************************************************** */
void CDlgThemeColor::SetColorButton(int index, COLORREF cColorNew)
{                                                    
	switch(index)
	{
		case 0:	m_pTheme->cText         = cColorNew; break;
		case 1:	m_pTheme->cBkg          = cColorNew; break;
		case 2:	m_pTheme->cBkgAlternate = cColorNew; break;
		case 3:	m_pTheme->cTextSel	    = cColorNew; break;
		case 4:	m_pTheme->cBkgSel       = cColorNew; break;
		case 5:	m_pTheme->cTextPlay	    = cColorNew; break;
		case 6:	m_pTheme->cBkgPlay      = cColorNew; break;
	}

	
	m_pThisPlaylist->OnChangeTheme();
}



// **********************************************
void CDlgThemeColor::InitLanguage()
{
	LNGRESSTRUCT lrs[14] = 
	{
		{ R_THEME_APPARENCE, LNG_DLGTHEME_APPARENCE },
		{ IDC_STATIC_THEMECOLOR, LNG_DLGTHEME_COLOR },
		{ R_THEME_TEXTCOLOR, LNG_DLGTHEME_TEXTCOLOR },
		{ R_THEME_BKGCOLOR, LNG_DLGTHEME_BKGCOLOR },
		{ R_THEME_BKGCOLOR_LINE, LNG_DLGTHEME_BKGCOLOR_LINE },
		{ R_THEME_SEL_TEXTCOLOR, LNG_DLGTHEME_SEL_TEXTCOLOR },
		{ R_THEME_SEL_BKGCOLOR, LNG_DLGTHEME_SEL_BKGCOLOR },
		{ R_THEME_PLAY_TEXTCOLOR, LNG_DLGTHEME_PLAY_TEXTCOLOR },
		{ R_THEME_PLAY_BKGCOLOR, LNG_DLGTHEME_PLAY_BKGCOLOR },
		{ IDC_STATIC_THEMECOLOR2,  LNG_DGLTHEME_ICONSET },
		{ R_THEME_PACKAGE,  LNG_DGLTHEME_PACKAGE },
		{ R_THEME_NEEDRESTART,  LNG_DGLTHEME_NEEDRESTART },
		{ IDOK, LNG_BTN_OK },
		{ IDCANCEL, LNG_BTN_CANCEL }
	};
	
	for (int i = 0 ; i<NUMELEMENTS(lrs); i++)
		DlgSetText(m_hDlg, lrs[i].idRes, GS(lrs[i].idLNG) );
	
	SetWindowText(m_hDlg, GS(LNG_DLGTHEME_TITLE) );
}

// /////////////////////////////////
// End part for CDlgThemeColor Dialog
// /////////////////////////////////