// FileDir.cpp: implementation of the CFileDir class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FileDir.h"
#include "Language.h"

#include <stdlib.h>
#include <shlobj.h>		// SHMalloc, ...ShellBrowser


/*  www.codeguru.com
 * It is possible to add a button to the SHBrowseForFolder dialog 
 * (or indeed any other control desired). This is achieved by dynamically 
 * subclassing the dialog - ie writing a new WindowProc for the dialog to handle 
 * the messages you're interested in. You pass all other messages back to the 
 * old WindowProc. 
 */ 

// internal function 
int CALLBACK BrowseRecurseCallbackProc(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData);
int CALLBACK BrowseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM pData);
 
extern BOOL gbCheckRecurseDir;
extern CLanguage    *lng;

// Control ID of our button checkbox (recurse subdirectories)
#define ID_CHECK_RECURSE 200 

// Window commands to pass to our WindowProc 
#define WM_BROWSE_CREATECHECKBOX (WM_USER+100) 

// Control ID of the tree control in the dialog 
#define BROWSETREECTRLID 0x3741 

// internal function
LRESULT CALLBACK BrowseFolderWindowProc(HWND hwnd,UINT msg, WPARAM wParam, LPARAM lParam);
 


//=================================================================== 
int CALLBACK BrowseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM pData) 
{ 
	if(uMsg == BFFM_INITIALIZED) 
	{	// Set the current directory
		SendMessage(hwnd, BFFM_SETSELECTION, TRUE, pData);
	} 
	return 0; 
}

//=================================================================== 
// Function name : BrowseRecurseCallbackProc 
// Description : Callback function for Browse dialog 
// Entry : 
// : HWND hwnd 
// : UINT uMsg 
// : LPARAM lp 
// : LPARAM pData 
// 
// Return type : int 
//======================================================= 
int CALLBACK BrowseRecurseCallbackProc(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM pData) 
{ 
	if(uMsg == BFFM_INITIALIZED) 
	{ 
		// Set the current directory
		SendMessage(hwnd, BFFM_SETSELECTION, TRUE, pData);
		// Save Old Window proc 
		WNDPROC OldWndProc = (WNDPROC)SetWindowLong(hwnd,GWL_WNDPROC, (LONG)BrowseFolderWindowProc ); 
		SetWindowLong(hwnd, GWL_USERDATA, (LONG)OldWndProc ); 
		// Add a new checkbox
		SendMessage( hwnd, WM_BROWSE_CREATECHECKBOX, 0, (LPARAM)pData ); 
	} 
	return 0; 
} 
//============================================================ 
// Function name : BrowseFolderWindowProc 
// Description : Window procedure for browse dialog 
// Entry : 
// : HWND hwnd 
// : UINT msg 
// : WPARAM wParam 
// : LPARAM lParam 
// 
// Return type : LRESULT 
//================================================================ 
LRESULT CALLBACK BrowseFolderWindowProc(HWND hwnd,UINT msg, WPARAM wParam, LPARAM lParam) 
{ 
	static WNDPROC OldWndProc = NULL; 
	static bool bGotWndProc = false; 
	static bool bButtonCreated = false; 

	switch ( msg ) 
	{ 
		case WM_BROWSE_CREATECHECKBOX: 
		{ 
			if ( bButtonCreated == false ) 
			{ 
			
				HWND hWndCheckbox = CreateWindow(
					_T("BUTTON"), 
					GS(LNG_SHBROWSEFOLDER_CHECKBOX), 
					BS_AUTOCHECKBOX | WS_CHILDWINDOW | WS_VISIBLE | WS_TABSTOP | BS_NOTIFY , 
					12,						// x
					8,						// y
					280,					// window width
					20,						// window height
					hwnd,					// Handle to parent or window
					(HMENU)ID_CHECK_RECURSE,	// Menu
					GetModuleHandle(NULL), 
					NULL);

			
				CheckDlgButton(hwnd, ID_CHECK_RECURSE, BST_CHECKED); 
				gbCheckRecurseDir = TRUE;
			} 
		} 
		break; 
	
	
		case WM_COMMAND: 
		{ 
			switch ( LOWORD(wParam) ) 
			{ 
				case ID_CHECK_RECURSE: 
				{ 
					if (IsDlgButtonChecked(hwnd,ID_CHECK_RECURSE))	
						gbCheckRecurseDir = TRUE;	
					else
						gbCheckRecurseDir = FALSE;	

				} 
			} 
		} 
		break; 
	} 


	// Call original window procedure 
	if ( bGotWndProc == false ) 
	{ 	OldWndProc = (WNDPROC) GetWindowLong( hwnd, GWL_USERDATA ); 
		if ( OldWndProc != NULL ) 
			bGotWndProc = true; 
		else 
			OldWndProc = (WNDPROC) GetWindowLong( hwnd, GWL_WNDPROC ); 
	} 
	
#ifdef STRICT 
	return CallWindowProc((WNDPROC)OldWndProc, hwnd, msg, wParam, lParam); 
#else 
	return CallWindowProc((FARPROC)OldWndProc, hwnd, msg, wParam, lParam); 
#endif 
	
} 






//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CFileDir::CFileDir()
{
	Init();
}

CFileDir::CFileDir(tstring &filename) 
{
	m_filename = filename;
	Init();
}


CFileDir::CFileDir(LPCTSTR pszName) 
{
	m_filename = pszName;
	Init();
}


CFileDir::~CFileDir()
{
	if(m_szpath_buffer) free(m_szpath_buffer) ;
    if(m_szdrive) free(m_szdrive) ;
    if(m_szdir) free(m_szdir) ;
    if(m_szfname) free(m_szfname) ;
    if(m_szext) free(m_szext);
}


void CFileDir::Init()
{
	m_szpath_buffer = NULL;
    m_szdrive= NULL;
    m_szdir= NULL;
    m_szfname= NULL;
    m_szext= NULL;
}

// ***************************************************************************
BOOL CFileDir::IsMP3File() 
{ 
	if (m_filename.empty())
		return FALSE;

	Split();
	
	if (_tcsicmp(m_szext, _T(".mp3"))==0)
		return TRUE;
	if (_tcsicmp(m_szext, _T(".mp2"))==0)
		return TRUE;
	
	return FALSE;
} 
// ***************************************************************************
BOOL CFileDir::FileExists() 
{ 
	if (m_filename.empty())		
		return FALSE;
	
	// Check if file exists 
	HANDLE hFile = CreateFile((LPCTSTR)m_filename.c_str(),GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	
	if( hFile == INVALID_HANDLE_VALUE )
		return FALSE;
	
	DWORD dwSizeOfFile = ::GetFileSize( hFile, NULL );
	if (dwSizeOfFile == 0xFFFFFFFF)
		return FALSE;
	
	CloseHandle(hFile);

	return TRUE;
} 


// ***************************************************************************
tstring CFileDir::ModifyPath(TCHAR *szPath)
{
	if (szPath == NULL)
		return m_filename;
	
	string newPath = AddTerminalSlash(szPath);
	
	Split();
	m_out = newPath;
	m_out += m_szfname;   /// not use GetFilename
	m_out += m_szext;

	m_filename = m_out; // affect new full filename (path+name)

	return m_out;
}

// ***************************************************************************
tstring CFileDir::ModifyName(TCHAR *szFilename)
{
	Split();
	m_out = GetPath();
	m_out += szFilename;

	m_filename = m_out; // affect new full filename (path+ new name)

	return m_out;
}



// ***************************************************************************
tstring CFileDir::MakeName(TCHAR *szDir, TCHAR *szFilename)
{
	m_out = szDir;
	m_out += "\\";
	m_out += szFilename;
	return m_out;
}

// ***************************************************************************
BOOL CFileDir::DirectoryExists(TCHAR *szName) 
{ 
	if (szName == NULL)
		return FALSE;
	
	// Check if directory exists 
	DWORD Code = GetFileAttributes(szName); 
	return (Code != -1) && (Code & FILE_ATTRIBUTE_DIRECTORY ); 
} 

// ***************************************************************************
BOOL CFileDir::MakeFullDir(TCHAR *szDir)
{
	if (szDir == NULL)
		return FALSE;
	
	// Create full directory, if it doesn't already exist
	if( !DirectoryExists(szDir) ) 
	{
		// First make sure parent dir exists
		TCHAR *p, *path = _tcsdup(szDir);

		if( (p=_tcsrchr(path,'\\')) || (p=_tcsrchr(path,'/')) ) 
		{
			// The parent is a dir, not a drive
			*p = '\0';

			if( !MakeFullDir(path) ) 
			{ 	// Can't create parent
				free(path);
				return FALSE;
			}
		}
		free(path);

		if( !::CreateDirectory(szDir,NULL) ) 
		{
			// Cannot create directory 
			return FALSE;
		}
	}

	return(TRUE);


}


// ***************************************************************************
void CFileDir::Split()
{
	 // Initialize our objects
	if(m_szpath_buffer == NULL) 
		m_szpath_buffer= (TCHAR*) malloc(sizeof(TCHAR)*_MAX_PATH);
    if(m_szdrive== NULL)
		m_szdrive= (TCHAR*) malloc(sizeof(TCHAR)*_MAX_DRIVE);
    if(m_szdir== NULL)
		m_szdir= (TCHAR*) malloc(sizeof(TCHAR)*_MAX_DIR);
    if(m_szfname== NULL)
		m_szfname= (TCHAR*) malloc(sizeof(TCHAR)*_MAX_FNAME);
    if(m_szext== NULL)
		m_szext= (TCHAR*) malloc(sizeof(TCHAR)*_MAX_EXT);
	
	memset( m_szpath_buffer, 0, sizeof(TCHAR)*_MAX_PATH );
    memset( m_szdrive, 0, sizeof(TCHAR)*_MAX_DRIVE );
    memset( m_szdir, 0, sizeof(TCHAR)*_MAX_DIR );
    memset( m_szfname, 0, sizeof(TCHAR)*_MAX_FNAME );
    memset( m_szext, 0, sizeof(TCHAR)*_MAX_EXT );
	
	// Copy the path
    _tcscpy( m_szpath_buffer, (TCHAR*)m_filename.c_str());
    
	// Split the given path
    _tsplitpath( m_szpath_buffer, m_szdrive, m_szdir, m_szfname, m_szext );
}


// ***************************************************************************
tstring CFileDir::AddTerminalSlash(TCHAR* szPath)
{	
	if (szPath == NULL)	
		return "";

	tstring newPath = szPath;

	if (newPath.empty())
		return "";

	// add terminal slash if needed
	if( newPath[newPath.size()-1] != '\\')
		newPath += "\\";

	return newPath;
}


// ***************************************************************************
tstring CFileDir::GetDrive()
{	Split();
	m_out = m_szdrive;	
	return m_out;
}

// ***************************************************************************
tstring CFileDir::GetDirectory()
{	Split();
	m_out = m_szdir;
	return m_out;
}

// ***************************************************************************
tstring CFileDir::GetPath()
{	Split();
	m_out = m_szdrive;
	m_out += m_szdir;
	return m_out;
}

// ***************************************************************************
tstring CFileDir::GetFilename()
{	Split();
	m_out = m_szfname;
	m_out += m_szext;
	return m_out;
}

// ***************************************************************************
tstring CFileDir::GetFullFilenameNoExt()
{	Split();
	m_out = m_szdrive;
	m_out += m_szdir;
	m_out += m_szfname;
	return m_out;
}
// ***************************************************************************
tstring CFileDir::GetShortFilename()
{	Split();
	m_out = m_szfname;
	return m_out;
}

// ***************************************************************************
tstring CFileDir::GetExtension()
{	Split();
	m_out = m_szext;
	return m_out;
}

// ***************************************************************************
BOOL CFileDir::ChooseDirectoryBrowser(HWND hDlg, TCHAR *szDir, TCHAR *szTitle, BOOL bRecursive)
{
	LPMALLOC pMalloc; 

	// Get's the Shell's default allocator 
	if (SHGetMalloc(&pMalloc) != NOERROR) 
	{
		return FALSE;
	}
	else
	{ 
		BROWSEINFO bi; 
		LPITEMIDLIST pidl; 
		
		if (szDir[0] == '\0')
			GetCurrentDirectory(MAX_PATH, szDir);
	
		// Zero the BROWSEINFO structure to start off with 
		ZeroMemory( &bi, sizeof(bi) ); 
		bi.lpszTitle      = szTitle; 
		bi.hwndOwner      = hDlg;
		bi.pidlRoot       = NULL;
		bi.pszDisplayName = szDir;
		bi.ulFlags        = BIF_RETURNONLYFSDIRS; //BIF_STATUSTEXT|BIF_RETURNONLYFSDIRS; 
		if (bRecursive == TRUE)
			bi.lpfn       = BrowseRecurseCallbackProc;	// Set Callback Recurse function 
		else
			bi.lpfn       = BrowseCallbackProc;			// Set Callback function 
		bi.lParam         = (LPARAM)szDir;
		bi.iImage         = 0;

		// This next call issues the dialog box 
		if ((pidl = ::SHBrowseForFolder(&bi)) == NULL) 
		{	pMalloc->Release(); 
			return FALSE;
		}
		else
		{ 	if (::SHGetPathFromIDList(pidl, szDir)) 
			{ 
				// At this point the buffer contains the selected path 
				TCHAR *p = szDir + _tcslen(szDir) - 1;
				if (*p == '\\')	*p = '\0';
			} 
			else
			{	pMalloc->Free(pidl); 
				pMalloc->Release(); 
				return FALSE;
			}
			
			// Free the PIDL allocated by SHBrowseForFolder 
			pMalloc->Free(pidl); 
		} 
		
		// Release the shell's allocator 
		pMalloc->Release(); 
	} 

	return TRUE;

}