//  FooButton.h
//  A versatile bitmap button
//  Copyright (c) 1997-2004 Ravi Bhavnani (ravib@ravib.com)
//
//  You are allowed to include the source code in any product (commercial, shareware,
//  freeware or otherwise) when your product is released in binary form.  You are allowed
//  to modify the source code in any way you want except you cannot modify the copyright
//  details at the top of each module.  If you want to distribute source code with your
//  application, then you are only allowed to distribute versions released by the author.
//  This is to maintain a single distribution point for the source code. 

#include "stdafx.h"

#include "FooButton.h"
#include "gen.h"         // (winampGeneralPurposePlugin plugin.hDllInstance)
#include <crtdbg.h>      // _ASSERT macro
#include <commctrl.h>    // TRACKMOUSEEVENT + Comctl32.lib
#include <tchar.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#ifndef COLOR_HOTLIGHT
#define COLOR_HOTLIGHT 26
#endif

// external declaration (plugin.hDllInstance see FooButton::drawBitamp)
extern winampGeneralPurposePlugin plugin;


// Pointy finger cursor is defined on Win2000 and up
#ifndef IDC_HAND
#define IDC_HAND MAKEINTRESOURCE(32649)
#endif

#define C_BitmapBorder_X      5     // horizontal border around bitmap for text buttons
#define C_DropDown_Width      5     // width of dropdown indicator
#define C_DropDown_Height     3     // height of dropdown indicator
#define C_DropDown_X          10    // border between left edge of drop down indicator and right edge of button
#define C_DropDown_Y          8     // border between top of drop down indicator and bottom of button
#define C_BitmapCheckGap      5     // gap between right edge of bitmap and left edge of checkbox

#define C_HyperlinkColor      RGB (0, 0, 224)   // hyperlink color

//CMapStringToPtr  FooButton::m_btnGroups;  // button groups

/////////////////////////////////////////////////////////////////////////////
// FooButton

//! Standard constructor.
FooButton::FooButton(HWND hdlg, int nIDDlgItem)
{
	m_hWnd = NULL;
	m_fOldProc = NULL;

	m_hWnd = GetDlgItem(hdlg, nIDDlgItem);

	_ASSERT(m_hWnd != NULL);

	m_fOldProc = (WNDPROC)::GetWindowLong(m_hWnd,GWL_WNDPROC);
	::SetWindowLong(m_hWnd,GWL_WNDPROC,(LONG)proc);

	::SetWindowLong(m_hWnd,GWL_USERDATA,LONG(this));


	
  // Set default properties
  setType (FooButton::Type::pushButton);
  setTextStyle (FooButton::Text::singleLineCenter);
  setTextColor (GetSysColor (COLOR_BTNTEXT));
  setFocusStyle (FooButton::Focus::noFocus);
  setBitmapId (0);

  // Initialize state
  m_bChecked = false;
  m_bTracking = false;
  m_bMultiClicked = false;
  m_bDefaultButton = false;

  // Load hyperlink cursor
  m_hCursor = NULL;
  loadHyperlinkCursor();
}

//! Standard destructor.
FooButton::~FooButton()
{
	if(m_hWnd) 
	{	::SetWindowLong(m_hWnd,GWL_USERDATA,LONG(NULL));
		m_hWnd = NULL;
	}
	
	// Free cursor
  if (NULL != m_hCursor)
	 DestroyCursor (m_hCursor);
}


//////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK FooButton::proc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{

  FooButton* This = (FooButton*)::GetWindowLong(hwnd, GWL_USERDATA);
	static POINT pt;
  switch(message) 
  { 
		case WM_LBUTTONDOWN:
			{ pt.x = LOWORD(lParam);
				pt.y = HIWORD(lParam);
				This->OnLButtonDown(wParam, pt);
			}
			break;

		
		case WM_LBUTTONUP:
			{ pt.x = LOWORD(lParam);
				pt.y = HIWORD(lParam);
				This->OnLButtonUp(wParam, pt);
				// !!! very important do default action with CallWindowProc before the rest
				CallWindowProc(This->m_fOldProc,hwnd,message,wParam,lParam);
				This->m_bMultiClicked = false;
				::InvalidateRect(hwnd, NULL, TRUE);
			}
			return TRUE;;
			
		
		case WM_MOUSEMOVE:
			{ pt.x = LOWORD(lParam);
				pt.y = HIWORD(lParam);
				This->OnMouseMove(wParam, pt);
			}
			break;
			
		case WM_MOUSELEAVE: //notify
			This->OnMouseLeave(wParam, lParam);
			break;

		case WM_SETFOCUS:
			This->OnSetFocus((HWND) wParam);
			break;

		case WM_KILLFOCUS:
			This->OnKillFocus((HWND) wParam);
			break;

		case WM_SETCURSOR:
			This->OnSetCursor((HWND) wParam, LOWORD(lParam), HIWORD(lParam));  
			return TRUE;;

		case WM_ERASEBKGND:
			This->OnEraseBkgnd((HDC) wParam);
			break;
  }
  
  if(!This) return 0;
  
	// Any  messages we don't process must be passed onto the original window function
  return CallWindowProc(This->m_fOldProc,hwnd,message,wParam,lParam);
}




/*
BEGIN_MESSAGE_MAP(FooButton, CButton)
	//{{AFX_MSG_MAP(FooButton)
  ON_WM_ERASEBKGND()
  ON_WM_KILLFOCUS()
  ON_WM_LBUTTONDOWN()
  ON_WM_LBUTTONUP()
  ON_WM_MOUSEMOVE()
  ON_WM_SETCURSOR()
  ON_WM_SETFOCUS()
  ON_MESSAGE(WM_MOUSELEAVE, OnMouseLeave)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
*/
/////////////////////////////////////////////////////////////////////////////
// FooButton operations

//! Resets static storage used by the class.  This method should be called
//! when the application exits.
/* 
void FooButton::reset()
{
  POSITION pos = m_btnGroups.GetStartPosition();
  while (pos != NULL) {
    CString strGroupName;
    void* pGroup = NULL;
    m_btnGroups.GetNextAssoc (pos, strGroupName, pGroup);
    ASSERT (pGroup != NULL);
    delete (CPtrArray *) pGroup;
  }
}
*/
//! Sets the button's type.
//! @param  type    The button's type.
void FooButton::setType
  (FooButton::Type type)
{
  // Initialize button properties
  m_bDropDown = false;
  m_bMulti = false;
  m_bStatic = false;
  m_bHot = false;
  m_bHyperlink = false;

  // Set button type
  switch (type) {
    case FooButton::Type::staticButton:
      m_bStatic = true;
      break;

    case FooButton::Type::checkBox:
    case FooButton::Type::radio:
    case FooButton::Type::checkButton:
    case FooButton::Type::pushButton:
      break;

    case FooButton::Type::pushButtonDropDown:
      m_bDropDown = true;
      break;

    case FooButton::Type::hotCheckButton:
    case FooButton::Type::hotPushButton:
      m_bHot = true;
      break;

    case FooButton::Type::hotPushButtonDropDown:
      m_bHot = true;
      m_bDropDown = true;
      break;

    case FooButton::Type::pushButtonMulti:
      m_bMulti = true;
      break;

    case FooButton::Type::hotPushButtonMulti:
      m_bHot = true;
      m_bMulti = true;
      break;

    case FooButton::Type::hyperlink:
      m_bHyperlink = true;
      break;

    default:
      _ASSERT (FALSE);
      return;
  }
  m_type = type;

  // Reset checked mode
  if ((m_type != FooButton::Type::checkButton) &&
      (m_type != FooButton::Type::hotCheckButton))
     m_bChecked = false;

  if (m_hWnd != NULL)
     ::InvalidateRect(m_hWnd, NULL, TRUE);
}

//! Sets the button's text style.
//! @param  textStyle   The button's text style.
void FooButton::setTextStyle
  (FooButton::Text textStyle)
{
  // Reset text styles
  m_bText = false;
  m_bCenter = false;
  m_bMultiLine = false;

  // Set text style
  switch (textStyle) {
    case FooButton::Text::none:
      break;

    case FooButton::Text::singleLine:
      m_bText = true;
      break;

    case FooButton::Text::singleLineCenter:
      m_bText = true;
      m_bCenter = true;
      break;

    case FooButton::Text::multiLine:
      m_bText = true;
      m_bMultiLine = true;
      break;

    default:
      _ASSERT (FALSE);
      return;
  }
  m_textStyle = textStyle;
  if (m_hWnd != NULL)
     ::InvalidateRect(m_hWnd, NULL, TRUE);

}

//! Sets the button's focus style.
//! @param  focusStyle   The button's focus style.
void FooButton::setFocusStyle
  (FooButton::Focus focusStyle)
{
  m_focusStyle = focusStyle;
  if (m_hWnd != NULL)
     ::InvalidateRect(m_hWnd, NULL, TRUE);
}

//! Sets the button's bitmap id.
//! @param  nBitmapId       Bitmap id (-1 means none).
//! @param  rgbTransparent  Bitmap's transparency color (default = RGB(255,0,255)
void FooButton::setBitmapId
  (int nBitmapId,
   COLORREF rgbTransparent)
{
  m_nBitmapId = nBitmapId;
  m_rgbTransparent = rgbTransparent;
  if (m_hWnd != NULL)
     ::InvalidateRect(m_hWnd, NULL, TRUE);
}

//! Sets the button's checked state if the button type is checkButton or
//! hotCheckButton.
//! @param    bChecked    Flag: button is checked.
//! @return   The button's checked state.
void FooButton::check
  (bool bChecked)
{
  if ((m_type == FooButton::Type::checkButton) ||
      (m_type == FooButton::Type::hotCheckButton) ||
      (m_type == FooButton::Type::checkBox) ||
      (m_type == FooButton::Type::radio)) {
      m_bChecked = bChecked;
      if (m_hWnd != NULL)
		::InvalidateRect(m_hWnd, NULL, TRUE);
  }

  // If checked, inform the button's parent group (if any)
/*  if (isChecked() && !m_strButtonGroup.IsEmpty()) {
      void* pGroup;
      getButtonGroup (m_strButtonGroup, pGroup);
      ASSERT (pGroup != NULL);
      int nButtons = ((CPtrArray *) pGroup)->GetSize();
      for (int nIndex=0; (nIndex < nButtons); nIndex++) {
          FooButton* pButton = (FooButton *) ((CPtrArray *) pGroup)->GetAt (nIndex);
          ASSERT (pButton != NULL);
          if ((pButton != this) && pButton->isChecked())
             pButton->check (false);
      }
  }
*/
}

//! Sets a multi pushbutton to its normal unpressed state.
void FooButton::clearMultiClick()
{
  if ((m_type == FooButton::Type::pushButtonMulti) ||
      (m_type == FooButton::Type::hotPushButtonMulti)) {
      m_bMultiClicked = false;
      ::InvalidateRect(m_hWnd, NULL, TRUE);
  }
}

//! Displays a popup menu if the button type is pushButton, pushButtonDropDown,
//! hotPushButton or hotPushButtonDropDown.
//! @param    pMenu   The popup menu to be displayed.
//! @return   The menu selection.
int FooButton::displayPopupMenu
  (HMENU hMenu)
{
  ::InvalidateRect(m_hWnd, NULL, TRUE);
  if (hMenu != NULL) {
      RECT rectButton;
      GetWindowRect (m_hWnd, &rectButton);
      return (TrackPopupMenu(hMenu,
                TPM_LEFTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON | TPM_RETURNCMD,
                rectButton.left, 
				rectButton.bottom, 
				NULL,
				GetParent(m_hWnd),
				NULL));

  }
  return (0);
}

//! Enables/disables the button.
//! @param    bEnable   Flag: button is enabled.
void FooButton::enable
  (bool bEnable)
{
  EnableWindow (m_hWnd, bEnable ? TRUE : FALSE);
  ::InvalidateRect(m_hWnd, NULL, TRUE);
}

//! Adds the button to a named button group.
//! @param    strGroupName    Name of button group.
//! @return   true if successful, false otherwise.
/*
bool FooButton::addToGroup
  (CString strGroupName)
{
  // Group name must be specified
  if (strGroupName.IsEmpty())
     return (false);

  // Create the group if it doesn't exist
  void* pGroup = NULL;
  getButtonGroup (strGroupName, pGroup);
  if (pGroup == NULL) {
      pGroup = new CPtrArray();
      ASSERT (pGroup != NULL);
      m_btnGroups.SetAt (strGroupName, pGroup);
  }

  // Remove the button from its current group
  removeFromGroup();

  // Add button to group if not already present
  m_strButtonGroup = strGroupName;
  int nButtons = ((CPtrArray *) pGroup)->GetSize();
  for (int nIndex=0; (nIndex < nButtons); nIndex++)
      if (((CPtrArray *) pGroup)->GetAt (nIndex) == this)
         return (true);
  ((CPtrArray *) pGroup)->Add (this);

  return (true);
}

//! Removes the button from its parent group.
//! @param    pGroup    The FooButtonGroup.
//! @return   true if successful, false otherwise.
bool FooButton::removeFromGroup()
{
  // Return if not a member of a button group
  if (m_strButtonGroup.IsEmpty())
     return (false);

  // Return if group doesn't exist
  void* pGroup = NULL;
  getButtonGroup (m_strButtonGroup, pGroup);
  if (pGroup == NULL) {
      ASSERT (FALSE);
      return (false);
  }

  // Remove the button from the group
  int nButtons = ((CPtrArray *) pGroup)->GetSize();
  for (int nIndex=0; (nIndex < nButtons); nIndex++) {
      if (((CPtrArray *) pGroup)->GetAt (nIndex) == this) {
          ((CPtrArray *) pGroup)->RemoveAt (nIndex);
          m_strButtonGroup.Empty();
          return (true);
      }
  }

  // Button wasn't part of the group
  ASSERT (FALSE);
  return (false);
}

/////////////////////////////////////////////////////////////////////////////
// FooButton message handlers

void FooButton::PreSubclassWindow() 
{
  // Check if it's a default button
	if (GetStyle() & 0x0FL)
     m_bDefaultButton = true;

  // Make the button owner-drawn
	ModifyStyle (0x0FL, BS_OWNERDRAW, SWP_FRAMECHANGED);
	CButton::PreSubclassWindow();
}
*/

void FooButton::OnKillFocus(HWND hwndGetFocus)
{
   ::InvalidateRect(m_hWnd, NULL, TRUE);
  //$$CButton::OnKillFocus (pNewWnd);
}

void FooButton::OnLButtonDown (UINT nFlags, POINT point)
{
  multiHitTest (point);   // sets m_bMultiClicked
  //$$CButton::OnLButtonDown (nFlags, point);
}

void FooButton::OnLButtonUp (UINT nFlags, POINT point)
{
  // If this is a checkbutton...
  if ((m_type == FooButton::Type::checkButton) ||
      (m_type == FooButton::Type::hotCheckButton) ||
      (m_type == FooButton::Type::checkBox) ||
      (m_type == FooButton::Type::radio)) {

      // Check ourself if mouse was released over button
      RECT rectButton;
      GetClientRect (m_hWnd, &rectButton);
      if (PtInRect (&rectButton, point))
         if ((m_type != FooButton::Type::radio)) // && m_strButtonGroup.IsEmpty())
            check (!isChecked());
         else
            check (true);
  }

  // Standard handling
  //$$CButton::OnLButtonUp (nFlags, point);
  //$$m_bMultiClicked = false;
  //$$::InvalidateRect(m_hWnd, NULL, TRUE);
}

void FooButton::OnMouseMove(UINT nFlags, POINT point) 
{
  if (m_bHot && !m_bTracking) {
		TRACKMOUSEEVENT tme;
		tme.cbSize = sizeof(tme);
		tme.hwndTrack = m_hWnd;
		tme.dwFlags = TME_LEAVE;
		tme.dwHoverTime = HOVER_DEFAULT;
		m_bTracking = _TrackMouseEvent(&tme);
	  ::InvalidateRect(m_hWnd, NULL, TRUE);
  }

  // Do default action
  //$$CButton::OnMouseMove(nFlags, point);
}

LRESULT FooButton::OnMouseLeave
  (WPARAM wParam,
   LPARAM lParam)
{
  // To ignore "unreferenced formal parameter" warning
  wParam = wParam;
  lParam = lParam;

	m_bTracking = false;
	m_bMultiClicked = false;
	::InvalidateRect(m_hWnd, NULL, TRUE);
	return 0;
}

BOOL FooButton::OnEraseBkgnd(HDC hDC) 
{
RECT    rect;   // button rectangle

  // Draw gray background
  GetClientRect (m_hWnd, &rect);

  ::SetBkColor(hDC, ::GetSysColor (COLOR_BTNFACE));
  ::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &rect, NULL, 0, NULL);

  //$$return CButton::OnEraseBkgnd(pDC);
  return TRUE;
}

BOOL FooButton::OnSetCursor (HWND /* pWnd */, UINT /* nHitTest */, UINT /* message */)
{
  if (m_bHyperlink && (NULL != m_hCursor)) {
      ::SetCursor (m_hCursor);
      return TRUE;
  }
  return FALSE;
}

void FooButton::OnSetFocus(HWND hOldWnd)
{
  ::InvalidateRect(m_hWnd, NULL, TRUE);
  //$$CButton::OnSetFocus (pOldWnd);
}

void FooButton::OnDrawItem (LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
  HDC hDC = lpDrawItemStruct->hDC;
  _ASSERT (hDC != NULL);

  // Draw frame for non-radio and non-checkbox buttons
  if ((m_type != FooButton::Type::checkBox) && (m_type != FooButton::Type::radio))
     drawFrame (hDC, lpDrawItemStruct);

  // Draw button bitmap
  int nBmpRightEdge = 0;
  drawBitmap (hDC, lpDrawItemStruct, nBmpRightEdge);

  // Draw frame for radio and checkbox buttons
  if ((m_type == FooButton::Type::checkBox) || (m_type == FooButton::Type::radio))
     drawRadioCheckFrame (hDC, lpDrawItemStruct, nBmpRightEdge);

  // Draw drop-down indicator
  drawDropDown (hDC, lpDrawItemStruct);

  // Draw caption
  int nCaptionRightEdge = 0;
  drawCaption (hDC, lpDrawItemStruct, nBmpRightEdge, nCaptionRightEdge);

  // Draw focus rectangle
  drawFocus (hDC, lpDrawItemStruct, nBmpRightEdge, nCaptionRightEdge);

  // Draw default button border
  drawDefaultBorder (hDC, lpDrawItemStruct);

  return;
}

/////////////////////////////////////////////////////////////////////////////
// FooButton helper functions

//! Draws the button's frame
void FooButton::drawFrame
  (HDC hDC,
   LPDRAWITEMSTRUCT lpDrawItemStruct)
{
  _ASSERT (hDC != NULL);
  _ASSERT (lpDrawItemStruct != NULL);

  // Draw button outline if we're not a static button or a hyperlink
  if (!m_bStatic && !m_bHyperlink) {
      if (m_bHot && !m_bChecked)
          drawHotButtonFrame (lpDrawItemStruct);
      else {
          // Draw frame for non-hot button
          UINT uFrameCtrl = DFCS_BUTTONPUSH;
          if (m_bChecked)
             uFrameCtrl |= DFCS_CHECKED;
          else
             if (((lpDrawItemStruct->itemState & ODS_SELECTED) == ODS_SELECTED) && !m_bMulti)
                uFrameCtrl |= DFCS_PUSHED;
          if ((lpDrawItemStruct->itemState & ODS_DISABLED) == ODS_DISABLED)
             uFrameCtrl |= DFCS_INACTIVE;
          DrawFrameControl (hDC, &(lpDrawItemStruct->rcItem), DFC_BUTTON , uFrameCtrl);

          // For multi pushbuttons, fill the area of the drop-down indicator with
          // a light color if the multi button is currently pressed
          if (m_bMulti && m_bMultiClicked)
             drawMultiDropDownRegion (hDC, lpDrawItemStruct);
      }
  }
}

//! Draws the button's frame for a radio/checkbox button
void FooButton::drawRadioCheckFrame
  (HDC hDC,
   LPDRAWITEMSTRUCT lpDrawItemStruct,
   int& nLeftEdge)
{
  _ASSERT (hDC != NULL);
  _ASSERT (lpDrawItemStruct != NULL);

  // Nothing to do if not a radiobutton or checkbox
  if ((m_type != FooButton::Type::checkBox) && (m_type != FooButton::Type::radio))
     return;

  // Compute frame dimensions
  RECT rectFrame;
  
  CopyRect(&rectFrame, (CONST RECT *)&lpDrawItemStruct->rcItem);
  rectFrame.left = nLeftEdge;
  if (nLeftEdge > 0)
     rectFrame.left += C_BitmapCheckGap;

  int nFrameWidthHeight = GetSystemMetrics (SM_CXSIZE) - 5;
  rectFrame.right = rectFrame.left + nFrameWidthHeight;
  int nOffset = ((rectFrame.bottom - rectFrame.top)  - nFrameWidthHeight) / 2;
  rectFrame.top += nOffset;
  rectFrame.bottom = rectFrame.top + nFrameWidthHeight;
  nLeftEdge = rectFrame.right;

  // Draw frame
  UINT uFrameCtrl = (m_type == FooButton::Type::checkBox ? DFCS_BUTTONCHECK : DFCS_BUTTONRADIO);
  if (m_bChecked)
     uFrameCtrl |= DFCS_CHECKED;
  if ((lpDrawItemStruct->itemState & ODS_SELECTED) == ODS_SELECTED)
     uFrameCtrl |= DFCS_PUSHED;
  if ((lpDrawItemStruct->itemState & ODS_DISABLED) == ODS_DISABLED)
     uFrameCtrl |= DFCS_INACTIVE;
  DrawFrameControl (hDC, &rectFrame, DFC_BUTTON , uFrameCtrl);
}

//! Draws the button's bitmap
void FooButton::drawBitmap
  (HDC hDC,
   LPDRAWITEMSTRUCT lpDrawItemStruct,
   int& nBitmapRightEdge)
{
  _ASSERT (hDC != NULL);
  _ASSERT (lpDrawItemStruct != NULL);

  // Load bitmap - return if unable
  nBitmapRightEdge = 0;
  if (m_nBitmapId == 0)
     return;
  HBITMAP hBitmap = ::LoadBitmap (plugin.hDllInstance, MAKEINTRESOURCE (m_nBitmapId));
  _ASSERT (hBitmap != NULL);
  if (hBitmap == NULL)
     return;

  // Get bitmap dimensions
  BITMAP  rBitmap;
  ::GetObject (hBitmap, sizeof (BITMAP), &rBitmap);
  POINT bitmapSize;
  bitmapSize.x = rBitmap.bmWidth;
  bitmapSize.y = rBitmap.bmHeight;

  // Position the bitmap - if the button contains no text or is a checkbox/radio,
  // leave C_BitmapBorder_X pixels from the L edge.  Otherwise, center it horizontally.
  RECT rectButton;
  GetClientRect (m_hWnd, &rectButton);
  POINT pt;
  if (m_bText || (m_type == FooButton::Type::checkBox) || (m_type == FooButton::Type::radio))
     pt.x = rectButton.left + C_BitmapBorder_X;
  else
     pt.x = (rectButton.right - rBitmap.bmWidth) / 2;
  pt.y = (rectButton.bottom - rBitmap.bmHeight) / 2;
  if ((m_type != FooButton::Type::checkBox) && (m_type != FooButton::Type::radio))
     if (!m_bStatic && !m_bHyperlink && !m_bMultiClicked)
        if (lpDrawItemStruct->itemState & ODS_SELECTED)
		{
			pt.x += 1;
			pt.y += 1;
		}

  // Draw the bitmap
  if ((lpDrawItemStruct->itemState & ODS_DISABLED) == ODS_DISABLED)
      DisabledBlt (hDC,
                   pt.x, pt.y,
                   rBitmap.bmWidth, rBitmap.bmHeight,
                   hBitmap,
                   0, 0);
  else
      TransparentBlt (hDC,
                      pt.x, pt.y,
                      rBitmap.bmWidth, rBitmap.bmHeight,
                      hBitmap,
                      0, 0,
                      m_rgbTransparent,
                      NULL);

  // Return right edge of bitmap
  nBitmapRightEdge = pt.x + rBitmap.bmWidth;

  // Free bitmap
  ::DeleteObject (hBitmap);
}

//! Draws the button's caption
void FooButton::drawCaption
  (HDC hDC,
   LPDRAWITEMSTRUCT lpDrawItemStruct,
   int nLeftEdge,
   int& nRightEdge)
{
  // Nothing to do if the isn't set up to display text
  nRightEdge = 0;
  if (!m_bText)
     return;

  // Tweak edges
  if (!m_bCenter)
     nLeftEdge += C_BitmapBorder_X; // looks nicer
  RECT rectButton;
  GetClientRect (m_hWnd, &rectButton);
  nRightEdge = rectButton.right;
  if (m_bDropDown)
     nRightEdge -= 6;
  else
     if (m_bMulti) {
        // bool bBitmap = (m_nBitmapId != 0);
        // nRightEdge -= (bBitmap ? 12 : 10);
        nRightEdge -= 12;
     }

  // Get available caption width - return if none
  int nAvailableWidth = nRightEdge - nLeftEdge;
  if (nAvailableWidth <= 0)
     return;

  // Get caption text
  TCHAR *strCaption;
  int nLen = GetWindowTextLength(m_hWnd);
  strCaption = (TCHAR*)GlobalAlloc(GPTR, nLen+1);
  GetWindowText(m_hWnd, strCaption, nLen+1); 
  

  // Determine drawing format
  DWORD  dwFormat = DT_SINGLELINE | DT_VCENTER | DT_END_ELLIPSIS;
  dwFormat |= (m_bCenter ? DT_CENTER : DT_LEFT);
  if (m_bMultiLine)
     dwFormat = DT_WORDBREAK | DT_LEFT | DT_WORD_ELLIPSIS | DT_VCENTER;

  // Determine dimensions of caption
  RECT rectCaption;
  GetClientRect (m_hWnd, &rectCaption);
  int nHeight = ::DrawText (hDC, strCaption,
                            nLen,
                            &rectCaption,
                            dwFormat | DT_CALCRECT);
  
 
  int nWidth = rectCaption.right - rectCaption.left;

  // Position caption horizontally within available area
  rectCaption.left = nLeftEdge;
  rectCaption.right = nRightEdge;
  if (m_bCenter && (nAvailableWidth > nWidth)) {
      int nOffsetX = (nAvailableWidth - nWidth) / 2;
      rectCaption.left += nOffsetX;
      rectCaption.right -= nOffsetX;
  }

  // Position caption vertically within button
  int nOffsetY = ((rectButton.bottom - rectButton.top) - nHeight)/2;
  rectCaption.top += nOffsetY;
  rectCaption.bottom = rectCaption.top + nHeight;

  // Offset caption if button is pressed
  if ((m_type != FooButton::Type::checkBox) && (m_type != FooButton::Type::radio))
     if (!m_bStatic && !m_bHyperlink && !m_bMultiClicked)
        if (lpDrawItemStruct->itemState & ODS_SELECTED)
           OffsetRect (&rectCaption, 1, 1);

  // Set text color
  SetBkMode (hDC, TRANSPARENT);
  SetBkColor (hDC, ::GetSysColor (COLOR_BTNFACE));
  SetTextColor (hDC, m_bHyperlink ? C_HyperlinkColor : m_rgbText);


  // Set underline mode if hyperlink
  HFONT hOldFont = NULL;
  HFONT hUnderlineFont;
  if (m_bHyperlink) {
      LOGFONT logFont;
      HFONT hFontDlg = (HFONT)::SendMessage(m_hWnd, WM_GETFONT, 0, 0);
			::GetObject(hFontDlg, sizeof(LOGFONT), &logFont); 

      logFont.lfUnderline = TRUE;
      hUnderlineFont  = CreateFontIndirect (&logFont);
      hOldFont = (HFONT)SelectObject (hDC, hUnderlineFont);
  }

  // Draw caption
  if ((lpDrawItemStruct->itemState & ODS_DISABLED) == ODS_DISABLED)
     {
       ::OffsetRect (&rectCaption, 1, 1);
       ::SetTextColor (hDC, ::GetSysColor (COLOR_3DHILIGHT));
       ::DrawTextEx (hDC, strCaption, nLen,
                     &rectCaption,
                     dwFormat,
                     NULL);
  
       ::OffsetRect (&rectCaption, -1, -1);
       ::SetTextColor (hDC, ::GetSysColor (COLOR_GRAYTEXT));
       ::DrawTextEx (hDC, strCaption, nLen,
                     &rectCaption,
                     dwFormat,
                     NULL);
     }
  else
     ::DrawTextEx (hDC, strCaption, nLen,
                   &rectCaption,
                   dwFormat,
                   NULL);

  GlobalFree(strCaption);

  // Free underline font
  if (hOldFont != NULL)
     SelectObject (hDC, hOldFont);
	
	if (hUnderlineFont != NULL)
		DeleteObject(hUnderlineFont);
}

//! Draws the button's drop-down indicator
void FooButton::drawDropDown
  (HDC hDC,
   LPDRAWITEMSTRUCT lpDrawItemStruct)
{
  _ASSERT (hDC != NULL);
  _ASSERT (lpDrawItemStruct != NULL);

  // Drop-down indicator is drawn for drop-down or multi buttons only
  RECT rectButton;
  GetClientRect (m_hWnd, &rectButton);
  int nDropDownLeftEdge = rectButton.right;
  if (!m_bDropDown && !m_bMulti)
     return;

  // Create pen of appropriate color
  COLORREF   rgbPen;
  if (m_bHyperlink)
      rgbPen = GetSysColor (COLOR_HOTLIGHT);
  else {
      rgbPen = GetSysColor (COLOR_BTNTEXT);
      if ((lpDrawItemStruct->itemState & ODS_DISABLED) == ODS_DISABLED)
         rgbPen = GetSysColor (COLOR_GRAYTEXT);
  }
  HPEN hPen = CreatePen (PS_SOLID | PS_INSIDEFRAME, 1, rgbPen);

  // Set X and Y offsets
  int nOffsetX = 0;
  if ((lpDrawItemStruct->itemState & ODS_SELECTED) || m_bMultiClicked)
     nOffsetX = 1;
  int nOffsetY = nOffsetX;

  // Draw the indicator
  HPEN hOldPen = (HPEN)SelectObject (hDC, hPen);
  nDropDownLeftEdge = rectButton.right - C_DropDown_X + nOffsetX;
  drawDropDownIndicator (hDC,
                         nDropDownLeftEdge,
                         rectButton.bottom - C_DropDown_Y + nOffsetY,
                         C_DropDown_Width, C_DropDown_Height);
  SelectObject (hDC, hOldPen);
  nDropDownLeftEdge = rectButton.right - (3 * C_DropDown_X)/2;

  // Draw the vertical separator for multi buttons
  if (m_bMulti && !m_bMultiClicked) {
      rectButton.top += 3;
      rectButton.bottom -= 3;
      rectButton.left = nDropDownLeftEdge;
      rectButton.right = rectButton.left + 2;
      DrawEdge (hDC, &rectButton, BDR_SUNKENOUTER, BF_RECT);
  }

	DeleteObject(hPen);
}

//! Draws a multi-button's pressed drop-down region
void FooButton::drawMultiDropDownRegion
  (HDC hDC,
   LPDRAWITEMSTRUCT lpDrawItemStruct)
{
  _ASSERT (hDC != NULL);
  _ASSERT (lpDrawItemStruct != NULL);
  if (!m_bMulti || !m_bMultiClicked)
     return;

  // Get bounds of multi part of button
	RECT rect;
	CopyRect(&rect, (CONST RECT *)&lpDrawItemStruct->rcItem);
	rect.left = rect.right - (3 * C_DropDown_X)/2 + 1;
	rect.bottom--;

  // Fill area
  UINT uFrameCtrl = DFCS_BUTTONPUSH | DFCS_CHECKED;
  DrawFrameControl (hDC, &rect, DFC_BUTTON , uFrameCtrl);
}

//! Draws the button's focus rectangle
void FooButton::drawFocus
  (HDC hDC,
   LPDRAWITEMSTRUCT lpDrawItemStruct,
   int nLeftEdge,
   int nRightEdge)
{
  _ASSERT (hDC != NULL);
  _ASSERT (lpDrawItemStruct != NULL);

  // Draw focus rectangle (for non-static non-hot buttons)
  if (!m_bStatic && !m_bHot && (getFocusStyle() != FooButton::Focus::noFocus))
     if ((lpDrawItemStruct->itemState & ODS_FOCUS) == ODS_FOCUS) {
        RECT rectButton;
        GetClientRect (m_hWnd, &rectButton);
        rectButton.left = nLeftEdge + 4;
        rectButton.right = nRightEdge - 4;
        rectButton.top += 4;
        rectButton.bottom -= 4;
        ::DrawFocusRect (hDC, &rectButton);
     }
}

//! Draws the border for a default button
void FooButton::drawDefaultBorder
  (HDC hDC,
   LPDRAWITEMSTRUCT lpDrawItemStruct)
{
  _ASSERT (hDC != NULL);
  _ASSERT (lpDrawItemStruct != NULL);

  // Only pushbuttons can have a default border
  FooButton::Type btnType = getType();
  if ((btnType != FooButton::Type::pushButton) &&
      (btnType != FooButton::Type::pushButtonDropDown) &&
      (btnType != FooButton::Type::pushButtonMulti))
     return;

  // Nothing to do if the default border isn't required
  if (getFocusStyle() != FooButton::Focus::defaultFocus)
     return;

  // Nothing do if the button doesn't have focus or isn't the default button
  if (((lpDrawItemStruct->itemState & ODS_FOCUS) == 0) || !m_bDefaultButton)
     return;

  COLORREF rgbBorder = GetSysColor (COLOR_3DDKSHADOW);
  HBRUSH hBorderBrush = CreateSolidBrush(rgbBorder);

  RECT innerRect;
  CopyRect(&innerRect,  (CONST RECT*)&lpDrawItemStruct->rcItem);
  if ((lpDrawItemStruct->itemState & ODS_SELECTED) == 0) {

		COLORREF rgbBorder = GetSysColor (COLOR_3DDKSHADOW);
		HPEN hBorderGreyPen = CreatePen(PS_SOLID, 1, rgbBorder);
		HPEN hOldPen = (HPEN)SelectObject (hDC, hBorderGreyPen );

		MoveToEx (hDC, innerRect.right - 2, innerRect.top + 1, NULL);
		LineTo (hDC, innerRect.right - 2, innerRect.bottom - 2);
		LineTo (hDC, innerRect.left, innerRect.bottom - 2);

		COLORREF rgbBrGrey = GetSysColor (COLOR_3DHILIGHT);
		HPEN hBrightGreyPen = CreatePen(PS_SOLID, 1, rgbBrGrey);
		hOldPen = (HPEN)SelectObject (hDC, hBrightGreyPen);

		MoveToEx (hDC, innerRect.right - 3, innerRect.top + 1, NULL);
		LineTo (hDC, innerRect.left + 1, innerRect.top + 1);
		LineTo (hDC, innerRect.left + 1, innerRect.bottom - 2);

		COLORREF rgbDkGrey = GetSysColor (COLOR_BTNSHADOW);
		HPEN hDarkGreyPen = CreatePen(PS_SOLID, 1, rgbDkGrey);
		hOldPen = (HPEN)SelectObject (hDC, hDarkGreyPen);
		MoveToEx (hDC, innerRect.right - 3, innerRect.top + 2, NULL);
		LineTo (hDC, innerRect.right - 3, innerRect.bottom - 3);
		LineTo (hDC, innerRect.left + 1, innerRect.bottom - 3);
		SelectObject (hDC, hOldPen);

		DeleteObject(hBorderGreyPen);
		DeleteObject(hBrightGreyPen);
		DeleteObject(hDarkGreyPen);


  } else {
      
	  innerRect.left -= 1;
	  innerRect.top -= 1;
	  innerRect.right += -1;
	  innerRect.bottom -= -1;

	  FrameRect (hDC, &innerRect, hBorderBrush);
  }
  FrameRect(hDC, &lpDrawItemStruct->rcItem, hBorderBrush);

  DeleteObject(hBorderBrush);
}

//////////////////////////////////////////////////////////////////////////////
//
//  Purpose:
//    Copies a bitmap transparently onto the destination DC.
//    See http://www.codeguru.com/Cpp/G-M/bitmap/specialeffects/article.php/c1749/
//
//  Returns:
//    None
//
void FooButton::TransparentBlt
  (HDC      hdcDest,            // destination device context
   int      nXDest,             // x-coord of destination rectangle's upper-left corner
   int      nYDest,             // y-coord of destination rectangle's upper-left corner
   int      nWidth,             // width of destination rectangle
   int      nHeight,            // height of destination rectangle
   HBITMAP  hBitmap,            // source bitmap
   int      nXSrc,              // x-coord of source rectangle's upper-left corner
   int      nYSrc,              // y-coord of source rectangle's upper-left corner
   COLORREF colorTransparent,   // transparent color
   HPALETTE hPal)               // logical palette to be used with bitmap (may be NULL)
{
  HDC hdc, hmemDC, hmaskDC, htempDC;
  hdc = hdcDest ;
  hmaskDC = CreateCompatibleDC(hdc);
  HBITMAP hmaskBitmap;

  // These store return of SelectObject() calls
  HBITMAP hOldMemBmp = NULL;
  HBITMAP hOldMaskBmp = NULL;
  HBITMAP hOldTempBmp = NULL;

  hmemDC = CreateCompatibleDC (hdc);
  htempDC = CreateCompatibleDC (hdc);
  HBITMAP hbmpImage;
  hbmpImage = CreateCompatibleBitmap (hdc, nWidth, nHeight);
  hOldMemBmp = (HBITMAP)SelectObject (hmemDC, hbmpImage);

  // Select and realize the palette
  if (GetDeviceCaps (hdc, RASTERCAPS) & RC_PALETTE && hPal) {
     ::SelectPalette( hdc, hPal, FALSE );
     ::RealizePalette(hdc);
     ::SelectPalette( hmemDC, hPal, FALSE );
  }

  hOldTempBmp = (HBITMAP) ::SelectObject (htempDC, hBitmap);
  BitBlt (hmemDC,0, 0, nWidth, nHeight, htempDC, nXSrc, nYSrc, SRCCOPY);

  // Create monochrome bitmap for the mask
  hmaskBitmap  = CreateBitmap (nWidth, nHeight, 1, 1, NULL);
  hOldMaskBmp = (HBITMAP)SelectObject (hmaskDC, hmaskBitmap);
  SetBkColor (hmemDC, colorTransparent);

  // Create the mask from the memory DC
  BitBlt (hmaskDC, 0, 0, nWidth, nHeight, hmemDC, 0, 0, SRCCOPY);

  // Set the background in memDC to black. Using SRCPAINT with black
  // and any other color results in the other color, thus making
  // black the transparent color
  SetBkColor (hmemDC, RGB (0,0,0));
  SetTextColor (hmemDC, RGB (255,255,255));
  BitBlt (hmemDC, 0, 0, nWidth, nHeight, hmaskDC, 0, 0, SRCAND);

  // Set the foreground to black. See comment above.
  SetBkColor (hdc, RGB (255,255,255));
  SetTextColor (hdc, RGB (0,0,0));
  BitBlt (hdc, nXDest, nYDest, nWidth, nHeight, hmaskDC, 0, 0, SRCAND);

  // Combine the foreground with the background
  BitBlt (hdc, nXDest, nYDest, nWidth, nHeight, hmemDC, 0, 0, SRCPAINT);

  if (hOldTempBmp)
     ::SelectObject (htempDC, hOldTempBmp);
  if (hOldMaskBmp)
     SelectObject (hmaskDC, hOldMaskBmp);
  if (hOldMemBmp)
     SelectObject (hmemDC, hOldMemBmp);

	DeleteDC(hmemDC);
	DeleteDC(htempDC);
	DeleteObject(hbmpImage);
	DeleteObject(hmaskBitmap);

}


//////////////////////////////////////////////////////////////////////////////
//
//  Purpose:
//    Draws a bitmap in a disabled mode onto the destination DC.
//    See http://www.codeguru.com/Cpp/G-M/bitmap/specialeffects/article.php/c1699/
//
//  Returns:
//    None
//
void FooButton::DisabledBlt
  (HDC      hdcDest,    // destination DC
   int      nXDest,     // x coord of destination rectangle's upper left corner
   int      nYDest,     // y coord of destination rectangle's upper left corner
   int      nWidth,     // width of destination rectangle
   int      nHeight,    // height of destination rectangle
   HBITMAP  hBitmap,    // source bitmap
   int      nXSrc,      // x-coord of source rectangle's upper-left corner
   int      nYSrc)      // y-coord of source rectangle's upper-left corner
{
  // Create a generic DC for all BitBlts
  HDC hDC = CreateCompatibleDC (hdcDest);
 
  _ASSERT(hDC != NULL);

  if (hDC) {

       // Create a DC for the monochrome DIB section
       HDC bwDC = CreateCompatibleDC(hDC);
 
       if (bwDC) {

            // Create the monochrome DIB section with a black and white palette
            struct
            {
              BITMAPINFOHEADER  bmiHeader;
              RGBQUAD           bmiColors[2];
            } RGBBWBITMAPINFO = {
                                  {                               // a BITMAPINFOHEADER
                                    sizeof(BITMAPINFOHEADER),     // biSize
                                    nWidth,                       // biWidth;
                                    nHeight,                      // biHeight;
                                    1,                            // biPlanes;
                                    1,                            // biBitCount
                                    BI_RGB,                       // biCompression;
                                    0,                            // biSizeImage;
                                    0,                            // biXPelsPerMeter;
                                    0,                            // biYPelsPerMeter;
                                    0,                            // biClrUsed;
                                    0                             // biClrImportant;
                                  },
                                  {
                                    { 0x00, 0x00, 0x00, 0x00 },
                                    { 0xFF, 0xFF, 0xFF, 0x00 }
                                  }
                                };
            void*     pbitsBW;
            HBITMAP   hBitmapBW = CreateDIBSection (bwDC,
                                                    (LPBITMAPINFO) &RGBBWBITMAPINFO,
                                                    DIB_RGB_COLORS, &pbitsBW, NULL, 0);
 
            if (hBitmapBW) {
                 // Attach the monochrome DIB section and the bitmap to the DCs
                 SelectObject (bwDC, hBitmapBW);
                 SelectObject (hDC, hBitmap);

                 // BitBlt the bitmap into the monochrome DIB section
                 BitBlt (bwDC, 0, 0, nWidth, nHeight, hDC, nXSrc, nYSrc, SRCCOPY);

                 // Paint the destination rectangle in gray
				RECT rect; 
				SetRect(&rect, nXDest, nYDest, nXDest + nWidth, nYDest + nHeight);

                 FillRect (hdcDest,
                           &rect,
                           GetSysColorBrush (COLOR_3DFACE));

                 // BitBlt the black bits in the monochrome bitmap into COLOR_3DHILIGHT bits in the destination DC
                 // The magic ROP comes from the Charles Petzold's book
                 HBRUSH hb = CreateSolidBrush (GetSysColor (COLOR_3DHILIGHT));
                 HBRUSH oldBrush = (HBRUSH) SelectObject (hdcDest, hb);
                 BitBlt (hdcDest, nXDest + 1, nYDest + 1, nWidth, nHeight, bwDC, 0, 0, 0xB8074A);

                 // BitBlt the black bits in the monochrome bitmap into COLOR_3DSHADOW bits in the destination DC
                 hb = CreateSolidBrush (GetSysColor (COLOR_3DSHADOW));
                 DeleteObject (SelectObject(hdcDest, hb));
                 BitBlt (hdcDest, nXDest, nYDest, nWidth, nHeight, bwDC, 0, 0, 0xB8074A);
                 DeleteObject (SelectObject (hdcDest, oldBrush));
            }
            DeleteDC (bwDC);
       }
       DeleteDC(hDC);
  }
}

//
//  Purpose:
//    Draws a dropdown indicator
//
//  Returns:
//    None
//
void FooButton::drawDropDownIndicator
  (HDC      hdcDest,            // destination DC
   int      nX,                 // x coord of dropdown indicator's upper left vertex
   int      nY,                 // y coord of dropdown indicator's upper left vertex
   int      nWidthDropDown,     // width of indicator
   int      nHeightDropDown)    // height of indicator
{
int   nCounter;   // generic counter

  _ASSERT (hdcDest);
  for (nCounter = 0; (nCounter < nHeightDropDown); nCounter++) {

      POINT   ptStart;    // starting point
      int     nWidth;     // line width

      ptStart.x = nX + nCounter;
      ptStart.y = nY + nCounter;
      nWidth = nWidthDropDown - nCounter;
      if (nWidth < 0)
         break;

      ::MoveToEx (hdcDest, ptStart.x, ptStart.y, NULL);
      ::LineTo (hdcDest, ptStart.x + nWidth - nCounter, ptStart.y);
  }
}

//! Draws the button's frame when hot tracking.
void FooButton::drawHotButtonFrame
  (LPDRAWITEMSTRUCT lpDrawItemStruct)
{
  // Get DC
  _ASSERT (lpDrawItemStruct != NULL);
  HDC hDC = lpDrawItemStruct->hDC;
  _ASSERT (hDC != NULL);

  // Draw frame
  RECT rectButton;
  GetClientRect (m_hWnd, &rectButton);
  if (m_bTracking) {
      COLORREF  rgbHighlight = GetSysColor (COLOR_BTNHIGHLIGHT);
      COLORREF  rgbShadow = GetSysColor (COLOR_BTNSHADOW);
      COLORREF  rgbBackground = GetSysColor (m_bChecked ? COLOR_3DLIGHT : COLOR_BTNFACE);

      FillSolidRect (hDC, &rectButton, rgbBackground);
      if ((lpDrawItemStruct->itemState & ODS_SELECTED) == ODS_SELECTED)
         Draw3dRect (hDC, &rectButton, rgbShadow, rgbHighlight);
      else
         Draw3dRect (hDC, &rectButton, rgbHighlight, rgbShadow);

      // For multi pushbuttons, fill the area of the drop-down indicator with
      // a light color if the multi button is currently pressed
      if (m_bMulti && m_bMultiClicked)
         drawMultiDropDownRegion (hDC, lpDrawItemStruct);
  }
}

//! Loads the default hyperlink cursor if possible.
void FooButton::loadHyperlinkCursor()
{
  // The following appeared in Paul DiLascia's Jan 1998 MSJ articles.
  // It loads a "hand" cursor from the winhlp32.exe module
  // (Lifted from Chris Maunder's CHyperLink class)

  // Get the windows directory
  TCHAR strWndDir[MAX_PATH];
  GetWindowsDirectory(strWndDir, MAX_PATH);
  
  _tcscat(strWndDir, _T("\\winhlp32.exe"));

  // This retrieves cursor #106 from winhlp32.exe, which is a hand pointer
  HMODULE hModule = LoadLibrary (strWndDir);
  if (hModule != NULL) {
      HCURSOR hHandCursor = ::LoadCursor(hModule, MAKEINTRESOURCE(106));
      if (hHandCursor != NULL)
          m_hCursor = CopyCursor (hHandCursor);
  }
  FreeLibrary(hModule);
}

//! Checks if the mouse was clicked on multi part of button.
void FooButton::multiHitTest
  (POINT pt)
{
  // Reset multi-clicked state
  m_bMultiClicked = false;
  if (!m_bMulti)
     return;

  // Get bounds of multi part of button
  RECT rect;
  GetWindowRect (m_hWnd, &rect);
  rect.left = rect.right - 2*C_DropDown_X + 2;

  // Check if mouse was clicked on multi part of button
  ClientToScreen (m_hWnd, &pt);
  if (PtInRect (&rect, pt))
     m_bMultiClicked = true;
}

//! Gets a button group by name
//! @param  strGroupName    Group name
//! @param  pGroup          Returned group
/*$$
void FooButton::getButtonGroup
  (CString strGroupName,
   void* & pGroup)
{
  pGroup = NULL;
  m_btnGroups.Lookup (strGroupName, pGroup);
}
*/



//////////////////////////////////////////////////////////////////////////////
void FooButton::FillSolidRectXY(HDC hdc, int x, int y, int cx, int cy, COLORREF clr)
{	RECT rect;
	SetRect(&rect, x, y, x+ cx, y +cy);
	::SetBkColor(hdc, clr);
	::ExtTextOut(hdc, 0, 0, ETO_OPAQUE, &rect, NULL, 0, NULL);
}
void FooButton::FillSolidRect(HDC hdc, LPCRECT lpRect, COLORREF clr)
{	::SetBkColor(hdc, clr);
	::ExtTextOut(hdc, 0, 0, ETO_OPAQUE, lpRect, NULL, 0, NULL);
}
//////////////////////////////////////////////////////////////////////////////
void FooButton::Draw3dRectXY(HDC hdc, int x, int y, int cx, int cy,	COLORREF clrTopLeft, COLORREF clrBottomRight)
{	FillSolidRectXY(hdc, x, y, cx - 1, 1, clrTopLeft);
	FillSolidRectXY(hdc, x, y, 1, cy - 1, clrTopLeft);
	FillSolidRectXY(hdc, x + cx, y, -1, cy, clrBottomRight);
	FillSolidRectXY(hdc, x, y + cy, cx, -1, clrBottomRight);
}
void FooButton::Draw3dRect(HDC hdc, LPRECT lpRect,	COLORREF clrTopLeft, COLORREF clrBottomRight)
{	Draw3dRectXY(hdc, lpRect->left, lpRect->top, lpRect->right - lpRect->left,
		lpRect->bottom - lpRect->top, clrTopLeft, clrBottomRight);
}



// End FooButton.cpp
