// Systray.cpp: implementation of the CSystray class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <commctrl.h>
#include <shellapi.h>		// NOTIFYICONDATA

#include "Systray.h"

#include "gen.h"
#include "resource.h"
#include "definition.h"
#include "winamp2.h"
#include "Util.h"
#include "Language.h"
#include "settings.h"
#include "DlgPlaylist.h"
#include "DlgSystray.h"

/*
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

*/

// -----------------------------
// Global declaration
CWinamp *amp = NULL;
CLanguage *lng = NULL;
CSetting *setting = NULL;
INITSTRUCT gParam;


// -----------------------------
// External declaration main.cpp
extern winampGeneralPurposePlugin plugin;
extern int systray_enabled;
extern CSystray *systray;

void config();



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
// ************************************************************************
CSystray::CSystray()
{
	
	// Wrapper class function Winamp
	amp = new CWinamp();
	amp->FindWinamp();

	// read parameters in 'C:\program files\Winamp\plugins\gen_EZPlaylist.ini' file
	setting = new CSetting;
	if (setting)
		setting->Init();

	// Initialize multi-language
	lng = new CLanguage;
	if(lng)
		lng->Start(gParam.szLanguage);

	// Initialize & Draw Icons in traskbar systray
	m_tips[0]= GS(LNG_TIPS_EXTPLAYLIST);
	m_tips[1]= GS(LNG_TIPS_PREVIOUSTRACK);
	m_tips[2]= GS(LNG_TIPS_PLAYPAUSE);
	m_tips[3]= GS(LNG_TIPS_STOP);
	m_tips[4]= GS(LNG_TIPS_NEXTTRACK);
	m_tips[5]= GS(LNG_TIPS_LOADFILE);
	m_tips[6]= GS(LNG_TIPS_VOLUME);
	m_tips[7]= GS(LNG_TIPS_EZMENU);
	
	for (int i = 0; i < NUM_ICONSYSTRAY; i ++)
		m_Icons[i] = LoadIcon(plugin.hDllInstance,MAKEINTRESOURCE(IDI_ICON0+i));
	
	IconUpdate();


	m_DlgMain = (CDlgPlaylist*)new(CDlgPlaylist);

	// ///////////////////////////////////////////////
	// Hook procedure SysTrayWndProc attach by Winamp 
	WNDPROC lpOldWndProc = (WNDPROC)GetWindowLong(plugin.hwndParent,GWL_WNDPROC);

	SetWindowLong(plugin.hwndParent, GWL_WNDPROC,  (LONG)this->SysTrayWndProc);
	SetWindowLong(plugin.hwndParent, GWL_USERDATA, (LONG)lpOldWndProc);

}
// ************************************************************************
CSystray::~CSystray()
{
	// delete all icons in taskbar
	systray_enabled=0;
	IconUpdate();

	// clean up Dialogbox EZ-playlist
	ezplaylist_destroy();
	if (m_DlgMain)


	// Save parameters
	if (setting != NULL)
	{
		setting->Save();
		delete setting;
	}

	if (amp != NULL)
		delete amp;

	if (lng != NULL)
		delete lng;


}



// ************************************************************************
void CSystray::IconUpdate()
{
	static int x=0;
	int i;
	if (x == systray_enabled) return;

	for (i = NUM_ICONSYSTRAY-1; i >= 0; i --)
	{
		if (x & (1<<i))
			IconDel(plugin.hwndParent,i);
	}
	
	x = systray_enabled;	
	for (i = 0; i < NUM_ICONSYSTRAY; i ++)
	{
		if (systray_enabled & (1<<i))
			IconAdd(plugin.hwndParent,i,m_Icons[i],m_tips[i]);
	}
}

// ****************************************************************
BOOL CSystray::IconAdd(HWND hwnd, UINT uID, HICON hIcon, TCHAR* lpszTip)
{
  NOTIFYICONDATA tnid;

  ZeroMemory(&tnid, sizeof(tnid));

  tnid.cbSize = sizeof(NOTIFYICONDATA);
  tnid.hWnd   = hwnd;
  tnid.uID    = uID+SYSTRAY_ICON_BASE;
  tnid.uFlags = NIF_ICON | NIF_TIP | NIF_MESSAGE;
  tnid.hIcon  = hIcon;
  tnid.uCallbackMessage = WM_USER + 27;
  _tcsncpy(tnid.szTip,lpszTip,sizeof(tnid.szTip)-1);
  
  return (Shell_NotifyIcon(NIM_ADD, &tnid));
}

// ****************************************************************
BOOL CSystray::IconDel(HWND hwnd, UINT uID) 
{
  NOTIFYICONDATA tnid;
  
  ZeroMemory(&tnid, sizeof(tnid));

  tnid.cbSize = sizeof(NOTIFYICONDATA);
  tnid.hWnd   = hwnd;
  tnid.uID    = uID+SYSTRAY_ICON_BASE;
  return(Shell_NotifyIcon(NIM_DELETE, &tnid));
}

// ******************************************************************
// Draw onwer menu with icon
//
#define N_BORDER_SIZE	2
#define N_BITMAP_SIZE	16

BOOL CSystray::OnOwnerDrawMenu_MeasureItem(LPMEASUREITEMSTRUCT lpMeasureItemStruct, int *m_nFontHeight, HFONT *m_menuFont)
{
	if (lpMeasureItemStruct->CtlType != ODT_MENU)
		return FALSE;

	TCHAR  str[] = _T("Very Long String");
	HDC hDC = GetWindowDC(NULL);

	if (!(HFONT)m_menuFont) 
	{
		NONCLIENTMETRICS info;
		info.cbSize = sizeof(info);
		SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(info), &info, 0);
		*m_menuFont = CreateFontIndirect(&info.lfMenuFont);
	}
	SelectObject (hDC, m_menuFont);

	SIZE size;
	::GetTextExtentPoint32(hDC, str, _tcslen(str), &size);
	ReleaseDC(NULL, hDC);

	lpMeasureItemStruct->itemHeight = N_BORDER_SIZE + ((size.cy > N_BITMAP_SIZE) ? size.cy : N_BITMAP_SIZE) + N_BORDER_SIZE;    
	lpMeasureItemStruct->itemWidth  = 2*N_BORDER_SIZE + N_BITMAP_SIZE + 2*N_BORDER_SIZE + size.cx + N_BORDER_SIZE;    
	*m_nFontHeight = size.cy;
	return TRUE;
}
// ******************************************************************
BOOL CSystray::OnOwnerDrawMenu_DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct,int m_nFontHeight, HFONT m_menuFont)
{
	if (lpDrawItemStruct->CtlType != ODT_MENU)
		return FALSE;

	HDC hDC = lpDrawItemStruct->hDC;
	LPCTSTR text   = (LPCTSTR) lpDrawItemStruct->itemData;
	BOOL bSelected = (lpDrawItemStruct->itemState) & ODS_SELECTED;
	BOOL bGrayed   = (lpDrawItemStruct->itemState) & ODS_GRAYED;
	BOOL bChecked  = (lpDrawItemStruct->itemState) & ODS_CHECKED;

	COLORREF crBack;
	crBack = bSelected ? ::GetSysColor (COLOR_HIGHLIGHT) : ::GetSysColor (COLOR_MENU);
	SetBkColor (hDC,crBack);
	RECT rect;
	rect = lpDrawItemStruct->rcItem;
	rect.left  += N_BORDER_SIZE + 2*N_BORDER_SIZE + N_BITMAP_SIZE;
	rect.right += N_BORDER_SIZE + 2*N_BORDER_SIZE + N_BITMAP_SIZE;
	::SetBkColor(hDC, crBack);
	::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &rect, NULL, 0, NULL);
	POINT ptText;
	ptText.x = lpDrawItemStruct->rcItem.left + 2*N_BORDER_SIZE + N_BITMAP_SIZE + 2*N_BORDER_SIZE; 
	ptText.y = lpDrawItemStruct->rcItem.top  + (lpDrawItemStruct->rcItem.bottom - lpDrawItemStruct->rcItem.top - m_nFontHeight)/2 - 1;

	SetBkMode(hDC, TRANSPARENT);
	int nLen = _tcslen(text);

	if (!bGrayed)
	{	if (bSelected)
		{	SetTextColor (hDC, ::GetSysColor (COLOR_HIGHLIGHTTEXT));
			TextOut (hDC, ptText.x, ptText.y, text, nLen);
		}
		else
		{	SetTextColor (hDC, ::GetSysColor (COLOR_MENUTEXT));
			TextOut (hDC, ptText.x, ptText.y, text, nLen);
		}
	}
	else
	{	if (bSelected)
		{	SetTextColor (hDC, GetSysColor(COLOR_MENU));
			TextOut (hDC, ptText.x, ptText.y, text, nLen);
		}
		else
		{	SetTextColor(hDC, GetSysColor(COLOR_HIGHLIGHTTEXT));
			TextOut (hDC, ptText.x + 1, ptText.y + 1, text, nLen);

			SetTextColor (hDC, GetSysColor(COLOR_GRAYTEXT));
			TextOut (hDC, ptText.x, ptText.y, text, nLen);
		}
	}

	POINT pt;
	pt.x = lpDrawItemStruct->rcItem.left + 2*N_BORDER_SIZE;
	pt.y = lpDrawItemStruct->rcItem.top + (lpDrawItemStruct->rcItem.bottom - lpDrawItemStruct->rcItem.top - N_BITMAP_SIZE)/2;

	UINT uIcon = IDI_EMPTY;

	switch(lpDrawItemStruct->itemID)
	{
		case IDM_SYSTRAY_EXTPLAYLIST:   uIcon = IDI_ICON0 ; break;
		case IDM_SYSTRAY_PREVIOUSTRACK: uIcon = IDI_ICON1 ; break;
		case IDM_SYSTRAY_PLAYPAUSE:     uIcon = IDI_ICON2 ; break;
		case IDM_SYSTRAY_STOP:          uIcon = IDI_ICON3 ; break;
		case IDM_SYSTRAY_NEXTTRACK:     uIcon = IDI_ICON4 ; break;
		case IDM_SYSTRAY_LOADFILE:      uIcon = IDI_ICON5 ; break;
	}
	HICON hIcon = NULL;
	
	hIcon = LoadIcon(plugin.hDllInstance, MAKEINTRESOURCE(uIcon));
	if(hIcon)	
	{	
		DrawIconEx (hDC, pt.x, pt.y, hIcon, N_BITMAP_SIZE, N_BITMAP_SIZE, NULL,	NULL, DI_NORMAL);
		DeleteObject (hIcon);
	}
			
	return TRUE;
}
// ******************************************************************
void CSystray::OnOwnerDrawMenu_CreatePopup(HWND hwnd)
{
	HMENU hMenu = CreatePopupMenu();
	AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_EXTPLAYLIST,   GS(LNG_TIPS_EXTPLAYLIST) ); 
	AppendMenu(hMenu, MF_SEPARATOR, NULL, _T(""));
	AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_PREVIOUSTRACK, GS(LNG_TIPS_PREVIOUSTRACK) );
	AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_PLAYPAUSE,     GS(LNG_TIPS_PLAYPAUSE) );
	AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_STOP,          GS(LNG_TIPS_STOP) );
	AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_NEXTTRACK,     GS(LNG_TIPS_NEXTTRACK) );
	AppendMenu(hMenu, MF_OWNERDRAW, IDM_SYSTRAY_LOADFILE,      GS(LNG_TIPS_LOADFILE) );

	POINT pos;
	GetCursorPos(&pos);
	SetForegroundWindow(hwnd);  

	TrackPopupMenu (hMenu, 0, pos.x, pos.y,0,hwnd, NULL);

	// BUGFIX: See "PRB: Menus for Notification Icons Don't Work Correctly"
	PostMessage(hwnd, WM_NULL, 0, 0);
	DestroyMenu(hMenu);
					
}

// ******************************************************************
LRESULT CALLBACK CSystray::SysTrayWndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	const int WMA_SYSTRAY = WM_USER+27;
	static int m_nFontHeight = 0;
	static HFONT m_menuFont = NULL;
	static int nPosMenuPopEZplaylist; //for modify item menu Show/Hide

	WNDPROC lpOldWndProc = (WNDPROC)GetWindowLong(hwnd, GWL_USERDATA);

	switch(message)
	{
		case WM_INITMENUPOPUP: 
			{
				HMENU	hmenuPopup = (HMENU) wParam;         // handle to submenu
				MENUITEMINFO mii;
				ZeroMemory(&mii, sizeof(mii));       
				mii.cbSize = sizeof(MENUITEMINFO);
				mii.fMask = MIIM_ID;

				if (GetMenuItemInfo(hmenuPopup, 0, TRUE, &mii))
				{
					if (mii.wID == WINAMP_HELP_ABOUT)
					{
						mii.cbSize = sizeof(MENUITEMINFO);
						mii.fMask = MIIM_ID;

						if (!GetMenuItemInfo(hmenuPopup, EZPLAYLIST_SHOW, FALSE, &mii) &&
							!GetMenuItemInfo(hmenuPopup, EZPLAYLIST_HIDE, FALSE, &mii))
						{
							nPosMenuPopEZplaylist = systray->AppendToWinampSystemMenu(hwnd, hmenuPopup);
						}
						else
						{
							HMENU hSubMenu = GetSubMenu(hmenuPopup, nPosMenuPopEZplaylist);
						
							if (systray->m_DlgMain->m_bVisible)
								ModifyMenu(hSubMenu, EZPLAYLIST_SHOW, MF_BYCOMMAND, EZPLAYLIST_HIDE, GS(LNG_WINAMPMENU_HIDE));
							else
								ModifyMenu(hSubMenu, EZPLAYLIST_HIDE, MF_BYCOMMAND, EZPLAYLIST_SHOW, GS(LNG_WINAMPMENU_SHOW));

						}
					}
				}
			}
			break;		
		

		
		// WM_MEASUREITEM (OnwerDraw Menu with icon)		
		case WM_MEASUREITEM: 
			systray->OnOwnerDrawMenu_MeasureItem((LPMEASUREITEMSTRUCT) lParam, &m_nFontHeight, &m_menuFont);
			break;
		// WM_DRAWITEM (OwnerDraw Menu with icon)
		case WM_DRAWITEM:
			systray->OnOwnerDrawMenu_DrawItem((LPDRAWITEMSTRUCT) lParam, m_nFontHeight, m_menuFont);
			break;
		case WM_DESTROY:
			if ((HFONT)m_menuFont) 
				DeleteObject(m_menuFont);		
			break;


		case WM_SYSCOMMAND:
			switch (LOWORD(wParam))
			{	
				case EZPLAYLIST_SHOW:
					systray->ezplaylist_create();
					break;
				case EZPLAYLIST_HIDE:
					ShowWindow(systray->m_DlgMain->m_hwndMain, SW_HIDE); 
					systray->m_DlgMain->m_bVisible = FALSE;
					break;
				case EZPLAYLIST_PROPERTY:
					config();
					break;
				case EZPLAYLIST_SYSTRAY:
					{
						SetForegroundWindow(hwnd);
						CDlgSysTray dlg;
						dlg.DoModal(IDD_DLGSYSTRAY, hwnd);
					}
					break;
			}
		case WM_COMMAND:
			switch (LOWORD(wParam))
			{	// Via EZPlaylist Popup Menu
				case IDM_SYSTRAY_EXTPLAYLIST:	systray->ezplaylist_create();				return 0;
				case IDM_SYSTRAY_PREVIOUSTRACK:	systray->winamp_previous_track(hwnd);		return 0;
				case IDM_SYSTRAY_PLAYPAUSE:		systray->winamp_play_pause(hwnd);			return 0;
				case IDM_SYSTRAY_STOP:			systray->winamp_stop(hwnd);					return 0;
				case IDM_SYSTRAY_NEXTTRACK:		systray->winamp_next_track(hwnd);			return 0;
				case IDM_SYSTRAY_LOADFILE:		systray->winamp_openfile(hwnd);				return 0;
			}
			break;
	
		case WMA_SYSTRAY:
		{
			// Via Taskbar
			int which = LOWORD(wParam) - 1024;

			switch (LOWORD(lParam))
			{
				case WM_LBUTTONDOWN:
					switch (which)
					{
						case 0:	systray->ezplaylist_create();				return 0; // show Main Window EZPlaylist
						case 1: systray->winamp_previous_track(hwnd);		return 0; 
						case 2: systray->winamp_play_pause(hwnd);			return 0; 
						case 3: systray->winamp_stop(hwnd);					return 0;
						case 4: systray->winamp_next_track(hwnd);			return 0;
						case 5: systray->winamp_openfile(hwnd);				return 0;
						case 6:	systray->winamp_volume_up(hwnd);			return 0;
						case 7: systray->OnOwnerDrawMenu_CreatePopup(hwnd);	return 0; // CreatePopup menu EZPlaylist
					}
					return 0;
			
				case WM_RBUTTONDOWN:
					switch (which)
					{
						case 6:	systray->winamp_volume_down(hwnd);			return 0;
						case 0:
						case 7:	
							{
								SetForegroundWindow(hwnd);
								CDlgSysTray dlg;
								dlg.DoModal(IDD_DLGSYSTRAY, hwnd);
							}
							return 0;	
					}
					return 0;
			}
		}
	}

//$$	return CallWindowProc(lpWndProcOld,hwnd,message,wParam,lParam);
return CallWindowProc(lpOldWndProc,hwnd,message,wParam,lParam);

	
}



// ******************************************************************
BOOL CSystray::ezplaylist_create(void)
{
	if (m_DlgMain->m_hwndMain != NULL)
	{
		ShowWindow (m_DlgMain->m_hwndMain, SW_RESTORE); 
	}
	else
	{
		m_DlgMain->m_hwndMain = m_DlgMain->DoModeless(IDD_DLGEZPLAYLIST, plugin.hwndParent);

		if (m_DlgMain->m_hwndMain == NULL)
			return (FALSE);

	}

	UpdateWindow (m_DlgMain->m_hwndMain);

	SetForegroundWindow(m_DlgMain->m_hwndMain);

	m_DlgMain->m_bVisible = TRUE;
	return (TRUE);
}


// ******************************************************************
void CSystray::ezplaylist_destroy(void)
{
	// clean up Dialogbox EZ-playlist
	DestroyWindow(m_DlgMain->m_hwndMain); 
	
	if (m_DlgMain)
		delete m_DlgMain;
	
}


// ******************************************************************
int CSystray::AppendToWinampSystemMenu(HWND hWnd, HMENU hMenuWinamp)
{
	HMENU hMenu = CreatePopupMenu();

	AppendMenu(hMenu, MF_STRING, 
		systray->m_DlgMain->m_bVisible ? EZPLAYLIST_HIDE : EZPLAYLIST_SHOW, 
		systray->m_DlgMain->m_bVisible ? GS(LNG_WINAMPMENU_HIDE) : GS(LNG_WINAMPMENU_SHOW));
	AppendMenu(hMenu, MF_SEPARATOR, 0, NULL);
	AppendMenu(hMenu, MF_STRING, EZPLAYLIST_SYSTRAY, GS(LNG_WINAMPMENU_SYSTRAY_CONFIG));
	AppendMenu(hMenu, MF_STRING, EZPLAYLIST_PROPERTY, GS(LNG_WINAMPMENU_EZPLAYLIST_CONFIG));

	int nPos=0;
	while (WINAMP_FILE_QUIT != GetMenuItemID(hMenuWinamp, nPos++));
 
	nPos = nPos - 2;

	InsertMenu(hMenuWinamp, nPos, MF_BYPOSITION|MF_POPUP, (UINT)hMenu, "&EZ-PLaylist");

	return nPos;
}


// ******************************************************************
// ****************** WRAPPER WINAMP FUNCTION ************************

void CSystray::winamp_previous_track(HWND hwnd)
{
	int a;

	// not playing, let's hit prev
	if ((a=amp->IsPlaying()) == 0) 
		// Previous
		amp->Previous();
	else if (a != 3 && SendMessage(hwnd,WM_USER,0,105) > 2000) // restart
		//Play
		amp->Play();
	else 
		// previous
		amp->Previous();
}
// -----------------------------------------------
void CSystray::winamp_play_pause(HWND hwnd)
{

	// not playing, let's hit play
	if (amp->IsPlaying() ) 
		amp->Pause();
	else  
		amp->Play();
}	
// -----------------------------------------------
void CSystray::winamp_stop(HWND hwnd)
{
	if (GetKeyState(VK_SHIFT) & (1<<15))
		amp->FadeoutAndStop();
	else
		amp->Stop();
}
// -----------------------------------------------
void CSystray::winamp_next_track(HWND hwnd)
{
	amp->Next();
}
// -----------------------------------------------
void CSystray::winamp_openfile(HWND hwnd)
{
	SetForegroundWindow(hwnd);
	if (GetKeyState(VK_CONTROL) & (1<<15))
		amp->OpenURLDlg(); 	// internet location
	else if (GetKeyState(VK_SHIFT) & (1<<15))
		amp->OpenDirDlg(); 	// open directory
	else
		amp->OpenFileDlg(); // open file
}
// -----------------------------------------------
void CSystray::winamp_volume_up(HWND hwnd)
{
	SetForegroundWindow(hwnd);
	int k, nMax;
	if (GetKeyState(VK_CONTROL) & (1<<15))
		nMax = 20;
	else
		nMax = 2;

	for (k=0; k<nMax; k++)
		amp->VolumeRaise();
}
// -----------------------------------------------
void CSystray::winamp_volume_down(HWND hwnd)
{
	SetForegroundWindow(hwnd);
	int k, nMax;
	if (GetKeyState(VK_CONTROL) & (1<<15))
		nMax = 20;
	else
		nMax = 2;

	for (k=0; k<nMax; k++)
		amp->VolumeLower();
}

