// Taginfo.cpp: implementation of the CTaginfo class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "Taginfo.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


extern bool IsStringEmpty(const string& s);
extern bool IsStringEmpty(string *s);
extern bool IsStringEmpty(wstring *s);
extern bool IsStringEmpty(const wstring& s);


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTaginfo::CTaginfo(int type_info)
{

//	type_info == 0  ID3V1
//	type_info == 1  ID3V2
//	type_info == 2  ID3V2 if null --> ID3V1
//	type_info == 3  ID3V1 if null --> ID3V2

	m_type_info = type_info;

}

CTaginfo::~CTaginfo()
{

}



// ***************************************************************************
void CTaginfo::GetFrameInfo(CMP3Info *mp3i, int fid, tstring &str)
{
//	m_type_info == 0  ID3V1
//	m_type_info == 1  ID3V2
//	m_type_info == 2  ID3V2 if null --> ID3V1
//	m_type_info == 3  ID3V1 if null --> ID3V2

	switch(m_type_info)
	{
		case 0:	// ID3v1
			GetFrameInfoID3v1(mp3i, fid, str);
			break;
			
		case 1: // ID3v2
			GetFrameInfoID3v2(mp3i, fid, str);
			break;
		
		case 2: // ID3v2 (NULL)--> ID3v1
			GetFrameInfoID3v2(mp3i, fid, str);
			if (str.compare(_T("")) == 0 || str.compare(_T(" ")) == 0)
			{	
				str.erase();
				GetFrameInfoID3v1(mp3i, fid, str);
			}
			break;
			
		case 3: // ID3v1 (NULL)--> ID3v2
			GetFrameInfoID3v1(mp3i, fid, str);
			if (str.compare(_T("")) == 0 || str.compare(_T(" ")) == 0)
			{	
				str.erase();
				GetFrameInfoID3v2(mp3i, fid, str);
			}
			
			break;
			
	}

}


// ***************************************************************************
void CTaginfo::GetFrameInfoID3v1(CMP3Info *mp3i, int fid, tstring &str)
{
	struct ID3V1MP3Info *id3i = mp3i->GetID3v1();
	
	if (!id3i)
		return;
	
	switch(fid)
	{
		case ID3TAG_COMM: // read the Comment
			if (!IsStringEmpty((LPTSTR)id3i->Comment)) 
				a2tcpy(str, (LPTSTR)id3i->Comment);
			break;
		
		case ID3TAG_TYER: // read the Year
			if (!IsStringEmpty((LPTSTR)id3i->Year)) 
				a2tcpy(str, (LPTSTR)id3i->Year);
			break;
		
		case ID3TAG_TALB: // read the Album
			if (!IsStringEmpty((LPTSTR)id3i->Album)) 
				a2tcpy(str, (LPTSTR)id3i->Album);
			break;
		
		case ID3TAG_TPE1: // read the Artist
			if (!IsStringEmpty((LPTSTR)id3i->Artist)) 
				a2tcpy(str, (LPTSTR)id3i->Artist);
			break;
		
		case ID3TAG_TIT2:  // read the Title:Song name
			if (!IsStringEmpty((LPTSTR)id3i->SongName)) 
				a2tcpy(str, (LPTSTR)id3i->SongName);
			break;
		
		case ID3TAG_TCON: // read the genre
			a2tcpy(str, (LPTSTR)GenreInt2Str(id3i->Genre));
			break;
		
	
		case ID3TAG_TRCK: // read the Track
			if (id3i->Track != 0) 
			{
				TCHAR szTrack[4];
				_stprintf(szTrack, _T("%d"), id3i->Track);
				a2tcpy(str, (LPTSTR)szTrack);
			}
			break;
	}

}


// ***************************************************************************
void CTaginfo::GetFrameInfoID3v2(CMP3Info *mp3i, int fid, tstring &str)
{
	// only supported frames allowed
	if (!ID3Frames[fid].Supported) 
		return; 
	if (ID3Frames[fid].Type != ID3ENC_STRING)	
		return;
		
	CMP3Frame *mp3f = mp3i->GetFrame(fid);
			
	if (mp3f)
		str = ((CMP3StringFrame *)mp3f)->GetString();	// handle only the first String
}

