#include "stdafx.h"
#include <commctrl.h>

#include "gen.h"
#include "definition.h"
#include "util.h"		
#include "mp3info/MP3Info.h"	// Use only for MatcTime function
#include "resource.h"
#include "Language.h"

// external declaration
extern winampGeneralPurposePlugin plugin;
extern INITSTRUCT gParam;
extern CLanguage    *lng;

extern bool IsStringEmpty(const string& s);
extern bool IsStringEmpty(string *s);
extern bool IsStringEmpty(wstring *s);
extern bool IsStringEmpty(const wstring& s);




TCHAR *CSSFontName[4]   = {
							_T("Arial, Helvetica, sans-serif"), 
							_T("\"Times New Roman\", Times, serif"), 
							_T("\"Courier New\", Courier, mono"), 
							_T("Verdana, Arial, Helvetica, sans-serif")
};

TCHAR *CSSFontStyle[2]  = {  _T("normal"), _T("italic") };

TCHAR *CSSFontWeight[2] = {  _T("normal"), _T("bold") };

TCHAR *CSSFontSize[7]   = { 
								_T("xx-small"), 
								_T("x-small"), 
								_T("small"), 
								_T("medium"), 
								_T("large"),
								_T("x-large"),
								_T("xx-large")
};


// Shortcut Hook procedure
// stuff to install the accelerator Main Window EZPlaylist
static LRESULT CALLBACK MyHookProcMain(int nCode, WPARAM wParam, LPARAM lParam);
static HACCEL gMainAccel = NULL;
static HHOOK gOldMainMsgHook = NULL;
HWND ghWndMainDlg = NULL;

// Shortcut Hook procedure
// stuff to install the accelerator Page ID3Tag Edit Window 
static LRESULT CALLBACK MyHookProcEditTag(int nCode, WPARAM wParam, LPARAM lParam);
static HACCEL gAccelEditTag = NULL;
static HHOOK gOldMsgHookEditTag = NULL;
HWND ghWndDlgEditTag = NULL;

// Tooltips 
HWND SetHookTooltipDialog(BOOL bActive, HWND hDlg, HINSTANCE hInstance);
static LRESULT CALLBACK GetMsgTooltipProc(int nCode, WPARAM wParam, LPARAM lParam);
static BOOL CALLBACK EnumTooltipChildProc(HWND hwndCtrl, LPARAM lParam);
HWND  ghTooltipDlg = NULL;
HHOOK ghTooltipHook = NULL;
HWND ghTooltip = NULL;


typedef int (*compfn)(const void*, const void*);
int compare(struct aMatchTime *elem1, struct aMatchTime *elem2);
	
struct aMatchTime
{	
	int nIndex;
	int nLength;
};


/* ********************************************************* */
void SetColumnOrder(int order[])
{
    for (int i = 0; i < COLUMN_MAX; i++)
        gParam.column_order[i] = order[i];
}

/* ********************************************************* */
void GetColumnOrder(int order[])
{
    for (int i = 0; i < COLUMN_MAX; i++)
        order[i] = gParam.column_order[i];
}

/* ********************************************************* */
void SetColumnShown(int shown[])
{
    for (int i = 0; i < COLUMN_MAX; i++)
        gParam.column_shown[i] = shown[i];
}

/* ********************************************************* */
void GetColumnShown(int shown[])
{
    for (int i = 0; i < COLUMN_MAX; i++)
        shown[i] = gParam.column_shown[i];
}

/* ********************************************************* */
void SetColumnWidths(int width[])
{
    for (int i=0; i < COLUMN_MAX; i++)
        gParam.column_width[i] = width[i];
}
/* ********************************************************* */
void GetColumnWidths(int width[])
{
    for (int i=0; i < COLUMN_MAX; i++)
        width[i] = gParam.column_width[i];
}

/* ********************************************************* */
void SetSortColumn(int column)
{
    gParam.sort_reverse = (column < 0) ? TRUE : FALSE;
    gParam.sort_column = abs(column) ; 
}

/* ********************************************************* */
int GetSortColumn(void)
{
    int column = gParam.sort_column ; 

    if (gParam.sort_reverse)
        column = -(column);

    return column;
}







// =++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++=
// =++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++=
// =++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++=
// =++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++=
// =++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++=
// =++==++==++==++==++==++==++==++==++==++==++==++==++==++==++==++=




















/* ************************************************************************ */
void SetHookAccelMain(BOOL bActive, int idAccel, HWND hDlg)
{
	if (bActive)
	{
		if (gOldMainMsgHook == NULL)
		{
			ghWndMainDlg = hDlg;
			gOldMainMsgHook = SetWindowsHookEx(WH_MSGFILTER, (HOOKPROC)MyHookProcMain, NULL, GetCurrentThreadId());
			gMainAccel = LoadAccelerators(plugin.hDllInstance, MAKEINTRESOURCE(idAccel));
		}
	}
	else
	{
		UnhookWindowsHookEx(gOldMainMsgHook);
		DestroyAcceleratorTable(gMainAccel);
	
		gOldMainMsgHook = NULL;
		gMainAccel = NULL;
		ghWndMainDlg = NULL;
	}
}

/* ******************************************************* */
// WH_MSGFILTER hook procedure / Shortcut Accelerator
static LRESULT CALLBACK MyHookProcMain(int nCode, WPARAM wParam, LPARAM lParam)
{
	//$$ if (nCode == MSGF_DIALOGBOX) 
	{
		
		
		if(ghWndMainDlg && gMainAccel)
		{
		//	if (GetFocus() == GetDlgItem(ghWndMainDlgn IDC_SEARCH))
			
			
			if (TranslateAccelerator(ghWndMainDlg, gMainAccel, (LPMSG)lParam))
				return(TRUE);	

		}
	
	}

	// default: call the next hook
	return CallNextHookEx(gOldMainMsgHook, nCode, wParam, lParam);
}



/* ************************************************************************ */
void SetHookAccelEditTag(BOOL bActive, int idAccel, HWND hDlg)
{
	if (bActive)
	{
		if (gOldMsgHookEditTag == NULL)
		{
			ghWndDlgEditTag = hDlg;
			gOldMsgHookEditTag = SetWindowsHookEx(WH_MSGFILTER, (HOOKPROC)MyHookProcEditTag, NULL, GetCurrentThreadId());
			gAccelEditTag = LoadAccelerators(plugin.hDllInstance, MAKEINTRESOURCE(idAccel));
		}
	}
	else
	{
		UnhookWindowsHookEx(gOldMsgHookEditTag);
		DestroyAcceleratorTable(gAccelEditTag);
	
		gOldMsgHookEditTag = NULL;
		gAccelEditTag = NULL;
		ghWndDlgEditTag = NULL;
	}
}

/* ******************************************************* */
// WH_MSGFILTER hook procedure / Shortcut Accelerator
static LRESULT CALLBACK MyHookProcEditTag(int nCode, WPARAM wParam, LPARAM lParam)
{
	if (nCode == MSGF_DIALOGBOX) 
	{
		if(ghWndDlgEditTag && gAccelEditTag)
			if (TranslateAccelerator(ghWndDlgEditTag, gAccelEditTag, (LPMSG)lParam))
				return(TRUE);	
	}

	// default: call the next hook
	return CallNextHookEx(gOldMsgHookEditTag, nCode, wParam, lParam);
}

// **********************************************
//
// Tooltips control Hook message
//
// **********************************************

// SetHookTooltipDialog - Create Hook procedure or destroy it
HWND SetHookTooltipDialog(BOOL bActive, HWND hDlg)
{
	if (bActive)
	{
		ghTooltipDlg  = hDlg;

		ghTooltip = CreateWindowEx(0, TOOLTIPS_CLASS, (LPSTR) NULL, 
						TTS_ALWAYSTIP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, 
						CW_USEDEFAULT, hDlg, (HMENU) NULL, plugin.hDllInstance, NULL); 

		if (ghTooltip == NULL) 
			return NULL; 


		// Enumerate the child windows to register them with the tooltip  control. 
		EnumChildWindows(hDlg, (WNDENUMPROC) EnumTooltipChildProc, 0);

		// Attach Hook procedure
		ghTooltipHook = SetWindowsHookEx(WH_MSGFILTER, (HOOKPROC)GetMsgTooltipProc, NULL, GetCurrentThreadId());
	}
	else
	{
		UnhookWindowsHookEx(ghTooltipHook);
		DestroyWindow(ghTooltip);
	
		ghTooltipDlg  = NULL;
		ghTooltip     = NULL;
		ghTooltipHook = NULL;
	}

	return ghTooltip;
}

// GetMsgTooltipProc - monitors the message stream for mouse messages intended 
//     for a control window in the dialog box. 
// Returns a message-dependent value. 
// nCode - hook code. 
// wParam - message flag (not used). 
// lParam - address of an MSG structure. 
LRESULT CALLBACK GetMsgTooltipProc(int nCode, WPARAM wParam, LPARAM lParam) 
{ 
	MSG *lpmsg; 

	lpmsg = (MSG *) lParam; 
	if (nCode < 0 || !(IsChild(ghTooltipDlg, lpmsg->hwnd))) 
		return (CallNextHookEx(ghTooltipHook, nCode, wParam, lParam)); 

	
	switch (lpmsg->message) 
	{ 
		case WM_MOUSEMOVE: 
		case WM_LBUTTONDOWN: 
		case WM_LBUTTONUP: 
		case WM_RBUTTONDOWN: 
		case WM_RBUTTONUP: 
			if (ghTooltip != NULL) 
			{ 
				MSG msg; 

				msg.lParam  = lpmsg->lParam; 
				msg.wParam  = lpmsg->wParam; 
				msg.message = lpmsg->message; 
				msg.hwnd    = lpmsg->hwnd; 
				SendMessage(ghTooltip, TTM_RELAYEVENT, 0, (LPARAM) (LPMSG) &msg); 
			} 
			break; 
		default: 
			break; 
	} 
	return (CallNextHookEx(ghTooltipHook, nCode, wParam, lParam)); 
} 


// EnumTooltipChildProc - registers control windows with a tooltip control by
//     using the TTM_ADDTOOL message to pass the address of a 
//     TOOLINFO structure. 
// Returns TRUE if successful, or FALSE otherwise. 
// hwndCtrl - handle of a control window. 
// lParam - application-defined value (not used). 
BOOL CALLBACK EnumTooltipChildProc(HWND hwndCtrl, LPARAM lParam) 
{ 
	TOOLINFO ti; 

	ti.cbSize = sizeof(TOOLINFO); 
	ti.uFlags = TTF_IDISHWND; 
	ti.hwnd = ghTooltipDlg; 
	ti.uId = (UINT) hwndCtrl; 
	ti.hinst = 0; 
	ti.lpszText = LPSTR_TEXTCALLBACK; 
	LRESULT lRet = SendMessage(ghTooltip, TTM_ADDTOOL, 0, (LPARAM) (LPTOOLINFO) &ti); 
	return TRUE; 
} 



/**************************************************************************** */
// DupString - allocates a copy of a string. 
// lpsz - address of the null-terminated string to copy.  
LPTSTR DupString(LPTSTR lpsz) 	
{     
	int cb = _tcslen(lpsz) + 1; 
    LPTSTR lpszNew = (LPTSTR)GlobalAlloc(LMEM_FIXED, cb);     
	if (lpszNew != NULL) 
        CopyMemory(lpszNew, lpsz, cb);     
	
	return lpszNew; 	
} 


/* *********************************************************** */ 
TCHAR *GetFormattedDateTime(void) 
{ 
	static TCHAR szTime[64]; 
    int cch = 0; 
 
 	// Date format
	cch = GetDateFormat( 
	        LOCALE_USER_DEFAULT, 
	        0, 
	        NULL,			// current date 
	        _T("yyyy/MM/dd"),   
	        szTime, 
	        sizeof szTime); 
 
    // cch includes null terminator, change it to 
    // a space to separate from time. 
    szTime[ cch - 1 ] = ' '; 
    
 	// Time format
    GetTimeFormat( 
	    LOCALE_USER_DEFAULT, 
	    TIME_NOSECONDS, 
	    NULL,   // use current time 
	    NULL,   // use default format 
	    szTime + cch, 
	    (sizeof szTime) - cch ); 
 
    return szTime; 
} 


/* ************************************************************** */
void  ListboxMoveItem(HWND hwndlist, BOOL bMoveUp)
{
	int nCount;
	TCHAR szTmp[MAX_PATH];
	int pos1, pos2;
	LPARAM itemData;

	nCount = SendMessage(hwndlist, LB_GETCOUNT, 0, 0);

	pos1   = SendMessage(hwndlist, LB_GETCURSEL, 0, 0);

	if (pos1 == LB_ERR) 
		return;
	if (nCount == 0)
		return;
	if (pos1 <= 0 && bMoveUp == TRUE)
		return;
	if ((pos1+1) >= nCount && bMoveUp== FALSE)
		return;

	if (bMoveUp == FALSE)
	{
		pos2 = pos1 + 1;
	}
	else
	{
		pos2 = pos1 - 1;
	}
	
	// get the string + itemData from the listbox
	SendMessage(hwndlist,LB_GETTEXT, pos1, (LPARAM)szTmp);	
	itemData =  SendMessage(hwndlist, LB_GETITEMDATA, (WPARAM)pos1, 0);
	// Delete old Item
	SendMessage(hwndlist,LB_DELETESTRING, pos1, 0);

	// insert Item at new position
	int nIdxnew = SendMessage(hwndlist, LB_INSERTSTRING, pos2, (LPARAM)szTmp);
	int nErr    = SendMessage(hwndlist, LB_SETITEMDATA,  nIdxnew, (LPARAM)itemData);
	SendMessage(hwndlist,LB_SETCURSEL, pos2, 0);

}


/* ************************************************************ */
void SetFilter(TCHAR *szFilter)
{
	// Use for OPENFILENAME 
	TCHAR chReplace;
	UINT u;

    chReplace  = szFilter[_tcslen(szFilter) - 1];
    for (u = 0; szFilter[u] != '\0'; u++)
        if (szFilter[u] == chReplace)
            szFilter[u] = '\0';
}


// ***************************************************************************
void AddFileExtension(TCHAR *szFile, TCHAR *szExt) 
{ 
	int nLen = _tcslen(szFile);
	int nLenExt  = _tcslen(szExt);

	if (nLen > nLenExt)
	{
		TCHAR *szFileExt = NULL;
		TCHAR *p = &szFile[nLen-nLenExt];

		// nLenExt+1: terminate string by '\0'
		szFileExt = (TCHAR*)malloc((nLenExt+1)*sizeof(TCHAR));		
		if (szFileExt == NULL)
		{	_tcscat(szFile, szExt);
			return;
		}
		_tcscpy(szFileExt, p);
		_tcslwr(szFileExt);

		// there is no extension so add it
		if (_tcscmp(szFileExt,szExt) != 0)
			_tcscat(szFile, szExt);

		free(szFileExt);
	}
	else
		_tcscat(szFile, szExt);

}
/* ******************************************************** */
void Time2Str(int nSecond, TCHAR *pTimeFrmt, BOOL bShortFormat)
{
	int s, m, h;
	s = m = h = 0;
	
	s = nSecond % 60;
	m = nSecond / 60;
	h = m / 60;
	m = m % 60;

	if (bShortFormat)
	{
		if (h == 0)
			_stprintf(pTimeFrmt, _T("%02d:%02d"), m,s);
		else
			_stprintf(pTimeFrmt, _T("%02d:%02d:%02d"), h, m,s);
	}
	else
	{
		if (h == 0)
			_stprintf(pTimeFrmt, GS(LNG_SHORTTIME_FRMT), m,s);
		else
			_stprintf(pTimeFrmt, GS(LNG_FULLTIME_FRMT), h, m,s);
	}
	
}

/* ******************************************************** */
bool IsStringEmpty(const string& s)
{
	int len = s.length();
	for (int i=0; i<len; i++)
		if (!isspace(s[i])) return(false);
	return true;
}
bool IsStringEmpty(string *s)
{
	if (s==NULL) return(true);		// NULL pointer is empty string
	return(IsStringEmpty(*s));
}

bool IsStringEmpty(wstring *s)
{
	if (s==NULL) return(true);		// NULL pointer is empty string
	return(IsStringEmpty(*s));
}
bool IsStringEmpty(const wstring& s)
{
	int len = s.length();
	for (int i=0; i<len; i++)
		if (!iswspace(s[i])) return(false);
	return true;
}
/* ******************************************************** */
void TrimString(tstring& str)
{
	int begin=0;
	int end=str.length()-1;
	bool changed = false;
	while(_istspace(str[begin])) {changed = true; begin++;}
	// now the end
	while(_istspace(str[end])) {changed = true; end--;}
	if (!changed) return;
	str = str.substr(begin, end-begin+1);
}



/* ******************************************************** */
void MatchTime(HWND hListview, int nLengthMax)
{
	struct aMatchTime  *pArray;
	
	LVITEM lvi; 
	int i, nCount;
	int nLength = 0;
	
	nCount = ListView_GetItemCount(hListview);
	
	pArray = (struct aMatchTime*)malloc(nCount*sizeof(struct aMatchTime));

	if (pArray == NULL)
		return;

	// Copy in array temp Index listview and PlayLength
	ZeroMemory(&lvi, sizeof(lvi));
	lvi.mask =  LVIF_PARAM; 
	
	for (i=0; i<nCount;i++)
	{
		lvi.iItem  = i;
		ListView_GetItem(hListview, &lvi);

		CMP3Info *mp3i = (CMP3Info *)lvi.lParam;    // Retreive item data  MP3Info
		
		if (!mp3i) continue;

		pArray[i].nIndex = i;
		pArray[i].nLength = mp3i->GetPlayLength();		
	}

	// Order array
	// MSDN April 1999, qsort:Article ID: Q73853 
	qsort((void *)pArray, nCount, sizeof(struct aMatchTime), (compfn)compare);


	// Selected songs from match Time array
	for (i=0, nLength=0; i<nCount;i++)
	{
		int idx;
		
		idx = pArray[i].nIndex;
		nLength += pArray[i].nLength;

		if (nLength < nLengthMax)
		{
			// Selected the song Match Time
			ZeroMemory(&lvi, sizeof(lvi)); 	
			lvi.mask      = LVIF_STATE; 
			lvi.stateMask = LVIS_SELECTED; 
			lvi.iItem     = idx;
			ListView_SetItemState(hListview, idx, LVIS_SELECTED , LVIS_SELECTED);
		}
		else
			i = nCount;  // force to exit loop
		
	}

	ListView_EnsureVisible(hListview, pArray[0].nIndex, TRUE);

	free(pArray); 

	SetFocus(hListview);
}

/* ******************************************************** */
int compare(struct aMatchTime *elem1, struct aMatchTime *elem2)
{
	if ( elem1->nLength < elem2->nLength)
		return -1;

	else if (elem1->nLength > elem2->nLength)
		return 1;

	else
		return 0;
}


/* ************************************************************************ */
void GetWinampPathPlugin(TCHAR *path_file)
{	
	TCHAR *p;

	// makes a .ini file in the winamp directory named "plugin.ini"
	GetModuleFileName(NULL, path_file, MAX_PATH);
	p = path_file + _tcslen(path_file);
	while (p >= path_file && *p != '\\') 
		p--;
	
	if (++p >= path_file) 
		*p = 0;

	_tcscat(path_file, _T("Plugins\\") );

}


// **************************************************************
void DlgSetText(HWND hDlg, int item, TCHAR* szText)
{
	if(szText==NULL)
		return;
	SetDlgItemText(hDlg, item, szText);
}

// **************************************************************
void SetText(HWND hWnd, TCHAR* szText)
{
	if(szText==NULL)
		return;
	SendMessage(hWnd, WM_SETTEXT, 0, (LPARAM)szText);
}

// **************************************************************
void MenuSetText(HMENU hMenu,int item ,TCHAR* szText )
{
	MENUITEMINFO mmi;
	TCHAR *szTmp = NULL;
	int nLen = _tcslen(szText)+1;

	
	if(szText==NULL)
		return;

	// Change les | en Tabulation pour les raccourci 
	// Open|Ctrl+O --> 'Open     Ctrl+O'
	szTmp = (TCHAR*)malloc(nLen*sizeof(TCHAR));
	if (szTmp)
	{	_tcscpy(szTmp, szText);

		for (int i=0; i< _tcslen(szTmp); i++)
		{	if (szTmp[i] == '|') 
				szTmp[i] = 0x09;
		}
	}
	else

		szTmp = szText;

	memset(&mmi, 0, sizeof(MENUITEMINFO));             
	mmi.cbSize     = sizeof(MENUITEMINFO);
	mmi.fMask      = MIIM_TYPE; 
	mmi.fType      = MFT_STRING;
	mmi.dwTypeData = szTmp;
	mmi.cch        = lstrlen(mmi.dwTypeData);                 
	SetMenuItemInfo( hMenu, item, TRUE, &mmi);
}

// **************************************************************
void MenuAddItem(HMENU hMenu,int item , int wID,TCHAR* szText)
{
	MENUITEMINFO item_info;

	if(szText==NULL)
		return;

	item_info.cbSize     = sizeof(item_info);
	item_info.fMask      = MIIM_TYPE|MIIM_ID; 
	item_info.fType      = MFT_STRING;
	item_info.dwTypeData = szText;
	item_info.wID        = wID;
	InsertMenuItem( hMenu, item, TRUE, &item_info);
}

// **************************************************************
HMENU MenuGetPopup( HMENU hMenu, int item )
{
	MENUITEMINFO item_info;

	item_info.cbSize = sizeof(item_info);
	item_info.fMask  = MIIM_SUBMENU; 
	GetMenuItemInfo( hMenu, item, TRUE, &item_info);
	return item_info.hSubMenu;

}


// **************************************************************
// Source extract from Album List plugin for Winamp 'Safai Ma'
void GetVersionInformation(int &major, int &minor, int &build)
{
	// Get version information
	char				szFullPath[256];
	DWORD				dwVerHnd=0;			// An 'ignored' parameter, always '0'
	VS_FIXEDFILEINFO*	lpVersion;			// Pointer to 'Fixed' file info
	UINT				uVersionLen;

	GetModuleFileName(plugin.hDllInstance, szFullPath, sizeof(szFullPath));
	DWORD dwVerInfoSize = GetFileVersionInfoSize(szFullPath, &dwVerHnd);
	if (dwVerInfoSize)
	{
		LPSTR   lpstrVffInfo;
		HANDLE  hMem;

		// allocate memory for version block
		hMem = GlobalAlloc(GMEM_MOVEABLE, dwVerInfoSize);
		lpstrVffInfo  = (LPSTR)GlobalLock(hMem);
		GetFileVersionInfo(szFullPath, dwVerHnd, dwVerInfoSize, lpstrVffInfo);
		BOOL bRetCode = VerQueryValue(	lpstrVffInfo, "\\", (LPVOID *)&lpVersion, &uVersionLen);
		if (bRetCode)
		{
			// access version information
			major = HIWORD(lpVersion->dwProductVersionMS);
			minor = LOWORD(lpVersion->dwProductVersionMS);
			build = LOWORD(lpVersion->dwFileVersionLS);
		}

		GlobalUnlock(hMem);
		GlobalFree(hMem);
	}
}



/*  Drawing Transparent Bitmap with ease with on the fly masks in MFC --> Win32 API
 *
 *   http://www.codeproject.com/bitmap/transbitmapmask.asp
 */

//**----------------------------------------------------------
//** STEP 2: Create the Mask and dump it into a CBitmap Object
//**----------------------------------------------------------
HBITMAP PrepareMask( HBITMAP hBmpSource,
                  
                  COLORREF clrpTransColor, // Pass null if unknown
                  int iTransPixelX,        // = 0
                  int iTransPixelY         // = 0
                  )
{
    BITMAP bm;
	HBITMAP hBmpMask;

   // Get the dimensions of the source bitmap
   GetObject((HBITMAP)hBmpSource, sizeof(bm),  &bm);
   
   // Create the mask bitmap
    hBmpMask = CreateBitmap( bm.bmWidth, bm.bmHeight, 1, 1, NULL);

   // We will need two DCs to work with. One to hold the Image
   // (the source), and one to hold the mask (destination).
   // When blitting onto a monochrome bitmap from a color, pixels
   // in the source color bitmap that are equal to the background
   // color are blitted as white. All the remaining pixels are
   // blitted as black.

   HDC hdcSrc, hdcDst;

   hdcSrc = CreateCompatibleDC(NULL);
   hdcDst = CreateCompatibleDC(NULL);

   // Load the bitmaps into memory DC
   HBITMAP hbmSrcT = (HBITMAP) SelectObject(hdcSrc, hBmpSource);
   HBITMAP hbmDstT = (HBITMAP) SelectObject(hdcDst, hBmpMask);

   // Dynamically get the transparent color
   COLORREF clrTrans;
   if (clrpTransColor == NULL)
   {
      // User did not specify trans color so get it from bmp
      clrTrans = GetPixel(hdcSrc, iTransPixelX, iTransPixelY);
   }
   else
   {
      clrTrans = clrpTransColor;
   }


   // Change the background to trans color
   COLORREF clrSaveBk  = SetBkColor(hdcSrc, clrTrans);

   // This call sets up the mask bitmap.
   BitBlt(hdcDst, 0,0,bm.bmWidth, bm.bmHeight, hdcSrc,0,0,SRCCOPY);

   // Now, we need to paint onto the original image, making
   // sure that the "transparent" area is set to black. What
   // we do is AND the monochrome image onto the color Image
   // first. When blitting from mono to color, the monochrome
   // pixel is first transformed as follows:
   // if  1 (black) it is mapped to the color set by SetTextColor().
   // if  0 (white) is is mapped to the color set by SetBkColor().
   // Only then is the raster operation performed.

   COLORREF clrSaveDstText = SetTextColor(hdcSrc, RGB(255,255,255));
   SetBkColor(hdcSrc, RGB(0,0,0));

   BitBlt(hdcSrc, 0,0,bm.bmWidth, bm.bmHeight, hdcDst,0,0,SRCAND);

   // Clean up by deselecting any objects, and delete the
   // DC's.
   SetTextColor(hdcDst, clrSaveDstText);

   SetBkColor(hdcSrc, clrSaveBk);
   SelectObject(hdcSrc, hbmSrcT);
   SelectObject(hdcDst,hbmDstT);

   DeleteDC(hdcSrc);
   DeleteDC(hdcDst);


   return hBmpMask;
}



