// Winamp general purpose plug-in mini-SDK
// Copyright (C) 1997, Justin Frankel/Nullsoft

#include "stdafx.h"
#include <commctrl.h>

#include "gen.h"
#include "resource.h"
#include "systray.h"
#include "util.h"

// internal function
int init();
void quit();
void config();
void config_write();
void config_read();
void config_getinifn(TCHAR *ini_file);
BOOL CALLBACK ConfigProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);


// Setting LINK Check library
//
// Standard library
//    kernel32.lib user32.lib gdi32.lib winspool.lib comdlg32.lib advapi32.lib ole32.lib oleaut32.lib uuid.lib
//
// Add specific 
//    shell32.lib comctl32.lib version.lib 


// Global declaration
TCHAR szAppName[] = _T("EZ-PlayList");
CSystray *systray;

winampGeneralPurposePlugin plugin =
{
	GPPHDR_VER,
	_T(""),
	init,
    config,
	quit,
};


// Internal  declaration
int systray_enabled =1;	// button in systray
int active_enabled  =1;	// 0:Inactive;  1:Active plugin




/* ************************************************************************ */
//BOOL WINAPI _DllMainCRTStartup(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
BOOL WINAPI DllMain(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
{
	return TRUE;
}

/* ************************************************************************ */
// this is the only exported symbol. returns our main header.
// if you are compiling C++, the extern "C" { is necessary, so we just #ifdef it
extern "C" 
__declspec( dllexport ) winampGeneralPurposePlugin* winampGetGeneralPurposePlugin()
{
	return &plugin;
}


/* ************************************************************************ */
int init()
{
	int nMajor=1, nMinor=1, nBuild=1;


	// add this code if win2000/nt/xp doesn't
	// load us winmain (experimental only)
	// also add comctl32.lib
	INITCOMMONCONTROLSEX iccs;
    iccs.dwICC = ICC_LISTVIEW_CLASSES;
    iccs.dwSize = sizeof(INITCOMMONCONTROLSEX);
	BOOL bRet = InitCommonControlsEx(&iccs);

	// Recupere la version de EZPlaylist
	GetVersionInformation(nMajor, nMinor, nBuild);  

	static TCHAR c[MAX_PATH];
	TCHAR filename[MAX_PATH],*p;
	GetModuleFileName(plugin.hDllInstance,filename,sizeof(filename));
	p = filename+lstrlen(filename);
	while (p >= filename && *p != '\\') p--;
	_stprintf((plugin.description=c), _T("%s Plug-in v%d.%d (%s)"), 
		szAppName,
		nMajor, nMinor, 
		p+1);


	config_read();

	if (active_enabled==1)
	{
		systray = (CSystray*)new CSystray();
	}



	return 0;
}

/* ************************************************************************ */
void quit()
{
	config_write();
	
	if (active_enabled==1)
		delete systray; 

}


/* ************************************************************************ */
void config_read()
{
	TCHAR ini_file[MAX_PATH];

	config_getinifn(ini_file);
	
	active_enabled  = GetPrivateProfileInt(szAppName, _T("ActiveEnabled") ,active_enabled, ini_file);
	systray_enabled = GetPrivateProfileInt(szAppName, _T("SystrayEnabled"),systray_enabled, ini_file);

}

/* ************************************************************************ */
void config_write()
{
	TCHAR ini_file[MAX_PATH];
	TCHAR szBuffer[32];

	config_getinifn(ini_file);
	
	_stprintf(szBuffer,_T("%d"),active_enabled);
	WritePrivateProfileString(szAppName, _T("ActiveEnabled"),szBuffer,ini_file);
	_stprintf(szBuffer,_T("%d"),systray_enabled);
	WritePrivateProfileString(szAppName, _T("SystrayEnabled"),szBuffer,ini_file);
}

/* ************************************************************************ */
void config_getinifn(TCHAR *ini_file)
{	
	TCHAR *p;

	// makes a .ini file in the winamp directory named "plugin.ini"
	GetModuleFileName(plugin.hDllInstance, ini_file, MAX_PATH);
	p = ini_file + _tcslen(ini_file);
	while (p >= ini_file && *p != '\\') p--;
	if (++p >= ini_file) *p = 0;
	_tcscat(ini_file, _T("plugin.ini"));
}

/* ************************************************************************ */
void config()
{
	DialogBox(plugin.hDllInstance,	MAKEINTRESOURCE(IDD_DLGCONFIGURE),
		plugin.hwndParent,	(DLGPROC)ConfigProc);
}


/* ************************************************************************ */
BOOL CALLBACK ConfigProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		CheckDlgButton(hwndDlg,IDC_CHECK_ENABLE_EZPLAYLIST, active_enabled==1 ? BST_CHECKED:BST_UNCHECKED);
		return FALSE;

	case WM_CTLCOLORSTATIC:
		if ((HWND) lParam ==  GetDlgItem(hwndDlg,IDC_CONFIG_NOTE))
		{
			SetBkMode( (HDC) wParam,TRANSPARENT);
			SetTextColor((HDC) wParam, RGB(255,0,0));
			return (HRESULT)GetSysColorBrush(COLOR_BTNFACE); // normal background
		}
		return NULL; //don't change colour of any other static control

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
			case IDOK:
				active_enabled = (IsDlgButtonChecked(hwndDlg, IDC_CHECK_ENABLE_EZPLAYLIST) == BST_CHECKED ? 1 : 0);
			case IDCANCEL:
				EndDialog(hwndDlg,0);
				return FALSE;
		}
	}
	return FALSE;
}



